/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.comms;

import java.io.*;
import java.net.*;

import fuegocore.notify.core.client.*;

/**
 * Server for debugging TCP-based communication
 */

public class DebugServer extends Thread {
    int state = 0;
    int port = 9000;
    int INIT = 0;
    int ACTIVE = 1;
    int SERVER_TIMEOUT = 1000;

    public DebugServer() {
    }

    public static void main(String[] args) {
        DebugServer debugServer1 = new DebugServer();
    }

    public void run() {
        ServerSocket listen_socket = null;

        // Init state: try to get the listener up.
        while (state == INIT) {
            try {
                listen_socket = new ServerSocket(port);

                System.out.println("Fuego Message transport started on port:" +
                                   port);

                state = ACTIVE;
            } catch (Exception e) {
                System.err.println("Failed to create server socket on port " +
                                   port +
                                   ". Trying..");
                listen_socket = null;
                state = INIT;
            }

            System.out.println("Server socket created");

            // Wait time between retrys
            try {
                if (state == INIT) {
                    sleep(SERVER_TIMEOUT);
                }
            } catch (Exception e) {
            }
        }
        // We have acquired the socket and listen to it.
        while (state == ACTIVE && listen_socket != null) {
            Socket socket;
            try {
                socket = listen_socket.accept();
                SocketHandler sh = new SocketHandler(socket);
                sh.start();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}


class SocketHandler extends Thread {
    Socket socket;
    public SocketHandler(Socket ref) {
        socket = ref;
    }

    public void run() {
        try {
            InputStream in =
                    socket.getInputStream();
             MicroRequest mq = new MicroRequest();
            mq.resurrect(in, new Object());
            OutputStream out = socket.getOutputStream();
            mq.persist(out, new Object());
            socket.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
