/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.comms;

import java.io.*;
import java.util.Set;
import java.util.HashSet;
import javax.servlet.*;
import javax.servlet.http.*; // Jetty

import org.mortbay.http.*;
import org.mortbay.http.handler.*;
import org.mortbay.jetty.servlet.*;
import fuegocore.notify.core.client.*;
import fuegocore.notify.core.server.handlers.*;

/**
 * Servlet for managing HTTP-based communication
 */

public class HTTPServlet extends HttpServlet {
    private static Set usedIds = new HashSet();
    public static int PORT = 8083;

    public HTTPServlet() {
    }

    public static int getPort() {
        return PORT;
    }

    public static void startHttpServlet() {
        int listenPort = PORT;

        try {
            HttpServer server = new HttpServer();
            HttpContext context = server.getContext("/");
            server.addListener(":" + PORT);
            ServletHandler handler = new ServletHandler();

            String home = System.getProperty("rn-home", ".");
            context.setResourceBase(home + "/admin");
            context.addHandler(new ResourceHandler());

            handler.addServlet("notify", "/notify/*",
                               "fuegocore.notify.comms.HTTPServlet");
            context.addHandler(handler);

            // Start the http server
            System.out.print("Starting HTTP servlet:");
            server.start();
            System.out.println("Started");

            System.out.println("Starting debug server");
            DebugServer db = new DebugServer();
            db.start();
            System.out.println("Started");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // Management interface
    public void doGet(HttpServletRequest request,
                      HttpServletResponse response) throws IOException,
            ServletException {

        String id = (String) request.getParameter("id");
        String id2 = (String) request.getParameter("id2");
        String id3 = (String) request.getParameter("id3");
        String id4 = (String) request.getParameter("id4");
        String debug = (String) request.getParameter("debug");

        String resp = fuegocore.notify.core.server.ui.ManagementUI.getHTMLPage(
                id,
                id2,
                id3, id4, debug);

        response.setContentType("text/html");
        ServletOutputStream out = response.getOutputStream();
        out.println(resp);
        out.close();

    }

    public void doPost(HttpServletRequest request,
                       HttpServletResponse response) throws IOException,
            ServletException {

        ServletInputStream in =
                request.getInputStream();
        boolean idNeeded = false;
        String connId = request.getHeader("Conn-Id");
        if (connId == null) {
            int val = (new Object()).hashCode();
            if (val < 0) {
                val = -val;
            }
            do {
                connId = String.valueOf(val);
                val += 1;
            } while (usedIds.contains(connId));
            usedIds.add(connId);
            idNeeded = true;
        }
        MicroRequest mq = new MicroRequest();
        mq.resurrect(in, connId);

        // mq closes the input stream
        System.out.println("Received from: " +
                           request.getRemoteAddr());

        LightweightHandler lw = new LightweightHandler();
        Object data = lw.processRequest(mq, request.getRemoteAddr());

        response.setContentType("application/octet-stream");
        if (idNeeded) {
            response.addHeader("Conn-Id", connId);
        }
        OutputStream out = response.getOutputStream();
        if (data instanceof String) {
            DataOutputStream dat = new DataOutputStream(out);
            dat.writeUTF((String) data);
            dat.close();
        } else if (data instanceof MicroRequest) {
            MicroRequest mr = (MicroRequest) data;
            mr.persist(out, connId);
        }
    }

}
