/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.comms;

import java.net.*;

import fuegocore.notify.profile.*;
import fuegocore.notify.util.*;

/*
 * Ping for RTT, and heartbeat. Deprecated.
 */
public class PingChannel implements Runnable {
    static final int port = CommProfile.getPingPort(); // port
    static final int maxTime = 5000; // ms
    static final int pingInterval = 100; // ms
    long cost = 0; // transmitted bytes
    boolean alive = true;

    DatagramSocket socket;
    InetAddress originAddress;
    long sendTime;
    long timeMeasured = -1;
    Thread pingThread;
    byte pNumber = 0;
    int missed = 0;
    int mark = 0;
    int UDPHEADER = 28; // 20 bytes for normal IP header, 8 bytes for UDP header.

    public long getLatency() {
        return timeMeasured;
    }

    public PingChannel(InetAddress ip) {
        originAddress = ip;
        missed = 0;
        mark = 0;
    }

    public long getCost() {
        return cost;
    }

    public int getMissedIncrement() {
        int i = missed - mark;
        mark = missed;
        return i;
    }

    public double getMissedPercentage() {
        int i = missed - mark;
        mark = missed;
        return i / missed;
    }

    public long executePing() {
        byte[] msg = new byte[1];
        msg[0] = ++pNumber;
        timeMeasured = -1;

        if (socket == null) {
            try {
                socket = new DatagramSocket();
            } catch (Exception e) {
                return (0);
            }
        }

        if (pingThread == null) {
            pingThread = new Thread(this);
            pingThread.start();
        }

        DatagramPacket packet = new DatagramPacket(msg, msg.length,
                originAddress,
                port);
        sendTime = System.currentTimeMillis();
        long timeLimit = sendTime + maxTime;
        cost += UDPHEADER + packet.getLength();

        try {
            socket.send(packet);
            while (System.currentTimeMillis() < timeLimit) {
                Thread.sleep(pingInterval);
                if (timeMeasured != -1) {
                    return (timeMeasured);
                }
            }
        } catch (Exception e) {}
        // Timeout or error occurred
        missed++;
        return (timeMeasured);
    }

    public void run() {
        byte[] buf = new byte[1];
        DatagramPacket reply = new DatagramPacket(buf, buf.length);

        try {
            while (alive) {
                socket.receive(reply);
                cost += UDPHEADER + reply.getLength();
                if (buf[0] == pNumber) {
                    timeMeasured = System.currentTimeMillis() - sendTime;
                    pingThread = null;
                    return;
                }
            }
        } catch (Exception e) {
            pingThread = null;
            return;
        }
        pingThread = null;
    }

    public void stop() {
        if (pingThread != null) {
            alive = false;
            pingThread = null;
        }
        socket.close();
        socket = null;
    }

    // Tester method for PingChannel
    public static void main(String[] args) {
        PingChannel pc = null;
        System.out.println("Getting object");

        try {
            pc = new PingChannel(InetAddress.getByName(args[0]));
        } catch (Exception e) {
            e.printStackTrace();
        }

        ValueHistory cm = new ValueHistory(10);
        System.out.println("Start ping");

        while (true) {
            long echoTime = pc.executePing(); // conduct actual test
            cm.add(echoTime);
            System.out.println("Echotime: " + echoTime + " val: " +
                               cm.getValue(0.8) +
                               " cost:" + pc.getCost());
            try {
                Thread.sleep(20);
            } catch (Exception e) {
            }

        }

    }
}
