/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.comms;

/**
 * This class manages transport queues, mainly timeouts
 * and deadlines.
 * Actually calls the manageQueue method in MessageTransport
 * when deadlines occur. This class provides the timer functionality.
 */

public class QueueManager extends Thread {
    boolean alive = true;
    long wakeup = 0;
    boolean update = false;
    IQueueManager _parent;

    public QueueManager(IQueueManager mt) {
        _parent = mt;
    }

    /** Sets a new deadline for queue manager
     * @param w the new deadline
     */
    public synchronized void newDeadline(long w) {
        wakeup = w - System.currentTimeMillis();
        if (wakeup < 0) {
            wakeup = 0;
        }
        notify(); // wakeup the thread and set the new deadline
    }

    /** Shutdown queue manager */
    public synchronized void shutdown() {
        alive = false;
        notify();
    }

    /** Start queue manager */
    public synchronized void kick() {
        if (alive) {
            return;
        }
        alive = true;
        start();
    }

    public void run() {
        fuegocore.notify.util.Log.write("QueueManager activated");
        while (alive) {
            update = false;
            try {
                synchronized (this) {
                    wait(wakeup);
                    fuegocore.notify.util.Log.write("QM: woke up");
                }
            } catch (Exception e) {
                fuegocore.notify.util.Log.write("Queuemanager woke up: " + e);
                update = true;
            }
            if (_parent != null) {
                fuegocore.notify.util.Log.write("QM calling: manageQueue");
                wakeup = _parent.manageQueue() - System.currentTimeMillis();
                if (wakeup < 0) {
                    wakeup = 0;
                }
            }
            update = false;
        }
    }
}
