/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.comms;

import java.util.*;

import fuegocore.message.*;
import fuegocore.message.protocol.*;
import fuegocore.message.service.*;
import fuegocore.notify.core.*;
import fuegocore.notify.core.serialization.EventCodecFactory;
import fuegocore.util.xas.CodecIndustry;

/** This thread is responsible for sending messages from a
 * session queue.
 * */
public class ServerSenderThread extends Thread {
    ServerChannel _mtp;
    boolean alive = true;
    boolean kick = false;
    int transmission_count = 0;

    public ServerSenderThread(ServerChannel mtp) {
        transmission_count = 0;
        _mtp = mtp;
        alive = true;
        fuegocore.notify.util.Log.writeTransport("SenderThread active");
    }

    public void shutdown() {
        alive = false;
    }

    public void run() {
        byte[] record = null;
        kick = true;
        Vector outbound = _mtp.getOutbound();
        boolean f = false;
        Notification msg = null;

        while (alive) {
            try {
                fuegocore.notify.util.Log.writeTransport("Sender waking");

                if (outbound.size() == 0) {
                    synchronized (outbound) {
                        outbound.wait();
                    }
                }

                fuegocore.notify.util.Log.writeTransport("New message");

                f = false;
                msg = null;
                boolean tflag = false;

                while (outbound.size() > 0) {

                    if (!alive) {
                        break;
                    }

                    fuegocore.notify.util.Log.writeTransport("Getting message");
                    synchronized (outbound) {
                        msg = (Notification) outbound.remove(0);
                    }

                    if (msg != null) {
                        MessageTransportService mts = MessageTransportService.
                                init();
                        CodecIndustry.registerContentFactory(new
                                EventCodecFactory());
                        MessageFactory mf = mts.getFactory();
                        fuegocore.message.service.Message m2 = mf.createMessage();

                        if (msg == null) {
                            continue;
                        }

                        try {
                            m2.putData("data", msg);
                            MessageService ms = mts.getService();

                            Url url = new Url(Url.Protocol.parse(fuegocore.
                                    notify.profile.
                                    UserProfile.DEF_MSG_PROTOCOL), "message",
                                              _mtp.getOrigin());
                            m2.setDestination(url);
                            ms.sendMessage(m2);
                        } catch (Exception e) {
                            e.printStackTrace();
                            // Message send failed, put message back to queue
                            tflag = true;
                            synchronized (outbound) {
                                outbound.add(msg);
                            }
                            Thread.sleep(_mtp.RETRANSMISSION_TIMER); // retransmission timer
                            transmission_count++;
                            if (transmission_count > _mtp.MAX_RETRY) {
                                _mtp.connectionFailure();
                            }
                        }
                        if (!tflag) {
                            transmission_count = 0;
                        }
                        tflag = false;
                    }
                } // end while
            } catch (Exception e) { // end try
                fuegocore.notify.util.Log.writeTransport(
                        "SenderThread Exception: " +
                        e);
                //break;
//        }
            } // end synchronize
        } // end while
    } // end method
} // end class
