/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.channel;

/**
 * Event channel parameters
 */

public class EventChannelParams {
    public int maxSubscribers = 100;
    public int eventHistorySize = 1000;
    boolean orderdingAndLogging = true;

    // expiry date

    public EventChannelParams() {}

    /**
     * Constructor
     * @param a the max number of subscribers
     * @param b the max event history size
     */
    public EventChannelParams(int a, int b) {
        maxSubscribers = a;
        eventHistorySize = b;
    }

    /**
     * Set ordering and logging feature
     * @param b boolean
     */
    public void setOrderAndLogging(boolean b) {
        orderdingAndLogging = b;
    }

    /**
     * Get the feature state
     * @return boolean
     */
    public boolean getOrderAndLogging() {
        return orderdingAndLogging;
    }

    /**
     * Get maximum number of subscribers
     * @return int
     */
    public int getMaxSubscribers() {
        return maxSubscribers;
    }

    /**
     * Set the maximum number of subscribers
     * @param a int
     */
    public void setMaxSubscribers(int a) {
        maxSubscribers = a;
    }

    /**
     * Set the event history size
     * @param a int
     */
    public void setEventHistorySize(int a) {
        eventHistorySize = a;
    }

    /**
     * Get event history size
     * @return int
     */
    public int getEventHistorySize() {
        return eventHistorySize;
    }

    /**
     * Serialize to string format
     * @return String
     */
    public String toString() {
        String ret = "";
        ret += "<maxsubs>" + maxSubscribers + "</maxsubs>";
        ret += "<historysize>" + eventHistorySize + "</historysize>";
        ret += "<orderandlog>" + orderdingAndLogging + "</orderandlog>";
        return ret;
    }

    /**
     * Parse from string representation
     * @param s String
     */
    public void parse(String s) {
        String s2 = fuegocore.notify.util.Auxil.getTag(s, "maxsubs");
        try {
            maxSubscribers = Integer.parseInt(s2);
        } catch (Exception e) {}
        s2 = fuegocore.notify.util.Auxil.getTag(s, "historysize");
        try {
            eventHistorySize = Integer.parseInt(s2);
        } catch (Exception e) {}
        s2 = fuegocore.notify.util.Auxil.getTag(s, "orderandlog");
        try {
            orderdingAndLogging = Boolean.getBoolean(s2);
        } catch (Exception e) {}
    }

}
