package fuegocore.notify.core.client;

import java.io.*;
import java.util.*;
import java.net.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.client.*;
import fuegocore.notify.core.serialization.*;

/**
 *  Simple message protocol
 */
public class SimpleMessageProtocol
    extends Thread {
  String host = "";
  int port = 80;
  boolean alive = true;
  String session = "";

  public String getSessionId() {
    return session;
  }

  public SimpleMessageProtocol(String host, int port, String session) {
    this.host = host;
    this.port = port;
    this.session = session;
  }

  // blocked pull of notifications from the session
  public void run() {
    EventConnection ec = EventConnection.get();

    MicroRequest mq = new MicroRequest();
    mq.deviceId = ec.getName();
    mq.op = "pullSessionSubs";
    mq.id = session;
    mq.payload = "";
    mq.lastserver = ec.getLastServer();
    mq.flag = 1;
    mq.count = 100;

    while (alive) {
      try {
        String ret = call(host, port, mq);
        // System.out.println("-- Got: " + ret);
        if (ret != null && ret.indexOf("failure") == -1) {
          Vector v = EventSerializationService.get("Notification").demultiplex(ret);
          if (v != null && v.size() > 0) {
            for (int i = 0; i < v.size(); i++) {
              Notification n = (Notification) v.elementAt(i);
              ec.route(n);
            }
          }
        }
      }
      catch (Exception e) {
        System.out.println("Got exception in run()");
        System.out.println("Doing retry");
        e.printStackTrace();
      }
    }
  }

  public void close() {
    alive = false;
  }

  public static String call(String op, String sid, String payload, boolean flag,
                            int count) {
    MicroRequest mq = new MicroRequest();
    mq.deviceId = EventConnection.get().getName();
    mq.op = op;
    mq.id = sid;
    mq.payload = payload;
    mq.lastserver = EventConnection.get().getLastServer();
    if (flag) {
      mq.flag = 1;
    }
    else {
      mq.flag = 0;
    }
    mq.count = count;
    return call(EventConnection.get().getHost(), EventConnection.get().getPort(),
                mq);
  }

  public static String call(String host, int port, MicroRequest mq) {
    String ret = null;
    HttpURLConnection http = null;

    http = null;
    InputStream iStrm = null;

    try {
      byte[] data = mq.persist();

      URL url = new URL("http://" + host + ":" + port +
                                             "/notify/");

      System.out.println("Trying URL:" + url);
      http = (HttpURLConnection)url.openConnection();
      http.setDoInput(true);
      http.setDoOutput(true);
      http.setRequestMethod("POST");
      http.setDefaultAllowUserInteraction(false);
      http.setRequestProperty("Content-type","application/octet-stream");
      http.setAllowUserInteraction(true);

      http.connect();

      OutputStream os = http.getOutputStream();
      os.write(data);
      os.flush();
      os.close();

      if (http.getResponseCode() == HttpURLConnection.HTTP_OK) {
        iStrm = http.getInputStream();
        DataInputStream din =
            new DataInputStream(
            iStrm);

        ret = din.readUTF();
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    finally {
      try {
        if (iStrm != null) {
          iStrm.close();
        }
        if (http != null) {
          http.disconnect();
        }
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }
    return ret;
  }

}
