/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.filter;

import java.util.*;

/**
 * This class represents the basic interfaces for all coverable, mergeable,
 * and matchable objects.
 */

public interface IFBase {
    /** Matches the object with the given object
     * @param et the EventTriplet object to match, may be null
     * @param noti the TripletTable, may not be null
     * @return boolean true if the match succeeded
     */
    public boolean match(Object et, Object noti);

    /** Merges the object with the given object
     * @param obj the object to be merged
     * @return boolean true if the merge succeeded
     */
    public boolean merge(Object obj);


    /** Merges the current object with the given object
     * @param obj the object to be merged with the current object
     * @return Object the merged result
     */
    public Object mergeObject(Object obj);


    /**
     * Covering test: is the given object covered
     * @param obj the object that covers
     * @return true if the current object was covered
     */
    public boolean isCovered(Object obj);

    /**
     * Covering test
     * @param obj the object that is covered
     * @return true if the current object covers the given object
     */
    public boolean covers(Object obj);

    /**
     * Overlapping test
     * @param obj the input object
     * @return true if the current object overlaps with the given object
     */
    public boolean overlaps(Object obj);

    /**
     * Get id for hash lookups.
     * @return String the identifier
     */
    public String getCoverId();

    /**
     * Get colors/interfaces
     * @return Vector colors/interfaces
     */
    public Vector getTid();

    /**
     * Add an entry to the list of interfaces
     * @param obj Object
     */
    public void addTid(Object obj);

    /**
     * Set the depth of this object in the poset tree.
     * @param i the depth to set
     */
    public void setDepth(int i);

    /**
     * Returns the depth of this object in the poset tree.
     * @return the depth as int
     */
    public int getDepth();

    /** Returns the cost of the filter (generality).
     *  Higher value denotes more general filters.
     * @return int the cost
     */
    public int getCost();

    /**
     * Returns the size of the object
     * If filter the # disjuncts, if disjunct the # predicates
     * @return the size
     */
    public int getSize();

    public boolean noSelectivity();
}
