/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.filter.merge;

import java.util.*;

import fuegocore.notify.core.*;

/**
 * Interface for merging algorithms
 */

public interface IMergingAlgorithm {
    /**
     * If set to false, filter covering is not checked after merging
     * @param check boolean
     */
    public void setCoverCheck(boolean check);

    /** Returns the set of merged templates for a single template
     *  We count the number of identical fields.  For perfect filtering,
     *  the mergeable filters must have the same structure.
     *
     *  Updates the given set of templates.
     *  @param t the set of templates
     *  @param tp the template to be incrementally merged
     *  @param merge boolean variable whether to actually do the merge or not
     *  @return index that was changed in the given vector
     */
    public int mergeSet(Vector t, Template tp, boolean merge);

    /** Returns the set of merged templates for a vector
     *  of templates. We count the number of identical fields.  For perfect filtering,
     *  the mergeable filters must have the same structure.
     * @param t Vector of templates to be perfectly merged
     * @return Vector containing the merge result
     */
    public Vector mergeSet(Vector t);

}
