package fuegocore.notify.core.filter.poset;

import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;
import fuegocore.notify.core.filter.merge.*;

/**
 * This class tracks merged nodes.
 */
public class MergeTracker {
  boolean debug = false;
  int comparisons = 0;

  /** Status code for successful merge */
  protected int MERGE_OK = 0;

  /** Status code for a failed merge */
  protected int MERGE_FAIL = 1;

  /** Status code for an empty merge set */
  protected int MERGE_EMPTY = 2;

  /** Hashtable for storing information about merges (Nodes).
   * This structure has only one entry for each node that may be the result
   * of multiple merged templates */
  Hashtable mergeTracker = new Hashtable();

  /** Hashtable for storing information about merged templates.
   * This structures has a record for each merged template and thus differs
   * from the mergeTracker */
  Hashtable mergedTemplates = new Hashtable();

  /** This structure stores templates that are covered by a merged
   * node of the same color. This is needed for maintaining equivalence
   * with the colored forest.
   */
  Hashtable coveredTemplatesDueToMerging = new Hashtable();
  Hashtable coveredTemplatesById = new Hashtable();
  Hashtable templates = null;

  /**
   * Test the structures
   * @return boolean
   */
  public boolean test() {
    int mergedTemplatesCount = 0;
    boolean ret = true;

    System.out.println("TEST MERGED FOREST ******************************");

    Enumeration k = mergeTracker.elements();
    // test all merged nodes
    for (Enumeration e = mergeTracker.keys(); e.hasMoreElements(); ) {
      Node x = (Node) e.nextElement();
      Vector v = (Vector) k.nextElement();

      if (templates.get(x) != null) {
        System.out.println("Merged node was found in templates: " + x);
        ret = false;
      }

      if (x.getData() == null) {
        System.out.println("x.getData() was null: " + x);
        ret = false;
      }
      else if (x.getData().getTid() == null || x.getData().getTid().size() < 1) {
        System.out.println("x.getData().getTid() was null or zero:" + x);
        System.out.println("Data: " + x.getData().toString());
        IFBase f0 = (IFBase) v.firstElement();
        System.out.println("Missing color:" + f0.getTid().firstElement());
        ret = false;
      }
      else if (x.getData().getTid().size() > 1) {
        System.out.println("x.getData().getTid() > 1:" + x);
        System.out.println("Data: " + x.getData().toString());
        System.out.println("Tid: " + x.getData().getTid());
        ret = false;
      }

      // test merged set
      for (int i = 0; i < v.size(); i++) {
        IFBase f = (IFBase) v.elementAt(i);

        if (f.getTid() == null || f.getTid().size() < 1) {
          System.out.println("IFBase color vector was null or zero sized");
          ret = false;
          break;
        }

        Node x2 = (Node) mergedTemplates.get(f.getCoverId() + "_" +
                                             f.getTid().firstElement());
        if (x2 != x) {
          System.out.println("Merged template not part of associated vector");
          System.out.println("Merged node:" + x);
          System.out.println("Reported incorrect node: " + x2);
          ret = false;
          break;
        }
        // test lookup structure
        mergedTemplatesCount++;
      }

      // test covered templates
      Vector mb = (Vector) coveredTemplatesDueToMerging.get(x);
      if (mb != null) {
        for (int i = 0; i < mb.size(); i++) {
          IFBase f = (IFBase) mb.elementAt(i);
          if (f.getTid() == null || f.getTid().size() < 1) {
            System.out.println(
                "covered IFBase color vector was null or zero sized");
            ret = false;
            break;
          }
          ValuePairHolder h = (ValuePairHolder) coveredTemplatesById.get(f.
              getCoverId() + "_" +
              f.getTid().firstElement());
          IFBase f2 = (IFBase) h.getFirst();
          Node x2 = (Node) h.getSecond();
          if (f != f2) {
            System.out.println(
                "Covered template set element could not be found");
            ret = false;
          }
          if (f2.getTid() == null || f2.getTid().size() < 1) {
            System.out.println("IFBase color vector was null or zero sized " +
                               "(coveredTemplatesById)");
            ret = false;
            break;
          }

          if (templates.get(x2) != null) {
            System.out.println(
                "Merged node was found in templates (coveredTemplatesById):" +
                x2);
            ret = false;
          }
        }
      }
    }
    if (ret && mergedTemplatesCount != mergedTemplates.size()) {
      System.out.println("MergeTracker test: mergedTemplates count error");
      System.out.println("Counter: " + mergedTemplatesCount);
      System.out.println("mergedTemplates.size(): " + mergedTemplates.size());
      ret = false;
    }
    return ret;
  }

  /**
   * Test the structures
   * @return String
   */
  public String runTests() {
    int mergedTemplatesCount = 0;
    boolean ret = true;
    String str = "";

    str = "MergeTracker tests\n";
    str = "TEST: merged nodes, references, null colors, merged components, size invariants\n";

    Enumeration k = mergeTracker.elements();
    // test all merged nodes
    for (Enumeration e = mergeTracker.keys(); e.hasMoreElements(); ) {
      Node x = (Node) e.nextElement();
      Vector v = (Vector) k.nextElement();

      if (templates.get(x) != null) {
        str += "Merged node was found in templates: " + x + "\n";
        ret = false;
      }

      if (x.getData() == null) {
        str += "x.getData() was null: " + x + "\n";
        ret = false;
      }
      else if (x.getData().getTid() == null || x.getData().getTid().size() < 1) {
        str += "x.getData().getTid() was null or zero:" + x + "\n";
        str += "Data: " + x.getData().toString() + "\n";
        IFBase f0 = (IFBase) v.firstElement();
        str += "Missing color:" + f0.getTid().firstElement() + "\n";
        ret = false;
      }
      else if (x.getData().getTid().size() > 1) {
        str += "x.getData().getTid() > 1:" + x + "\n";
        str += "Data: " + x.getData().toString() + "\n";
        str += "Tid: " + x.getData().getTid() + "\n";
        ret = false;
      }

      // test merged set
      for (int i = 0; i < v.size(); i++) {
        IFBase f = (IFBase) v.elementAt(i);

        if (f.getTid() == null || f.getTid().size() < 1) {
          str += "IFBase color vector was null or zero sized\n";
          ret = false;
          break;
        }

        Node x2 = (Node) mergedTemplates.get(f.getCoverId() + "_" +
                                             f.getTid().firstElement());
        if (x2 != x) {
          str += "Merged template not part of associated vector\n";
          str += "Merged node:" + x + "\n";
          str += "Reported incorrect node: " + x2 + "\n";
          ret = false;
          break;
        }
        // test lookup structure
        mergedTemplatesCount++;
      }

      // test covered templates
      Vector mb = (Vector) coveredTemplatesDueToMerging.get(x);
      if (mb != null) {
        for (int i = 0; i < mb.size(); i++) {
          IFBase f = (IFBase) mb.elementAt(i);
          if (f.getTid() == null || f.getTid().size() < 1) {
            str +=
                "covered IFBase color vector was null or zero sized\n";
            ret = false;
            break;
          }
          ValuePairHolder h = (ValuePairHolder) coveredTemplatesById.get(f.
              getCoverId() + "_" +
              f.getTid().firstElement());
          IFBase f2 = (IFBase) h.getFirst();
          Node x2 = (Node) h.getSecond();
          if (f != f2) {
            str +=
                "Covered template set element could not be found\n";
            ret = false;
          }
          if (f2.getTid() == null || f2.getTid().size() < 1) {
            str += "IFBase color vector was null or zero sized " +
                "(coveredTemplatesById)\n";
            ret = false;
            break;
          }

          if (templates.get(x2) != null) {
            str +=
                "Merged node was found in templates (coveredTemplatesById):" +
                x2 + "\n";
            ret = false;
          }
        }
      }
    }
    if (ret && mergedTemplatesCount != mergedTemplates.size()) {
      str += "MergeTracker test: mergedTemplates count error\n";
      str += "Counter: " + mergedTemplatesCount + "\n";
      str += "mergedTemplates.size(): " + mergedTemplates.size() + "\n";
      ret = false;
    }

    if (ret) {
      str += "RESULT: merge tracker ok\n";
    }
    else {
      str += "RESULT: merger tracker failed\n";
    }

    return str;
  }

  /**
   * Get covering ops counter
   * @return int
   */
  public int getOps() {
    return comparisons;
  }

  /**
   * Reset covering ops counter
   */
  public void resetOps() {
    comparisons = 0;
  }

  /**
   * Set the templates hashtable
   * @param t Hashtable
   */
  public void setTemplates(Hashtable t) {
    templates = t;
  }

  /**
   * Constructor
   * @param t Hashtable the templates hashtable
   */
  public MergeTracker(Hashtable t) {
    setTemplates(t);
  }

  /**
   * Checks that the given filter is not covered by any filter in a merger
   * @param x Node the merged node
   * @param z IFBase
   * @return boolean
   */
  public boolean coveredByMergerPart(Node x, IFBase z) {
    Vector v = this.getMergedComponents(x);
    Vector removeList = new Vector();

    for (int i = 0; i < v.size(); i++) {
      IFBase f = (IFBase) v.elementAt(i);
      if (f.toString().equals(z.toString())) {
        System.out.println(
            "Fatal error: coveredByMergerpart, identical filters");
        System.out.println("Filter:" + z);
        System.out.println("Node:" + x);
        System.out.println("Should have been detected earlier");

        if (f == z) {
          System.out.println("Object was the same");
        }

        Node x2 = (Node) mergedTemplates.get(z.getCoverId() + "_" +
                                             z.getTid().firstElement());
        if (x2 == x) {
          System.out.println("Filter was stored in mergedTemplates. Error!");
        }
        else {
          System.out.println(
              "Filter was not stored in mergedTemplates. Error!");
        }
        System.exit(0);
      }

      // We use type-based optimization
      if ( ( (Template) f).getType().equals( ( (Template) z).getType())) {
        comparisons++;
        if (f.covers(z)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Removes all nodes from the covered vector that are covered by the given
   * filter
   * @param v Vector
   * @param z IFBase
   * @param color Object
   * @return boolean
   */
  public boolean coversMergerPart(Vector v, IFBase z, Object color) {
    Vector removeList = new Vector();

    for (int i = 0; i < v.size(); i++) {
      IFBase f = (IFBase) v.elementAt(i);
      comparisons++;
      // We use type-based optimization
      if ( ( (Template) f).getType().equals( ( (Template) z).getType())) {
        comparisons++;
        if (z.covers(f)) {
          removeList.add(f);
          coveredTemplatesById.remove(f.getCoverId() + "_" + color);
        }
      }
    }

    v.removeAll(removeList);

    if (debug) {
      System.out.println("coversMergedPart: removed " + removeList.size());
    }

    return true;
  }

  /**
   * Add a template that is covered by a merged node
   * @param x Node
   * @param z IFBase
   * @return boolean
   */
  protected boolean addCoveredTemplate(Node x, IFBase z) {
    Object color = (Object) x.getData().getTid().firstElement();

    if (z == x.getData()) {
      System.out.println("addCoveredTemplate: fatal error: x == z");
      System.exit(0);
    }

    // Check that the input filter is not covered by any individual
    // node in the merger
    // Note that this could be optimized
    if (coveredByMergerPart(x, z)) {
      return true;
    }

    Vector mb = (Vector) coveredTemplatesDueToMerging.get(x);
    if (mb == null) {
      mb = new Vector();
      coveredTemplatesDueToMerging.put(x, mb);
      return false;
    }

    // remove any filters covered by the new filter
    coversMergerPart(mb, z, color);

    mb.add(z);

    ValuePairHolder pair = new ValuePairHolder(z, x);

    if (x == null || x.getData().getTid() == null ||
        x.getData().getTid().size() == 0) {
      System.out.println("MergeTracker fatal error: node data missing in " +
                         " addCoveredTemplate");
    }

    coveredTemplatesById.put(z.getCoverId() + "_" + color, pair);
    return true;
  }

  /**
   * Remove a covered template
   * @param x Node
   * @param z IFBase
   * @param color Object
   * @return boolean
   */
  protected boolean removeCoveredTemplate(Node x, IFBase z, Object color) {

    if (debug) {
      System.out.println("removeCoveredTemplate: Removing covered template: " +
                         x);
    }

    if (x == null) {
      System.out.println("removeCoveredTemplate: node missing information");
      return false;
    }

    Vector mb = (Vector) coveredTemplatesDueToMerging.get(x);
    if (mb == null) {
      System.out.println("Could not find covered templates for node: " + x);
      return false;
    }

    boolean b = removeSet(mb, z);

    if (!b) {
      System.out.println("Could not remove covered template");
      return false;
    }

    if (mb.isEmpty()) {
      Vector v = (Vector) coveredTemplatesDueToMerging.remove(x);
      if (v != null) {
        v = null;
      }
    }

    coveredTemplatesById.remove(z.getCoverId() + "_" + color);

    if (x.getData().getTid() == null || x.getData().getTid().size() == 0) {
      System.out.println("Fatal error in MergeTracker: x.getData().getTid() " +
                         " was null or zero");
      return false;
    }
    Object color2 = (Object) x.getData().getTid().firstElement();

    if (!color.equals(color2)) {
      System.out.println("Fatal error in MergeTracker: color mismatch: " +
                         "[" + color + "]" + " != [" + color2 + "]");
      return false;
    }
    return b;
  }

  /**
   * Returns a vector of objects that were added and were covered by the node,
   * but are no longer covered. If the set size is one, the remaining filter
   * is returned
   * @param x Node the node
   * @param z IFBase the covering object to be used in selection
   * @return Vector the non-covered set
   */
  protected Vector getNonCoveredTemplates(Node x, IFBase z) {
    Vector ret = new Vector();

    Vector mb = (Vector) coveredTemplatesDueToMerging.get(x);
    if (mb == null) {
      return null;
    }

    for (int i = 0; i < mb.size(); i++) {
      IFBase ifb = (IFBase) mb.elementAt(i);
      comparisons++;
      if (!z.covers(ifb)) {
        ret.add(ifb);
      }
    }
    return ret;
  }

  /**
   * Returns true if the node was merged
   * @param x Node
   * @return boolean
   */
  protected boolean isMerged(Node x) {
    // the hash signature must contain the color since merged
    // nodes have one-to-one correspondence with colors
    Vector mb = (Vector) mergeTracker.get(x);
    if (mb == null) {
      return false;
    }
    return true;
  }

  /**
   * Remove a merged node. Purges cover set also.
   * @param n Node
   * @return boolean
   */
  protected boolean removeMergedNode(Node n) {
    if (debug) {
      System.out.println("removeMergedNode: removing: " + n);
    }

    if (n == null || n.getData() == null) {
      System.out.println("removeMergedNode: node missing information");
      return false;
    }

    Vector mb = (Vector) mergeTracker.remove(n);
    if (mb == null) {
      System.out.println("removeMergeNode: node not found");
      return false;
    }

    // ensure that all have been removed
    for (int i = 0; i < mb.size(); i++) {
      IFBase b = (IFBase) mb.elementAt(i);
      mergedTemplates.remove(b.getCoverId() + "_" + b.getTid().firstElement());
    }

    // Remove nodes that the removed node covers
    Vector v = (Vector) coveredTemplatesDueToMerging.remove(n);
    if (v != null) {
      for (int i = 0; i < v.size(); i++) {
        IFBase fb = (IFBase) v.elementAt(i);
        String key = fb.getCoverId() + "_" + fb.getTid().firstElement();
        coveredTemplatesById.remove(key);
      }
      v.clear();
    }
    mb.clear();
    return true;
  }

  /**
   * Return merge components
   * @param x Node
   * @return Vector
   */
  protected Vector getMergedComponents(Node x) {
    return (Vector) mergeTracker.get(x);
  }

  /**
   * Merge y to x
   * @param x Node
   * @param y IFBase
   * @return boolean
   */
  protected boolean mergeNodes(Node x, IFBase y) {
    IFBase o = x.getData();

    // For now only templates are mergeable
    if (! (o instanceof Template)) {
      if (debug) {
        System.out.println("mergeNodes: invalid object");
      }
      return false;
    }
    try {
      // the hash signature must contain the color since merged
      // nodes have one-to-one correspondence with colors
      Object color = (Object) x.getData().getTid().firstElement();

      // Ensure that the color is correct, there should be exactly one
      // color
      if (y.getTid() == null) {
        y.addTid(color);
      }
      else if (!y.getTid().contains(color)) {
        y.getTid().add(color);
      }

      Template t = (Template) o;
      Template tc = (Template) t.clone();

      if (debug) {
        System.out.println(" - attempting to merge");
      }

      if (tc.merge(y)) {
        x.setData(tc);

        if (debug) {
          System.out.println(" - merge success");
          System.out.println(" - old filter: " + t.toString());
          System.out.println(" - result:" + tc.toString());
        }

        // Record the templates that were merged
        mergedTemplates.put(y.getCoverId() + "_" + color, x);

        if (templates.get(y.getCoverId()) != null) {
          System.out.println("Fatal: input to merge was contained in templates");
          throw new PosetException();
        }

        Vector mb = (Vector) mergeTracker.get(x);

        if (mb == null) {
          // previously non-merged node
          mb = new Vector();
          mergeTracker.put(x, mb);
        }

        if (mb.size() == 0) { // new merge
          if (mb.contains(t)) {
            System.out.println("mergeNodes: fatal error: mb contained t");
            throw new PosetException();
          }

          mb.add(t); // store original
          if (debug) {
            System.out.println("Added: " + tc.getType());

            // Record the original filter also as merged
          }
          mergedTemplates.put(t.getCoverId() + "_" + color, x);
          // remove from templates
          templates.remove(t.getCoverId());
        }
        else {
          // Delete covered entries from previously merged entries
          Vector mb2 = (Vector) mb.clone();
          for (int i = 0; i < mb2.size(); i++) {
            IFBase ty = (IFBase) mb2.elementAt(i);
            comparisons++;
            if (y.covers(ty)) { // remove any covered templates
              mb.remove(ty);
              mergedTemplates.remove(ty.getCoverId() + "_" + color);
            }
          }
        }

        // we store the other object in any case
        if (mb.contains(y)) {
          System.out.println("mergeNodes: fatal error: mb contained y");
          throw new PosetException();
        }
        mb.add(y);

        // Ensure that the color is correct, there should be exactly one
        // color
        if (tc.getTid() == null) {
          tc.addTid(color);
        }
        else if (!tc.getTid().contains(color)) {
          tc.getTid().add(color);
        }

        return true;
      }
      else if (debug) {
        System.out.println(" - merge failure");
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return false;
  }

  /**
   * Removes y from the Vector v
   * @param v Vector
   * @param y IFBase
   * @return boolean
   */
  protected boolean removeSet(Vector v, IFBase y) {
    if (v == null || y == null) {
      return false;
    }
    Object remove = null;

    for (int i = 0; i < v.size(); i++) {
      IFBase ifb = (IFBase) v.elementAt(i);
      if (ifb.getCoverId().equals(y.getCoverId())) {
        remove = ifb;
        break;
      }
    }
    if (remove != null) {
      v.remove(remove);
      return true;
    }
    return false;
  }

  /**
   * Unmerge y from x
   * @param x Node
   * @param y IFBase
   * @param color Object
   * @return int
   */
  protected int unmergeNodes(Node x, IFBase y, Object color) {
    Vector mb = (Vector) mergeTracker.get(x);
    if (mb == null) {
      System.out.println("unmergeNodes: Missing merge set");
      return MERGE_FAIL;
    }
    if (!removeSet(mb, y)) {
      // merge was not succesful
      System.out.println("unmergeNodes: Could not remove node from merge set");
      System.out.println("merge set size: " + mb.size());
      System.out.println("merge set: " + mb);
      System.out.println("missing object: " + y);
      System.out.println("- - - - -");

      return MERGE_FAIL;
    }

    // Remove node from the set of merged templates
    if (mergedTemplates.remove(y.getCoverId() + "_" + color) == null) {
      System.out.println(
          "MergeTracker: fatal entry not found in mergedTemplates");
      return MERGE_FAIL;
    }

    if (mb.isEmpty()) {
      // The node may be deleted, there are no nodes anymore
      System.out.println("unmergeNodes: node empty");
      return MERGE_EMPTY;
    }
    if (mb.size() == 1) {
      System.out.println("Only one entry remaining");
      // only one remaining node, no need to merge
      IFBase first = (IFBase) mb.firstElement();
      x.data = first; // set merged element
      // we leave the first element there
      // it will be transformed to a single non-merged node
      // Remove node from the set of merged templates
      mergedTemplates.remove(x.data.getCoverId() + "_" + color);

      return MERGE_OK;
    }

    // re-merge the set
    IMergingAlgorithm merge =
        fuegocore.notify.core.filter.merge.MergingService.get("perfect");

    if (debug) {
      for (int i = 0; i < mb.size(); i++) {
        IFBase ff = (IFBase) mb.elementAt(i);
        System.out.println(i + ":" + ff.toString());
      }
    }

    // merge set
    Vector merged = merge.mergeSet(mb);

    if (merged.size() == 0) {
      System.out.println("unmergedNodes: fatal error merge size was zero");
      return MERGE_FAIL;
    }
    else if (merged.size() > 1) {
      System.out.println("unmergedNodes: fatal error merge size greater " +
                         "than zero");
      return MERGE_FAIL;
    }

    IFBase first = (IFBase) merged.firstElement();

    if (debug) {
      System.out.println(" - remerge success");
      System.out.println(" - old filter: " + x.data.toString());
      System.out.println(" - result:" + first.toString());
    }

    x.data = first; // set merged element

    // Ensure that the color is correct, there should be exactly one
    // color
    if (first.getTid() == null) {
      first.addTid(color);
    }
    else if (!first.getTid().contains(color)) {
      first.getTid().add(color);
    }

    return MERGE_OK;
  }

  public Vector getAllNodes() {
    Vector ret = new Vector();
    // Merged nodes
    for (Enumeration e = mergeTracker.keys(); e.hasMoreElements(); ) {
      Object obj = e.nextElement();
      Node n = (Node) obj;
      ret.add(n.getData());
    }
    return ret;
  }

  public Vector getAll() {
    Vector ret = new Vector();
    // Merged nodes
    for (Enumeration e = mergeTracker.keys(); e.hasMoreElements(); ) {
      Object obj = e.nextElement();
      Node n = (Node) obj;
      ret.add(n);
    }
    return ret;
  }

  /**
   * Collapse merged node, called from eliminate-color
   * @param x Node the current node to be collapsed
   * @param start Node the node that started eliminate-color
   */
  public void collapseMergedNode(Node x, Node start) {
    // store a copy of the object for later inclusion
    // Note that we do not remove the color yet!
    // It is needed when the merged node is put back to the
    // structure due to unmerging.

    // we have to add all the components of the merged node
    // to the covered template list
    Vector v = (Vector) ( (Vector) (mergeTracker.get(x))).clone();

    // add normal filters covered by the merge
    Vector mb = (Vector) coveredTemplatesDueToMerging.remove(x);

    if (mb != null) { // may well be null
      // We remove all templates covered due to merging associated
      // with x and associate them with start
      v.addAll(mb);
    }

    for (int i = 0; i < v.size(); i++) {
      IFBase fb = (IFBase) v.elementAt(i);
      addCoveredTemplate(start, fb);
    }
    mb = null;
  }

  /**
   * Splits a node using the color, because merge may be performed only
   * on one color. Does not insert the node in the forest.
   * @param old Node
   * @param color Object
   * @return Node
   */
  protected Node colorSplit(Node old, Object color) {

    if (debug) {
      System.out.println("**** performing color split for: " + color);
    }
    Node newNode = new Node();
    try {
      if (isMerged(old)) { // assert
        System.out.println("Fatal error. Old node should have > 1 colors");
        System.exit(0);
      }

      if (debug) {
        System.out.println("!!!! Performing color split: " +
                           old.getData().getTid().size());
        System.out.println("!!!! Old Node " + old + " color: " + color);
        System.out.println("!!!! New Node " + newNode);
      }

      // first, we remove the color from the node
      Template t = (Template) old.getData();

      // Remove the color from the old template
      // Does not change the old node in any way
      t.getTid().remove(color);

      if (debug) {
        System.out.println("Removing color from: " + old);
      }

      if (t.getTid().isEmpty()) {
        System.out.println("colorSplit: fatal tid was empty");
        throw new PosetException();
      }

      // We clone the object, assuming it is a template
      Template t2 = (Template) t.clone();

      if (t2.getTid() != null) {
        t2.getTid().clear();
      }
      t2.addTid(color); // we add the color

      newNode.setData(t2);
      newNode.setParent(old.getParent());
      // note that at this point the new object is not yet inserted anywhere
    }
    catch (Exception e) {
      e.printStackTrace();
      return null;
    }
    return newNode;
  }

  /**
   * Get merged colors from the node to the parent, includes the given node
   * @param x Node
   * @param inc true if the given node is included
   * @return Hashtable
   */
  protected Hashtable getMergedColors(Node x, boolean inc) {
    Node y = x;
    Hashtable mergedColors = new Hashtable();
    if (y == null) {
      return mergedColors;
    }

    do {
      if (y == x && !inc) {
        ;
      }
      else
      if (isMerged(y)) {
        if (y.getData().getTid() != null) {
          mergedColors.put(y.getData().getTid().firstElement(), y);
          if (y.getData().getTid().size() != 1) {
            System.out.println("Fatal: Color index error in getMergedColors");
            System.out.println("Size:" + y.getData().getTid().size());
          }
        }
        else {
          System.out.println("getMergedColors: fatal: tid was null");
        }
      }
      y = y.parent;
    }
    while (y != null);
    return mergedColors;
  }
}
