package fuegocore.notify.core.filter.poset;

import java.util.*;

import fuegocore.notify.core.filter.*;

/**
 * This class implements the basic Node object used in the filter forest
 * data structures.
 */
public class Node
    implements Comparable {
  Vector children; // children of this node
  Node parent; // parent node
  IFBase data;
  int count; // used as a color count for sorting
  int copyCount; // how many copies (object is removed when this reached 0)
  int subtreesize;
  boolean mergeFlag;
  int sigVal;

  // values used by merging forest

  public static final int EMPTY_VAL = 0;
  public static final int COVERED_VAL = 1;
  public static final int COVERING_VAL = 2;
  public static final int MERGE_VAL = 3;
  public static final int ERROR_VAL = 4;

  public Node() {
    children = new Vector();
    copyCount = 0;
    count = 0;
    subtreesize = 0;
    mergeFlag = false;
    sigVal = EMPTY_VAL;
  }

  /**
   * Get temporary cover flag
   * @return boolean
   */
  public int getSigVal() {
    return sigVal;
  }

  /**
   * Set temporary cover flag
   * @param m boolean
   */
  public void setSigVal(int m) {
    sigVal = m;
  }

  /**
   * Get temporary merge flag
   * @return boolean
   */
  public boolean getMerge() {
    return mergeFlag;
  }

  /**
   * Set temporary merge flag
   * @param m boolean
   */
  public void setMerge(boolean m) {
    mergeFlag = m;
  }

  /**
   * Set the count property
   * @param c int
   */
  public void setCount(int c) {
    count = c;
  }

  /**
   * Get the count property
   * @return int
   */
  public int getCount() {
    return count;
  }

  /**
   * Get the parent of this node
   * @return Node
   */
  public Node getParent() {
    return parent;
  }

  /**
   * Set the parent
   * @param p Node
   */
  public void setParent(Node p) {
    parent = p;
  }

  /**
   * Get the size of the subtree
   * @return int
   */
  public int getSubTreeSize() {
    return subtreesize;
  }

  /**
   * Set the size of the subtree
   * @param i int
   */
  public void setSubTreeSize(int i) {
    subtreesize = i;
  }

  /**
   * Get the data object
   * @return IFBase
   */
  public IFBase getData() {
    return data;
  }

  /**
   * Set the data object
   * @param f IFBase
   */
  public void setData(IFBase f) {
    data = f;
  }

  /**
   * Get the children of this node
   * @return Vector
   */
  public Vector getChildren() {
    return children;
  }

  /**
   * Comparison based on depth.
   * @param other the other object to compare to
   * @return int the comparison value
   */
  public int compareTo(Object other) {
    if (other instanceof Node) {
      int a = ( (Node) other).getData().getCost();
      return getData().getCost() - a;
    }
    else {
      throw new ClassCastException();
    }
  }

  /**
   * Compare object using subtree size
   * @param other Object
   * @return int
   */
  public int compareToTreeSize(Object other) {
    if (other instanceof Node) {
      int a = ( (Node) other).getSubTreeSize();
      return getSubTreeSize() - a;

    }
    else {
      throw new ClassCastException();
    }
  }

}
