package fuegocore.notify.core.filter.poset;

import java.util.*;

/**
 * Manages different poset implementations, creates a new instance
 * of the given algorithm
 */
public class PosetService {
  private static PosetService ps;
  private Hashtable algorithms;

  public PosetService() {
    algorithms = new Hashtable();
    register("poset", new Poset());
    register("default", new FilterForest());
    register("balancedforest", new BalancedFilterForest());
  }

  /** Add a new poset algorithm to the framework.
   *
   * @param name Name of the algorithm
   * @param im Instance of the algorithm
   */
  public void register(String name, IFilterPoset im) {
    algorithms.put(name, im);
  }

  /** Remove a poset implementation
   *
   * @param name the name of the algorithm
   */
  public void deregister(String name) {
    algorithms.remove(name);
  }

  public IFilterPoset getDefault() {
    return (IFilterPoset) algorithms.get("default");
  }

  /**
   * Factory method to get a poset implementation
   * @return IFilterPoset
   */
  public static synchronized IFilterPoset get() {
    return get("default");
  }

  /**
   * Factory method to get a poset implementation
   * @param name String
   * @return IFilterPoset
   */
  public static synchronized IFilterPoset get(String name) {
    if (ps == null) {
      ps = new PosetService();
    }
    if (ps.algorithms != null) {
      try {
        Class abc = (Class) ps.algorithms.get(name).getClass();
        Object obj = abc.newInstance();
        return (IFilterPoset) obj;
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }
    return null;
  }
}
