/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.filter.poset;

import java.io.*;
import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;
import fuegocore.notify.core.filter.merge.*;
import fuegocore.notify.core.filter.test.*;
import fuegocore.notify.util.*;
import fuegocore.notify.core.filter.poset.forwards.*;

/**
 * Poset test class for testing the various
 *  filter forest and poset implementations.
 */
public class PosetTest extends FilterForest {
    public int REPLICATIONS = 4;
    public boolean VARIABLE = true;
    public boolean NEWIFACEADD = false;
    public String DRIVEANDPATH = null;

    /* Use buffer that filters impossible additions, namely
     1) add that is covered, 2) add after del that covers */
    public boolean USE_BUFFER = false;

    /* Use peer-to-peer or hierarchical mode */
    public boolean USE_DISTRIBUTED = false;

    /* Minimize posets (periodic updates) */
    public boolean MINIMIZE_POSETS = false;

    /** Compute forwards sets (only with DISTRIBUTED) */
    public boolean COMPUTE_FORWARDS = false;

    /** Test forwards set computation (only with DISTRIBUTED) */
    public boolean TEST_FORWARDS = false;

    /**  Use local clients, only useful with FORWARDS */
    public boolean USE_LOCAL_CLIENTS = false;

    /** DS for buffer layer */
    HashMap bufferRoots = new HashMap();


    public int LIMIT_BROKERS = -1;

    public static void main(String[] args) {

        PosetTest p = new PosetTest();

            if (!p.runHierarchicalTest(2)) {
              System.out.println("Hierarchical test failed");
              System.exit(0);
            }
            else {
              System.out.println("Hierarchical test ok");
            }

        if (!p.runDistTestLocal(2)) {
            System.out.println("Distributed (peer-to-peer) test failed");
            System.exit(0);
        } else {
            System.out.println("Distributed (peer-to-peer) test ok");
        }

        if (!p.runDistTest(2)) {
            System.out.println("Distributed (peer-to-peer) test failed");
            System.exit(0);
        } else {
            System.out.println("Distributed (peer-to-peer) test ok");
        }

            if (!p.runLocalTest()) {
              System.out.println("Local test failed");
              System.exit(0);
            }
            else {
              System.out.println("Local test ok");
            }
    }

    private void testCovering(Vector orig, Vector mod) {
        int covers = 0, covered = 0, identical = 0;
        int covers2 = 0, covered2 = 0, identical2 = 0;

        for (int i = 0; i < orig.size(); i++) {
            Template t = (Template) orig.elementAt(i);
            for (int j = 0; j < orig.size(); j++) {
                Template t2 = (Template) orig.elementAt(j);
                if (t.covers(t2)) {
                    covers++;
                    if (t2.covers(t)) {
                        identical++;
                    }
                } else if (t2.covers(t)) {
                    covered++;
                }
            }
        }

        for (int i = 0; i < mod.size(); i++) {
            Template t = (Template) mod.elementAt(i);
            for (int j = 0; j < mod.size(); j++) {
                Template t2 = (Template) mod.elementAt(j);
                if (t.covers(t2)) {
                    covers2++;
                    if (t2.covers(t)) {
                        identical2++;
                    }
                } else if (t2.covers(t)) {
                    covered2++;
                }
            }
        }
        System.out.println("Covers " + covers + " covered " + covered +
                           " identical " +
                           identical);
        System.out.println("Covers2 " + covers2 + " covered2 " + covered2 +
                           " identical2 " +
                           identical2);
        if (covers != covers2 || covered != covered2 || identical != identical2) {
            System.out.println("Cover test failed");
        } else {
            System.out.println("Cover test ok");
        }
    }



    public boolean runDistTest(int dim) {
        boolean ret = true;
        System.out.println("******************************************");

        // we change to distributed config
        // FORWARDS SETS
        USE_BUFFER = true;
        USE_DISTRIBUTED = true;
        VARIABLE = false;
        COMPUTE_FORWARDS = true;
        USE_LOCAL_CLIENTS = false;
        REPLICATIONS = 10;

        System.out.println("******************************************");

        for (int k = 3; k <= 15; k++) {
            System.out.println("Testing: round " + k);
            ret = testForestAndPoset(500, EventTriplet.TYPE_INT, dim, 100,
                                          true,
                                          k);
            if (!ret) {
                return ret;
            }
        }

        System.out.println("******************************************");

        USE_LOCAL_CLIENTS = true;

        for (int k = 3; k <= 15; k++) {
            System.out.println("Testing: round " + k);
            ret = testForestAndPoset(500, EventTriplet.TYPE_INT, dim, 100,
                                          true,
                                          k);
            if (!ret) {
                return ret;
            }
        }

        System.out.println("******************************************");

        return ret;
    }


    public boolean runDistTestLocal(int dim) {
        boolean ret = true;
        System.out.println("******************************************");

        // we change to distributed config
        // FORWARDS SETS
        USE_BUFFER = true;
        USE_DISTRIBUTED = true;
        VARIABLE = false;
        COMPUTE_FORWARDS = true;
        USE_LOCAL_CLIENTS = false;
        REPLICATIONS = 10;

        System.out.println("******************************************");

        USE_LOCAL_CLIENTS = true;
        LIMIT_BROKERS = 4;

        for (int k = 10; k <= 100; k += 10) {
            System.out.println("Testing: round " + k);
            ret = testForestAndPoset(500, EventTriplet.TYPE_INT, dim, 100,
                                          true,
                                          k);
            if (!ret) {
                return ret;
            }
        }

        System.out.println("******************************************");
        LIMIT_BROKERS = -1;

        return ret;
    }

    public boolean runHierarchicalTest(int dim) {
        boolean ret = true;
        System.out.println("******************************************");

        // we change to distributed config
        // FORWARDS SETS
        USE_BUFFER = true;
        USE_DISTRIBUTED = true;
        VARIABLE = false;
        COMPUTE_FORWARDS = false;
        USE_LOCAL_CLIENTS = false;
        REPLICATIONS = 10;

        System.out.println("******************************************");

        for (int k = 3; k <= 15; k++) {
            System.out.println("Testing: round " + k);
            ret = testForestAndPoset(500, EventTriplet.TYPE_INT, dim, 100,
                                          true,
                                          k);
            if (!ret) {
                return ret;
            }
        }

        System.out.println("******************************************");

        for (int k = 100; k <= 500; k += 100) {
            System.out.println("Testing: round " + k);
            ret = testForestAndPoset(k, EventTriplet.TYPE_INT, dim, 100,
                                          true,
                                          4);
            if (!ret) {
                return ret;
            }
        }

        System.out.println("******************************************");

        return ret;
    }

    public boolean runLocalTest() {

        // Local config: 2D queries
        boolean ret = true;

        NEWIFACEADD = true;
        REPLICATIONS = 5;
        VARIABLE = false;
        boolean rangeOnly = true;
        USE_BUFFER = false;
        USE_DISTRIBUTED = false;
        COMPUTE_FORWARDS = false;
        USE_LOCAL_CLIENTS = false;

        for (int k = 50; k <= 500; k += 50) { // 50
            System.out.println("Testing: template count: " + k);
            ret = testForestAndPoset(k, EventTriplet.TYPE_INT, 2, 100,
                                          rangeOnly,
                                          k);
            if (!ret) {
                return ret;
            }
        }

        for (int k = 50; k <= 500; k += 50) { // 50
            System.out.println("Testing: template count: " + k);
            ret = testForestAndPoset(500, EventTriplet.TYPE_INT, 2, 100,
                                          rangeOnly,
                                          k);
            if (!ret) {
                return ret;
            }
        }

        // Local config: 3D queries
        for (int k = 50; k <= 500; k += 50) { // 50
            System.out.println("Testing: template count: " + k);
            ret = testForestAndPoset(k, EventTriplet.TYPE_INT, 3, 100,
                                          rangeOnly,
                                          k);
            if (!ret) {
                return ret;
            }
        }
        VARIABLE = true;

// Local config: variable 3D queries
        for (int k = 50; k <= 500; k += 50) { // 50
            System.out.println("Testing: template count: " + k);
            ret = testForestAndPoset(k, EventTriplet.TYPE_INT, 3, 100,
                                          rangeOnly,
                                          k);
            if (!ret) {
                return ret;
            }
        }
        NEWIFACEADD = false;

        return ret;
    }


    public Vector getMinimalCover(Vector a, String prefix, int ifaces) {
        Vector ret = new Vector();

        // we measure only the merging time...
        try {
            for (int i = 0; i < ifaces; i++) {
                Vector temp = new Vector();
                for (int j = 0; j < a.size(); j++) {
                    Template t = (Template) a.elementAt(j);
                    if (t.getInterface().equals(prefix + i)) {
                        temp.add(t);
                    }
                }
                FilterForest ps = new FilterForest();
                try {
                    ps.add(temp);
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }
                ret.addAll(ps.get());
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        return ret;
    }

    /** Test driver for the data structure.
     *  @param count the number of templates to test
     *  @param type the type of templates
     *  @param fields the number of fields, 1 for attribute filters
     *  @param names number of names for filters
     *  @param equ whether or not equality mode (for strings) should be used
     *  @param ifaces the number of interfaces
     *  @param suffix the suffix for file name
     *  @return true if the test was succesful
     */
    public boolean testForestAndPoset(int count, int type, int fields,
                                           int names,
                                           boolean equ, int ifaces) {
        boolean ret = true;

        reset(); // the data structure must be reset

        // create random subscriptions with one filter
        // First, create the schema
        Vector schema = new Vector();

        Template tm = new Template();

        for (int i = 0; i < fields; i++) {
            tm.addFilter(new Filter("val" + i, type,
                                    ""));
        }

        // type is not taken into account but it is needed for some filter service calls
        tm.addFilter(new Filter("type", EventTriplet.TYPE_STRING,
                                "koe"));
        schema.add(tm);

        FilterServiceTestDriver driver = new FilterServiceTestDriver(false);
        if (equ) {
            driver.RANGE_ONLY = true;
        }
        driver.FILTERNAMES = names;
        driver.TEMPLATEFIELDS = fields;

        driver.populateFields();

        //-----------------------------------------------------------------------

        /* ******************************************************************* */
        /* TEST POSET ALGORITHMS: ADD AND REMOVE                               */



        for (int test = 0; test < REPLICATIONS; test++) {
            System.out.println("Replication: " + test);

            // create templates using the schema
            Vector temps = null;
            if (!VARIABLE) {
                temps = driver.createTemplatesUsingSchemas(count, schema);
            } else {
                temps = driver.createVariableTemplatesUsingSchemas(count,
                        schema);
            }

            // create templates using the schema
            Vector temps2 = null;

            if (!VARIABLE) {
                temps2 = driver.createTemplatesUsingSchemas(count, schema);
            } else {
                temps2 = driver.createVariableTemplatesUsingSchemas(count,
                        schema);
            }

            // assign interfaces
            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                t.setInterface("iface" + (i % ifaces));
                Template t2 = (Template) temps2.elementAt(i);
                if (NEWIFACEADD) { // add totally new iface for add/remove
                    t2.setInterface("iface2_" + (i % ifaces));
                } else { // use the old iface
                    t2.setInterface("iface" + (i % ifaces));
                }
            }

            // Create a set containing the interfaces
            Vector ifaceSet = new Vector();

            int ifaceCount = ifaces;

            if (USE_LOCAL_CLIENTS) { // use one fewer neighbours...
                ifaceCount--;
            }

            if (LIMIT_BROKERS > -1) {
                ifaceCount = LIMIT_BROKERS;
            }

            for (int i = 0; i < ifaceCount; i++) {
                ifaceSet.add("iface" + i);
            }

            System.out.println("IfaceSet size: " + ifaceSet.size());

            // Remove redundancy (minimal covers)

            if (MINIMIZE_POSETS) {
                if (ifaces != temps.size()) {
                    temps = getMinimalCover(temps, "iface", ifaces);

                    if (NEWIFACEADD) {
                        temps2 = getMinimalCover(temps2, "iface2_", ifaces);
                    } else {
                        temps2 = getMinimalCover(temps2, "iface", ifaces);
                    }

                    /// we swap to ensure that temps is the smallest (add/del requires this)
                    if (temps.size() > temps2.size()) {
                        Vector temps4 = temps2;
                        temps2 = temps;
                        temps = temps2;
                    }
                }
            }
            // Create notifications for matching tests
            Vector notifications = new Vector();

            for (int j = 0; j < 100; j++) {
                int i = (int) (Math.random() * schema.size());
                Template schem = (Template) schema.elementAt(i);
                notifications.add(driver.getNotificationUsingSchema(schem));
            }

            // remove random templates and add them again
            reset();

            // Add test root invariance check
            int poset_size1 = 0;
            int poset_size2 = 0;
            int poset_size3 = 0;
            int poset_size4 = 0;

            // Add/remove test root invariance check
            int poset_size11 = 0;
            int poset_size22 = 0;
            int poset_size33 = 0;
            int poset_size44 = 0;

            /* ******************************************************************* */

            reset();

            if (USE_BUFFER) {
                resetBuffer();
            }

            this.SIBLING_PURITY_ON_ELIMINATE = false;
            this.SKIP_SIBLING_PURITY = true;

            /* ADD TEST */
            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                if (t.getInterface() == null) {
                    System.out.println(
                            "Fatal: interface was null before addition!");
                    System.exit(0);
                }
                try {
                    if (USE_BUFFER) {
                        boolean b = bufferLayerAdd(t, this, t.getInterface());
                        if (b) {
                            add(t, t.getInterface());
                        }
                    } else {
                        add(t, t.getInterface());
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }
            }

            System.out.println("Forest after add: " + get().size());
            poset_size1 = get().size();

            // test color index
            if (!testColorIndex()) {
                System.exit(0);
            }

            /* ******************************************************************* */

            Vector[] ifaceTest = new Vector[100];

            // table for checking matching count numbers
            int matchCheckTable[] = new int[100];

            // Create benchmark set for notifications (add)
            for (int p = 0; p < notifications.size(); p++) {
                Notification n = (Notification) notifications.elementAt(p);
                Vector ifs = new Vector();
                for (int k = 0; k < temps.size(); k++) {
                    Template tz = (Template) temps.elementAt(k);
                    if (tz.match(null, n)) {
                        if (!ifs.contains(tz.getInterface())) {
                            ifs.add(tz.getInterface());
                        }
                    }
                }
                ifaceTest[p] = ifs;
                matchCheckTable[p] = ifs.size();
            }


            /* Match test */
            System.out.println("Testing matching for add");
            matchTest(notifications, this, matchCheckTable, ifaceTest);

            /* ******************************************************************* */
            /* REMOVE TEST */
            System.out.println("Starting add/remove matching");

            /* ******************************************************************* */

            /** ADD and REMOVE for plain forest */
            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);

                int before_remove = nodes.size();

                // Perform remove algorithm
                try {
                    if (remove(t, t.getInterface()) == FilterForest.POSET_ERROR) {
                        // filter may not be there if ifaces < temps.size()
                        if (ifaces == temps.size()) {
                            System.out.println(
                                    "Forest: Template was not found in remove test: " +
                                    t.getInterface());
                            System.out.println(t.getTid());
                        }
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }

                int after_remove = nodes.size();

               Template t2 = (Template) temps2.elementAt(i);

                try {
                    if (USE_BUFFER) {
                       boolean b = bufferLayerAddAfterDel(t2, this,
                                t2.getInterface());
                         if (b) {
                            add(t2, t2.getInterface());
                        }
                    } else {
                        add(t2, t2.getInterface());
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }
             }


            /** Note that the insertion/deletion false positive and negative
             * detection is tricky when the interface < temps or temps2,
             * because of duplicates and minimum color property. Therefore this
             * is tested only when interfaces==temps.size, and otherwise we
             * test the integrity of the templates in the structure.
             */

            if (ifaces == temps2.size() || NEWIFACEADD) {
                testAddRemoveMatching(notifications, this, temps2);
            } else {
                testAddRemoveMatching(notifications, this,
                                      this.getAllAsTemplates());

            }
            System.out.println("** Root after add/remove:" + this.get().size());
            poset_size11 = this.get().size();

            if (!testColorIndex()) {
                System.exit(0);
            }

            System.out.println("Add test for forest2");

            /* ******************************************************************* */
            /* ******************************************************************* */
            /* ADD TEST */
            BalancedFilterForest bff = new BalancedFilterForest();

            if (USE_DISTRIBUTED) {
                bff.REMOVE_ALL_REDUNDANCY = true;
            }

            if (USE_BUFFER) {
                resetBuffer();
                bff.NO_COVERED_UPDATES = true;
            }

            ForestForwardsSet fw_forest = null;

            if (COMPUTE_FORWARDS) {
                fw_forest = new ForestForwardsSet(bff, ifaceSet);
                fw_forest.NONCOMPLETE = false; // exact set
            }

            int total_dist_updates = 0;
            int temp_cost = 0;

            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                try {
                    if (USE_BUFFER) {
                        long timex = System.currentTimeMillis();
                        boolean b = bufferLayerAdd(t, bff, t.getInterface());
                        if (b) {
                            total_dist_updates++;
                            bff.add(t, t.getInterface());
                        }
                    } else {
                        bff.add(t, t.getInterface());
                    }

                    if (TEST_FORWARDS) {
                        Hashtable forwardsSet = ForwardsSet.computeForwardsSet(
                                bff.
                                getAllNodes(), ifaceSet);

                        Vector v = (Vector) forwardsSet.get(t);

                        Hashtable fw = fw_forest.get();

                        Vector v2 = (Vector) fw.get(t.getCoverId());

                        if (v == null) {
                            v = new Vector();
                        }
                        if (v2 == null) {
                            v2 = new Vector();
                        }

                        if (!v.containsAll(v2) || !v2.containsAll(v)) {
                            System.out.println("Forwards set error for forest");
                            System.out.println("Naive:" + v);
                            System.out.println("Forest:" + v2);
                            Node xf = bff.lookupNode(t);
                            System.out.println("Root filter: " + xf.getParent());
                            System.out.println("Failing filter:" + xf.getData());
                            if (xf.getParent() != null) {
                                System.out.println("Root filter data: " +
                                        xf.getParent().getData());
                            }
                            System.out.println("Covering nodes");
                            for (int zi = 0; zi < bff.getAllNodes().size(); zi++) {
                                IFBase y = (IFBase) bff.getAllNodes().elementAt(
                                        zi);

                                if (y.covers(t)) {
                                    System.out.println(y);
                                    Vector v3 = (Vector) fw.get(y.getCoverId());
                                    if (y == t) {
                                        System.out.println(
                                                "Non-applicable: Forwards set:" +
                                                v3);
                                    } else {
                                        System.out.println(
                                                "Applicable: Forwards set:" +
                                                v3);
                                    }

                                    System.out.println("Tid:" + y.getTid());
                                    System.out.println("---------");
                                }
                            }

                            Vector c = xf.getParent().getChildren();

                            System.out.println("--\nChildren test -------\n");

                            System.out.println(
                                    "Parent's children set contains current:" +
                                    c.contains(xf));

                            for (int zi = 0; zi < c.size(); zi++) {
                                Node y = (Node) c.elementAt(zi);
                                System.out.println(y.getData());
                            }

                            forwardsSet = ForwardsSet.computeForwardsSet(bff.
                                    getAllNodes(), ifaceSet, t);

                            System.exit(0);
                        }

                        System.out.println("--------- FW temp cost : " +
                                           temp_cost +
                                           " / forest " + fw_forest.getTotal());
                        System.out.println("Cost v:" + v.size());
                        if (v != null) {
                            temp_cost += v.size();
                        }

                        if (temp_cost != fw_forest.getTotal()) {
                            System.out.println("Cost error");
                            System.out.println("--------- FW temp cost : " +
                                               temp_cost +
                                               " / forest " +
                                               fw_forest.getTotal());
                            Node xf = bff.lookupNode(t);
                            System.out.println("Root filter: " + xf.getParent());
                            if (xf.getParent() != null) {
                                System.out.println("Root filter data: " +
                                        xf.getParent().getData());
                            }
                            System.out.println("Naive:" + v);
                            System.out.println("Forest:" + v2);
                            System.out.println("Node: " + t);

                            bff.testSiblingPurity(xf.getParent());

                            System.out.println("Covering nodes");
                            for (int zi = 0; zi < bff.getAllNodes().size(); zi++) {
                                IFBase y = (IFBase) bff.getAllNodes().elementAt(
                                        zi);
                                if (y.covers(t)) {
                                    System.out.println(y);
                                    Vector v3 = (Vector) fw.get(y.getCoverId());
                                    System.out.println("Forwards set:" + v3);
                                    System.out.println("---------");
                                }
                            }
                            System.exit(0);
                        }
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }
            }


            // We check the integrity of the forwards sets after the delete phase
            if (COMPUTE_FORWARDS) {
                if (!compareForwardsSets(bff.getAllNodes(), ifaceSet,
                                         fw_forest,
                                         null)) {
                    System.out.println(
                            "Fatal error forest2: forwards sets differ");
                    System.out.println(fwComparisonResult);
                    System.exit(0);
                }
                fw_forest.resetTotal();
            }

            System.out.println("Forest2 after add: " + bff.get().size());
            poset_size2 = bff.get().size();

            /* Match test */
            matchTest(notifications, bff, matchCheckTable, ifaceTest);

            // test color index
            if (!bff.testColorIndex()) {
                System.exit(0);
            }

            /* ******************************************************************* */
            /** ADD and REMOVE with SORT */
            System.out.println("Add/remove test for forest2");
            /* This does not affect the poset size so those measurements are not done again */
            total_dist_updates = 0;
            temp_cost = 0;
            Hashtable temp_fw = new Hashtable();

            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                int countd = 0;

                if (TEST_FORWARDS) {
                    temp_fw = ForwardsSet.computeForwardsSet(bff.getAllNodes(),
                            ifaceSet);
                }

                boolean remove_ok = true;
                try {
                    // Perform remove algorithm
                    if (bff.remove(t, t.getInterface()) ==
                        FilterForest.POSET_ERROR) {
                        remove_ok = false;
                        // filter may not be there if ifaces < temps.size()
                        if (ifaces == temps.size()) {
                            System.out.println(
                                    "Forest2: Template was not found in remove test");
                        }
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }


                if (TEST_FORWARDS) {
                    if (remove_ok) { // only if the filter is contained
                        // we have to the checking previously due to duplicate handling
                        // this is not a problem with add

                        countd = ForwardsSet.computeForwardsSetRemove(bff.
                                getAllNodes(), ifaceSet, t, temp_fw);

                        temp_cost += countd;

                    }
                    System.out.println(
                            "*****************************************");
                }

                Template t2 = (Template) temps2.elementAt(i);
                try {
                    if (USE_BUFFER) {
                        long timex = System.currentTimeMillis();
                        boolean b = bufferLayerAddAfterDel(t2, bff,
                                t2.getInterface());
                        if (b) {
                            total_dist_updates++;
                            bff.addAfterDel(t2, t2.getInterface());
                        }
                    } else {
                        bff.add(t2, t2.getInterface());
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }

                if (TEST_FORWARDS) {
                    Hashtable forwardsSet = ForwardsSet.computeForwardsSet(bff.
                            getAllNodes(), ifaceSet);

                    Vector v = (Vector) forwardsSet.get(t2);
                    if (v != null) {
                        temp_cost += v.size();
                    }
                    System.out.println("--------- FW temp cost : " + temp_cost +
                                       " / forest2 " + fw_forest.getTotal());

                    System.out.println(
                            "---------------------------------------");

                    if (temp_cost != fw_forest.getTotal()) {
                        System.out.println("Cost error for add/remove (forest2)");
                        System.out.println("countd:" + countd);
                        System.out.println("--------- FW temp cost : " +
                                           temp_cost +
                                           " / forest2 " + fw_forest.getTotal());
                        System.exit(0);

                    }

                }
              }


            // test color index
            if (!bff.testColorIndex()) {
                System.exit(0);
            }

            /** Note that the insertion/deletion false positive and negative
             * detection is tricky when the interface < temps or temps2,
             * because of duplicates and minimum color property. Therefore this
             * is tested only when interfaces==temps.size, and otherwise we
             * test the integrity of the templates in the structure.
             */

            if (ifaces == temps2.size() || NEWIFACEADD) {
                testAddRemoveMatching(notifications, bff, temps2);
            } else {
                testAddRemoveMatching(notifications, bff, bff.getAllAsTemplates());

            }
            System.out.println("** forest2 root after add/remove:" +
                               bff.get().size());
            poset_size22 = bff.get().size();

            /* ***************************************************************** */
            /*FOREST3 ADD TEST */

            BalancedFilterForest mergef = new BalancedFilterForest();

            if (USE_DISTRIBUTED) {
                // use only external color elimination
                mergef.REMOVE_ALL_REDUNDANCY = true;
            }

            if (USE_BUFFER) {
                resetBuffer();
                mergef.NO_COVERED_UPDATES = true;
            }

            ForestForwardsSet fw_forest2 = null;
            if (COMPUTE_FORWARDS) {
                fw_forest2 = new ForestForwardsSet(mergef, ifaceSet);
                fw_forest2.NONCOMPLETE = true; // inexact set
            }

            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                try {
                    if (USE_BUFFER) {
                        boolean b = bufferLayerAdd(t, mergef, t.getInterface());
                        if (b) {
                            mergef.add(t, t.getInterface());
                        }
                    } else {
                        mergef.add(t, t.getInterface());
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }

            }


            System.out.println("Forest3 after add: " + mergef.get().size());
            poset_size4 = mergef.get().size();

            // Match test
            matchTest(notifications, mergef, matchCheckTable, ifaceTest);

            // test color index
            if (!mergef.testColorIndex()) {
                System.exit(0);
            }

            /* ******************************************************************* */
            /** ADD and REMOVE with SORT */
            System.out.println("Add/remove test for forest3");

            /* This does not affect the poset size so those measurements are not done again */

             for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                try {
                    // Perform remove algorithm
                    if (mergef.remove(t, t.getInterface()) ==
                        FilterForest.POSET_ERROR) {
                        // filter may not be there if ifaces < temps.size()
                        if (ifaces == temps.size()) {
                            System.out.println(
                                    "Forest3: Template was not found in remove test");
                        }
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }

                Template t2 = (Template) temps2.elementAt(i);
                try {
                    if (USE_BUFFER) {
                        boolean b = bufferLayerAddAfterDel(t2, mergef,
                                t2.getInterface());
                        if (b) {
                            mergef.addAfterDel(t2, t2.getInterface());
                        }
                    } else {
                        mergef.add(t2, t2.getInterface());
                    }
                } catch (PosetException pe) {
                    pe.printStackTrace();
                    System.exit(0);
                }
          }


            if (COMPUTE_FORWARDS) {
                fw_forest2.resetTotal();
            }

            // test color index
            if (!mergef.testColorIndex()) {
                System.exit(0);
            }

            /** Note that the insertion/deletion false positive and negative
             * detection is tricky when the interface < temps or temps2,
             * because of duplicates and minimum color property. Therefore this
             * is tested only when interfaces==temps.size, and otherwise we
             * test the integrity of the templates in the structure.
             */

            if (ifaces == temps2.size() || NEWIFACEADD) {
                testAddRemoveMatching(notifications, mergef, temps2);
                //  testAddRemoveMatching(notifications, mergef, mergef.getAllAsTemplates());
            } else {
                testAddRemoveMatching(notifications, mergef,
                                      mergef.getAllAsTemplates());
            }

            System.out.println("** forest3 root after add/remove:" +
                               mergef.get().size());
            poset_size44 = mergef.get().size();

            /* ******************************************************************* */
            /* ******************************************************************* */
            /* ******************************************************************* */
            /** Filters poset test */
            Poset pos = new Poset();
            if (USE_BUFFER) {
                resetBuffer();
            }

            PosetForwardsSet fw_poset = null;

            if (COMPUTE_FORWARDS) {
                fw_poset = new PosetForwardsSet(pos, ifaceSet);
            }

            temp_cost = 0;
            /* ADD TEST */
            total_dist_updates = 0;

            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                if (USE_BUFFER) {
                    long timex = System.currentTimeMillis();
                    boolean b = bufferLayerAdd(t, pos, t.getInterface());
                    if (b) {
                        total_dist_updates++;
                        pos.add(t, t.getInterface());
                    }
                } else {
                    pos.add(t, t.getInterface());
                }

                if (TEST_FORWARDS) {
                    Hashtable forwardsSet = ForwardsSet.computeForwardsSet(pos.
                            getAllNodes(), ifaceSet);

                    Vector v = (Vector) forwardsSet.get(t);

                    Hashtable fw = fw_poset.get();

                    Vector v2 = (Vector) fw.get(t.getCoverId());

                    if (v == null) {
                        v = new Vector();
                    }
                    if (v2 == null) {
                        v2 = new Vector();
                    }

                    if (!v.containsAll(v2) || !v2.containsAll(v)) {
                        System.out.println("Forwards set error for poset");
                        System.out.println("Naive:" + v);
                        System.out.println("Poset:" + v2);
                        System.out.println("Covering nodes");

                        System.exit(0);
                    }

                    System.out.println("--------- FW temp cost : " + temp_cost +
                                       " / poset " + fw_poset.getTotal());

                    System.out.println("Cost v:" + v.size());

                    if (v != null) {
                        temp_cost += v.size();
                    }

                    if (temp_cost != fw_poset.getTotal()) {
                        System.out.println("Cost error");
                        System.out.println("--------- FW temp cost : " +
                                           temp_cost +
                                           " / poset " + fw_poset.getTotal());

                        System.out.println("Naive:" + v);
                        System.out.println("Poset:" + v2);

                        System.exit(0);
                    }
                }
        }



            System.out.println("Filters poset after add: " + pos.get().size());
            poset_size3 = pos.get().size();

            /* ******************************************************************* */
            /* Match test */
            matchTest(notifications, pos, matchCheckTable, ifaceTest);

            // We check the integrity of the forwards sets after the delete phase
            if (COMPUTE_FORWARDS) {
                if (!compareForwardsSets(pos.getAllNodes(), ifaceSet,
                                         null,
                                         fw_poset)) {
                    System.out.println(
                            "Fatal error poset: forwards sets differ");
                    System.out.println(fwComparisonResult);
                    System.exit(0);
                }
                fw_poset.resetTotal();
            }
            /* ******************************************************************* */

            // test color index
            if (!pos.testColorIndex()) {
                System.exit(0);
            }

            System.out.println("Add/remove test with filters poset");
            total_dist_updates = 0;
            temp_cost = 0;

            /** ADD and REMOVE with POSET */
            for (int i = 0; i < temps.size(); i++) {
                Template t = (Template) temps.elementAt(i);
                int countd = 0;
                if (TEST_FORWARDS) {
                    temp_fw = ForwardsSet.computeForwardsSet(pos.getAllNodes(),
                            ifaceSet);
                }

                long dtime1 = System.currentTimeMillis();
                // Perform remove algorithm
                boolean remove_ok = true;
                if (pos.remove(t, t.getInterface()) == FilterForest.POSET_ERROR) {
                    remove_ok = false;
                    // filter may not be there if ifaces < temps.size()
                    if (ifaces == temps.size()) {
                        System.out.println(
                                "Poset: Template was not found in remove test");
                    }
                }

                if (TEST_FORWARDS) {
                    if (remove_ok) { // only if the filter is contained
                        // we have to the checking previously due to duplicate handling
                        // this is not a problem with add
                        countd = ForwardsSet.computeForwardsSetRemove(pos.
                                getAllNodes(),
                                ifaceSet, t, temp_fw);

                        temp_cost += countd;

                    }
                }

                Template t2 = (Template) temps2.elementAt(i);
                if (USE_BUFFER) {
                    long timex = System.currentTimeMillis();
                    boolean b = bufferLayerAddAfterDel(t2, pos, t2.getInterface());
                    if (b) {
                        total_dist_updates++;
                        pos.add(t2, t2.getInterface());
                    }
                } else {
                    pos.add(t2, t2.getInterface());
                }

                if (TEST_FORWARDS) {
                    Hashtable forwardsSet = ForwardsSet.computeForwardsSet(pos.
                            getAllNodes(), ifaceSet);

                    Vector v = (Vector) forwardsSet.get(t2);
                    if (v != null) {
                        temp_cost += v.size();
                    }
                    System.out.println("--------- FW temp cost : " + temp_cost +
                                       " / poset " + fw_poset.getTotal());

                    System.out.println(
                            "---------------------------------------");

                    if (temp_cost != fw_poset.getTotal()) {
                        System.out.println("Cost error for add/remove (poset)");
                        System.out.println("countd:" + countd);
                        System.out.println("--------- FW temp cost : " +
                                           temp_cost +
                                           " / poset " + fw_poset.getTotal());
                        System.exit(0);

                    }

                }
            }
            System.out.println("** Poset root after add/remove:" +
                               pos.get().size());
            poset_size33 = pos.get().size();


            /** Note that the insertion/deletion false positive and negative
             * detection is tricky when the interface < temps or temps2,
             * because of duplicates and minimum color property. Therefore this
             * is tested only when interfaces==temps.size, and otherwise we
             * test the integrity of the templates in the structure.
             */

            if (ifaces == temps2.size() || NEWIFACEADD) {
                testAddRemoveMatching(notifications, pos, temps2);
            } else {
                testAddRemoveMatching(notifications, pos, pos.getAllAsTemplates());
            }

            // test color index
            if (!pos.testColorIndex()) {
                System.exit(0);
            }

            // Note that the size may differ with the settings
            // For example interface elimination makes a difference
            System.out.println("Original size: " + temps.size() +
                               " Forest size: " + this.getNodeCount() +
                               " Forest temps size: " + this.getAllNodes().size() +
                               " Forest2 size: " + bff.getNodeCount() +
                               " Forest2 temps size: " +
                               bff.getAllNodes().size() +
                               " Forest3 size: " + mergef.getNodeCount() +
                               " Forest3 temps size: " +
                               mergef.getAllNodes().size() +
                               " Poset size: " + pos.getNodeCount() + " " +
                               " Poset temps size: " + pos.getAllNodes().size());

            // Test hashtable integrity
            if (ifaces == temps.size() && templates.size() != pos.getNodeCount()) {
                System.out.println(
                        "Hashtable size incorrect with filters poset: " +
                        templates.size() +
                        " / " + pos.getNodeCount());
            } else {
                System.out.println("Filters poset ok");
            }
            if (ifaces == temps.size() && templates.size() != getNodeCount()) {
                System.out.println("Hashtable size incorrect with forest: " +
                                   templates.size() +
                                   " / " + getNodeCount());
            } else {
                System.out.println("Forest ok");
            }

            if (ifaces == temps.size() &&
                templates.size() != bff.getAllNodes().size()) {
                System.out.println(
                        "Hashtable size incorrect with forest2: " +
                        templates.size() +
                        " / " + bff.getAllNodes().size());
            } else {
                System.out.println("Forest2 ok");
            }

            if (getNodeCount() != templates.size()) {
                System.out.println("Forest node count error");
                System.out.println("NodeCount:" + getNodeCount());
                System.out.println("Templates:" + templates.size());
                System.exit(0);
            }

            // Add test root-size invariance
            if (poset_size1 != poset_size2) {
                System.out.println("Add test root set size error");
                System.out.println("Forest1: " + poset_size1);
                System.out.println("Forest2: " + poset_size2);
                System.exit(0);
            }

            if (poset_size1 != poset_size3) {
                System.out.println("Add test root set size error");
                System.out.println("Forest1: " + poset_size1);
                System.out.println("Filters poset: " + poset_size3);

                System.exit(0);
            } else {
                System.out.println("Add-test root set size ok");
            }

            // We check the integrity of the forwards sets after the delete phase
            if (COMPUTE_FORWARDS) {
                System.out.println("-- testing forwards sets after add/del");
                if (!compareForwardsSets(pos.getAllNodes(), ifaceSet,
                                         fw_forest,
                                         fw_poset)) {
                    System.out.println("Fatal error: forwards sets differ");
                    System.out.println(fwComparisonResult);
                    // System.exit(0);
                }
            }

            // There is no add/remove root invariance in this case
            // Add test/remove root-size invariance

            // note that the add/del chain may change root set size when
            // non-redundancy is not guaranteed.
            if (poset_size22 != poset_size33) { // NRF must be equal to poset
                System.out.println("Add/remove test root set size error");
                System.out.println("Forest1: " + poset_size11);
                System.out.println("Forest2: " + poset_size22);
                System.out.println("Poset size:" + poset_size33);
                System.out.println("Forest1 size:" + templates.size());
                System.out.println("Forest2 size:" + bff.getAllNodes().size());
                System.out.println("Poset size:" + pos.getNodeCount());
                Vector set1 = getAllAsTemplates();
                Vector set2 = bff.getAllAsTemplates();
                Vector set3 = pos.getAllAsTemplates();
                // Test cover
                Vector noncovered1 = MergingService.get().getNonCovered(set1);
                Vector noncovered2 = MergingService.get().getNonCovered(set2);
                Vector noncovered3 = MergingService.get().getNonCovered(set3);
                System.out.println("Forest1 noncovered: " + noncovered1.size());
                System.out.println("Forest2 noncovered: " + noncovered2.size());
                System.out.println("Poset size noncovered:" + noncovered3.size());

                testElimination(temps, temps2);

                System.exit(0);
            } else {
                System.out.println("Add/remove test root set size ok");
            }
        }

        /*** STATISTICS ********/
        return ret;
    }

    String fwComparisonResult = "";

    public boolean compareForwardsSets(Vector v, Vector neighbours,
                                       ForestForwardsSet forestfw,
                                       PosetForwardsSet posetfw) {
        fwComparisonResult = "";
        // We assume that all relevant nodes are in the poset
        Hashtable forwardsSet = ForwardsSet.computeForwardsSet(v,
                neighbours);
        Hashtable ffw = null;
        if (forestfw != null) {
            ffw = forestfw.get();
        }
        Hashtable pfw = null;
        if (posetfw != null) {
            pfw = posetfw.get();
        }

        boolean correct = true;
        for (Enumeration e = forwardsSet.keys(); e.hasMoreElements(); ) {
            IFBase x = (IFBase) e.nextElement();
            Vector v1 = (Vector) forwardsSet.get(x);
            Vector v2 = null, v3 = null;
            if (ffw != null) {
                v2 = (Vector) ffw.get(x.getCoverId());
            }
            if (pfw != null) {
                v3 = (Vector) pfw.get(x.getCoverId());
            }

            if (v1 == null) {
                v1 = new Vector();
            }
            if (v2 == null) {
                v2 = new Vector();
            }
            if (v3 == null) {
                v3 = new Vector();
            }

            if (v1.size() > 0) {
                // for forest we allow false positives
                if (forestfw != null && !v2.containsAll(v1)) { //! (v1.containsAll(v2) && v2.containsAll(v1))) {
                    fwComparisonResult += "Forest forwards set error" + "\n";
                    fwComparisonResult += "Original: " + v1 + "\n";
                    fwComparisonResult += "Forest: " + v2 + "\n";
                    fwComparisonResult += "Filter: " + x.toString() + "\n";
                    boolean b = forestfw.getPoset().get().contains(x);
                    fwComparisonResult += "Filter is a root? " + b + "\n";
                    fwComparisonResult += "Tid? " + x.getTid() + "\n";
                    forwardsSet = ForwardsSet.computeForwardsSet(forestfw.
                            getPoset().
                            getAllNodes(), neighbours, x);
                    correct = false;
                }
                if (posetfw != null &&
                    !(v1.containsAll(v3) && v3.containsAll(v1))) {
                    fwComparisonResult += "first: " + v1.containsAll(v3) +
                            " second: " +
                            v3.containsAll(v1) + "\n";
                    fwComparisonResult += "Poset forwards set error" + "\n";
                    fwComparisonResult += "Original: " + v1 + "\n";
                    fwComparisonResult += "Poset: " + v3 + "\n";
                    fwComparisonResult += "Filter: " + x.toString() + "\n";
                    boolean b = posetfw.getPoset().isRootNode(x);
                    fwComparisonResult += "Filter is a root? " + b + "\n";
                    fwComparisonResult += "Tid? " + x.getTid() + "\n";

                    forwardsSet = ForwardsSet.computeForwardsSet(posetfw.
                            getPoset().
                            getAllNodes(), neighbours, x);

                    correct = false;
                }
            }
        }
        if (correct) {
            fwComparisonResult += "Forwards sets identical\n";
        } else {
            fwComparisonResult +=
                    "There were errors in forwards set computation\n";
        }

        return correct;
    }

    private void testAddRemoveMatching(Vector notifications, IFilterPoset pos,
                                       Vector temps2) {

        System.out.println("Testing with [" + pos.getClass().getName() + "]");
        // add/remove match test
        for (int p = 0; p < notifications.size(); p++) {
            Notification n = (Notification) notifications.elementAt(p);
            Vector sortMatchRes = pos.match(n);
            Vector bench = new Vector();

            for (int q = 0; q < temps2.size(); q++) {
                Template t = (Template) temps2.elementAt(q);
                if (t.match(null, n)) {
                    if (t.getTid() == null) {
                        if (!bench.contains(t.getInterface())) {
                            bench.add(t.getInterface());
                        }
                    } else {
                        for (int k = 0; k < t.getTid().size(); k++) {
                            Object obj = t.getTid().elementAt(k);
                            if (!bench.contains(obj)) {
                                bench.add(obj);
                            }
                        }
                    }
                }
            }

            if (!sortMatchRes.containsAll(bench) ||
                !bench.containsAll(sortMatchRes)) {
                System.out.println("Match error in add/remove test");
                System.out.println("Forest: " + sortMatchRes.size());
                System.out.println("Test:" + bench.size());

                Vector vv = (Vector) sortMatchRes.clone();
                Vector vv2 = (Vector) vv.clone();
                Vector vv3 = (Vector) bench.clone();
                vv2.removeAll(vv3);
                vv3.removeAll(vv);

                System.out.println("Forest/bench:" + vv2);
                System.out.println("Bench/forest:" + vv3);

                System.out.println("False positives");
                if (pos instanceof FilterForest) {
                    FilterForest ff = (FilterForest) pos;
                    for (int k = 0; k < vv2.size(); k++) {
                        Object obj = (Object) vv2.elementAt(k);
                        System.out.println("Excess iface: " + obj);
                        for (int q = 0; q < temps2.size(); q++) {
                            Template t = (Template) temps2.elementAt(q);
                            if (t.match(null, n)) {
                                if (t.getInterface().equals(obj)) {
                                    Node tl = (Node) ff.lookupNode(t);
                                    System.out.println(
                                            "Found missing template: " + tl);
                                    if (tl != null) {
                                        System.out.println("Template: " +
                                                tl.getData());
                                        System.out.println("Colors: " +
                                                tl.getData().getTid());
                                    }
                                }
                            }
                        }
                    }
                }

                System.out.println("False negatives");
                for (int k = 0; k < vv3.size(); k++) {
                    Object obj = (Object) vv3.elementAt(k);
                    System.out.println("Missing iface: " + obj);
                }
                if (pos instanceof FilterForest) {
                    FilterForest ff = (FilterForest) pos;
                    System.out.println(
                            "Testing if the missing matching templates " +
                            "are stored in the forest");

                    for (int k = 0; k < vv3.size(); k++) {
                        Object obj = (Object) vv3.elementAt(k);
                        for (int q = 0; q < temps2.size(); q++) {
                            Template t = (Template) temps2.elementAt(q);
                            if (Template.match(t, n.get().getTriplets())) {
                                if (t.getInterface().equals(obj)) {
                                    Node tl = (Node) ff.lookupNode(t);

                                    System.out.println(
                                            "Found missing template: " + tl);
                                    if (tl != null) {
                                        System.out.println("Template: " +
                                                tl.getData());
                                        System.out.println("Colors: " +
                                                tl.getData().getTid());
                                        if (!tl.getData().getTid().contains(obj)) {
                                            System.out.println(
                                                    "Fatal: color not present");
                                        } else {
                                            System.out.println("Color " + obj +
                                                    " is present. Why was it not matched?");
                                        }
                                        ff.printColorsFromNode(tl, null);
                                    }
                                }
                            }
                        }
                    }
                }
                //     testColorIndex();
                //     System.exit(0);
            }
        }
    }

    private void matchTest(Vector notifications, IFilterPoset pos,
                           int[] matchCheckTable, Vector[] ifaceTest) {

        int total_ops = 0;

        for (int p = 0; p < notifications.size(); p++) {
            Notification n = (Notification) notifications.elementAt(p);
            Vector sortMatchRes = pos.match(n);
            total_ops += pos.getOps(); // get matching ops
            if (sortMatchRes == null) {
                if (matchCheckTable[p] != 0) {
                    System.out.println("False negative in " +
                                       pos.getClass().getName());
                    System.exit(0);
                }
            } else {
                //System.out.println("Matched: " + sortMatchRes.size() + "/"+matchCheckTable[p]);
                Vector tempRes = (Vector) sortMatchRes.clone();
                int xp = sortMatchRes.size();
                sortMatchRes.removeAll(ifaceTest[p]);
                Vector missing = (Vector) ifaceTest[p].clone();
                missing.removeAll(tempRes);
                if (matchCheckTable[p] != xp || sortMatchRes.size() > 0 ||
                    missing.size() > 0) {
                    System.out.println("Interface count error in " +
                                       pos.getClass().getName());
                    System.out.println("Match check table:" + matchCheckTable[p]);
                    System.out.println("Return set size:" + xp);
                    System.out.println("Intersection of sets: " +
                                       sortMatchRes.size());

                    System.out.println("Missing: " + missing.size());
                    for (int k = 0; k < missing.size(); k++) {
                        Object obj = (Object) missing.elementAt(k);
                        System.out.println("Missing iface: " + obj);
                        if (pos instanceof FilterForest) {
                            ((FilterForest) pos).printAllIface(obj);

                            if (((FilterForest) pos).testMatch(n, obj)) {
                                System.out.println("Templates did match");
                            } else {
                                System.out.println("Templates did not match");
                            }
                        }
                    }
                    System.exit(0);
                }

            }
        }
        System.out.println("Total matching ops: " + total_ops + " for " +
                           pos.getClass().getName());
        System.out.println("*****");
    }



    public void resetBuffer() {
        bufferRoots.clear();
    }

    public void testElimination(Vector temps, Vector temps2) {
        Vector v = new Vector();

        for (int i = 0; i < temps.size(); i++) {
            IFBase t = (IFBase) temps.elementAt(i);

            boolean insert = true;
            Vector delList = new Vector();
            for (int j = 0; j < v.size(); j++) {
                IFBase t2 = (IFBase) v.elementAt(j);
                if (((Template) t).getInterface().equals((((Template) t2).
                        getInterface()))) {
                    if (t2.covers(t)) {
                        insert = false;
                    } else if (t.covers(t2)) {
                        delList.add(t2);
                    }
                }
            }
            v.removeAll(delList);
            if (insert) {
                v.add(t);
            }
        }

        for (int i = 0; i < temps.size(); i++) {
            IFBase t = (IFBase) temps.elementAt(i);
            v.remove(t);

            IFBase t2 = (IFBase) temps2.elementAt(i);
            boolean insert = true;
            Vector delList = new Vector();
            for (int j = 0; j < v.size(); j++) {
                IFBase t3 = (IFBase) v.elementAt(j);

                if (((Template) t2).getInterface().equals((((Template) t3).
                        getInterface()))) {

                    if (t3.covers(t2)) {
                        insert = false;
                    } else if (t2.covers(t3)) {
                        delList.add(t3);
                    }
                }
            }

            v.removeAll(delList);
            if (insert) {
                v.add(t2);
            }
        }
        Vector notcov = MergingService.get().getNonCovered(v);
        System.out.println("** Temp root size after add/del: " + notcov.size());
        System.out.println("** Temp Set size: " + v.size());
    }

    /**
     * Implements a buffer layer that protects from illegal adds.
     * Ensures that there is no need for color-elimination.
     * @param t IFBase
     * @param color Object
     * @return boolean
     */
    public boolean bufferLayerAdd(IFBase t, IFilterPoset pos, Object color) {
        HashSet set = (HashSet) bufferRoots.get(color);

        if (set == null) {
            set = new HashSet();
            bufferRoots.put(color, set);
        }
        boolean ret = true;

        //  This set is uncomparable
        Iterator i = set.iterator();
        HashSet removeList = new HashSet();

        try {
            while (i.hasNext()) {
                IFBase t2 = (IFBase) i.next();
                // remove stale entries
                if (!pos.removable(t2, color)) {
                    removeList.add(t2);
                    continue;
                } else if (t2.covers(t)) {
                    ret = false;
                    break;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        if (ret) {
            set.add(t);
        }
        set.removeAll(removeList);
        return ret;
    }

    /**
     * Implements a buffer layer that protects from illegal adds.
     * Ensures that there is no need for color-elimination.
     * @param t IFBase
     * @param color Object
     * @return boolean
     */
    public boolean bufferLayerAddAfterDel(IFBase t, IFilterPoset pos,
                                          Object color) {
        HashSet set = (HashSet) bufferRoots.get(color);

        if (set == null) {
            set = new HashSet();
            bufferRoots.put(color, set);
        }
        boolean ret = true;

        //  This set is uncomparable
        Iterator i = set.iterator();
        HashSet removeList = new HashSet();

        try {
            while (i.hasNext()) {
                IFBase t2 = (IFBase) i.next();
                // remove stale entries
                if (!pos.removable(t2, color)) {
                    removeList.add(t2);
                    continue;
                } else if (t2.covers(t)) {
                    ret = false;
                    break;
                } else if (t.covers(t2)) {
                    ret = false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        if (ret) {
            set.add(t);
        }
        set.removeAll(removeList);
        return ret;
    }
}
