package fuegocore.notify.core.filter.poset.advanced;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.poset.*;
import java.util.*;
import fuegocore.notify.core.filter.IFBase;
import fuegocore.notify.core.filter.merge.MergingService;

/**
 * Root merger algorithm.
 * - note: successor sets may have non root entries.
 *
 * Sasu Tarkoma, 2005
 */
public class RootMerger
    extends FilterForest {
  boolean mDebug = false;
  boolean mTest = false;

  /* Vector containing merged nodes */
  Vector merged = new Vector();

  /* Components of a merger */
  Hashtable components = new Hashtable();

  /* Hashtable for immediate mSuccessors to merged nodes */
  Hashtable mSuccessors = new Hashtable();

  /* Hashtable of predecessor nodes */
  Hashtable mPreds = new Hashtable();

  public RootMerger() {
  }

  public void testRootCover() {
    Vector v = get();
    Vector rootSet = getMerged();

    System.out.println("Testing root cover");

    for (int i = 0; i < v.size(); i++) {
      IFBase f = (IFBase) v.elementAt(i);
      if (merged.contains(f) && !rootSet.contains(f)) {
        System.out.println("Fatal: merger filter not in merged set");
      }
      if (!rootSet.contains(f)) {
        if (mPreds.get(f) == null || ( (Vector) mPreds.get(f)).size() == 0) {
          System.out.println(
              "Found a root filter not contained in merged or covered by a merger!");
          System.out.println(f);
        }
      }
    }
  }

  public void testRootCover(Vector nonc) {
    if (nonc == null) {
      return;
    }

    System.out.println("Testing root cover with input size " + nonc.size());
    for (int i = 0; i < nonc.size(); i++) {
      IFBase f = (IFBase) nonc.elementAt(i);
      if (merged.contains(f)) {
        System.out.println("Fatal: noncovered in merged set");
      }
      if (mPreds.get(f) != null && ( (Vector) mPreds.get(f)).size() > 0) {
        System.out.println("Input filter: " + f);
        Vector v = (Vector) mPreds.get(f);
        if (v == null) {
          System.out.println("No mergers! should be in the root set!");
        }
        else {
          System.out.println("Mergers: " + v.size());
        }
        for (int j = 0; j < v.size(); j++) {
          IFBase z = (IFBase) v.elementAt(j);
          if (!merged.contains(z)) {
            System.out.println("Found an obsolete reference!");
          }
          if (!z.covers(f)) {
            System.out.println("Fatal merger did not cover filter");
            System.out.println("Merger:" + z);
          }
          else {
            System.out.println("Merger covered filter ok");
          }

          Vector comps = (Vector) components.get(z);

          System.out.println("Components size: " + comps.size());
          System.out.println("Components: " + comps);
          if (comps != null) {
            System.out.println("Components contains node: " + comps.contains(f));
          }

          Vector s = (Vector) mSuccessors.get(z);
          if (s != null && !s.contains(f)) {
            System.out.println(
                "Find mismatch in predecessor / successor mapping!");
          }
        }
      }
      if (components.get(f) != null) {
        System.out.println("Filter has components");
        if (mSuccessors.get(f) != null) {
          System.out.println("Filter has successors");
        }
      }
    }
  }

  public void testPredSet(IFBase f) {
    Vector v = (Vector) mPreds.get(f);
    if (v == null) {
      return;
    }
    for (int j = 0; j < v.size(); j++) {
      IFBase z = (IFBase) v.elementAt(j);
      if (!z.covers(f)) {
        System.out.println("Fatal merger did not cover filter");
        System.out.println("Input:" + f);
        System.out.println("Merger:" + z);

        Vector comps = (Vector) components.get(z);

        System.out.println("Components size: " + comps.size());
        System.out.println("Components: " + comps);
        if (comps != null) {
          System.out.println("Components contains node: " + comps.contains(f));
        }

        System.exit(0);
      }
    }
  }

  public void addToComponents(IFBase m, IFBase x) {
    Vector v = (Vector) components.get(m);
    if (v == null) {
      v = new Vector();
      components.put(m, v);
    }
    if (!v.contains(x)) {
      v.add(x);
    }
  }

  public void delFromComponents(IFBase m, IFBase x) {
    Vector v = (Vector) components.get(m);
    if (v != null) {
      v.remove(x);
      if (v.size() == 0) {
        components.remove(v);
        v = null;
      }
    }
  }

  public void addToCovered(IFBase m, IFBase x) {
    Vector v = (Vector) mSuccessors.get(m);
    if (v == null) {
      v = new Vector();
      mSuccessors.put(m, v);
    }
    if (!v.contains(x)) {
      v.add(x);
    }
  }

  public void delFromCovered(IFBase m, IFBase x) {
    Vector v = (Vector) mSuccessors.get(m);
    if (v != null) {
      v.remove(x);
      if (v.size() == 0) {
        mSuccessors.remove(v);
        v = null;
      }
    }
  }

  public void delFromPreds(IFBase m, IFBase x) {
    Vector v = (Vector) mPreds.get(m);
    if (v != null) {
      v.remove(x);
      if (v.size() == 0) {
        mPreds.remove(v);
        v = null;
      }
    }
  }

  public void connect(IFBase m, IFBase z) {
    addToCovered(m, z);
    addToPred(z, m);
  }

  public void disconnect(IFBase m, IFBase z) {
    delFromCovered(m, z);
    delFromPreds(z, m);
  }

  public void addToPred(IFBase m, IFBase x) {
    Vector v = (Vector) mPreds.get(m);
    if (v == null) {
      v = new Vector();
      mPreds.put(m, v);
    }
    if (!v.contains(x)) {
      v.add(x);
    }
  }

  /**
   * Returns true if the given filter in the structure is covered by a merger with the same color
   * @param f IFBase
   * @return boolean
   */
  public boolean coveredByMerger(IFBase f) {
    Vector v = (Vector) mPreds.get(f);
    if (v != null) {
      for (int i = 0; i < v.size(); i++) {
        IFBase x = (IFBase) v.elementAt(i);
        if (forwardEquivalent(x, f)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Replaces the first merger with the second.
   * @param merger IFBase the first merger
   * @param newMerger IFBase the second merger
   * @param inheritComps boolean true if components are preserved
   */
  public void replaceMerger(IFBase merger, IFBase newMerger,
                            boolean inheritComps) {

    if (!merged.contains(merger)) {
      System.out.println("Fatal error in replaceMerger: merger not found:" +
                         merger);
      System.exit(0);
    }

    merged.remove(merger);

    // Components are marked as covered
    Vector comps = (Vector) components.remove(merger);
    if (mDebug) {
      System.out.println("Components: " + comps);
    }

    // Covered sets
    if (comps != null) {
      if (inheritComps) {
        Vector comps2 = (Vector) components.get(newMerger);
        if (comps2 == null) {
          comps2 = new Vector();
          components.put(newMerger, comps2);
        }
        comps2.addAll(comps);
      }
      for (int i = 0; i < comps.size(); i++) {
        IFBase x = (IFBase) comps.elementAt(i);
        disconnect(merger, x);
        connect(newMerger, x);
        if (test) {
          if (!newMerger.covers(x)) {
            System.out.println("New merger does not cover its component!");
            System.out.println("Merger: " + newMerger);
            System.out.println("Component: " + x);
            System.out.println("Old merger: " + merger);
            System.exit(0);
          }
        }
      }
    }
    else {
      System.out.println("Warning: components were null for merger: " + merger);
    }

    Vector covered = (Vector) mSuccessors.remove(merger);

    // Covered sets
    if (covered != null) {
      for (int i = 0; i < covered.size(); i++) {
        IFBase x = (IFBase) covered.elementAt(i);
        disconnect(merger, x);
        connect(newMerger, x);
        if (test) {
          if (!newMerger.covers(x)) {
            System.out.println("New merger does not cover an element!");
            System.out.println("Merger: " + newMerger);
            System.out.println("Component: " + x);
            System.out.println("Old merger: " + merger);
            System.exit(0);
          }
        }

      }
    }
    else {
      System.out.println("Warning: mSuccessors were null for merger: " + merger);
    }
    comps.clear();
    covered.clear();
  }

  public void removeMerger(IFBase merger) {

    if (!merged.contains(merger)) {
      System.out.println("Fatal error in removeMerger: merger not found:" +
                         merger);
      System.exit(0);
    }

    merged.remove(merger);

    // Components are marked as covered
    Vector comps = (Vector) components.remove(merger);

    // Covered sets
    if (comps != null) {
      for (int i = 0; i < comps.size(); i++) {
        IFBase x = (IFBase) comps.elementAt(i);
        disconnect(merger, x);
      }
    }
    else {
      System.out.println("Warning: components were null for merger: " + merger);
    }

    Vector covered = (Vector) mSuccessors.remove(merger);
    // Covered sets
    if (covered != null) {
      for (int i = 0; i < covered.size(); i++) {
        IFBase x = (IFBase) covered.elementAt(i);
        disconnect(merger, x);
      }
    }
    else {
      System.out.println("Warning: mSuccessors were null for merger: " + merger);
    }
    comps.clear();
    covered.clear();
  }

  /**
   * Returns the merged root
   * @return Vector
   */
  public Vector getMerged() {
    Vector ret = new Vector();
    ret.addAll(merged);

    if (mDebug) {
      System.out.println("Merge set size: " + merged.size());
    }

    Vector root = this.get();

    for (int i = 0; i < root.size(); i++) {
      IFBase x = (IFBase) root.elementAt(i);
      if (mPreds.get(x) == null || ( (Vector) mPreds.get(x)).size() == 0) {
        ret.add(x);
      }
    }

    return ret;
  }

  public boolean merge_set(Vector S, IFBase f) {

    // scan input set for merging
    for (int i = 0; i < S.size(); i++) {
      Object o = (Object) S.elementAt(i);
      IFBase x = null;
      if (o instanceof Node) {
        x = (IFBase) ( (Node) o).getData();
      }
      else {
        x = (IFBase) o;
      }

      if (x == f) {
        continue; // skip this node
      }

      IFBase z = (IFBase) x.mergeObject(f);

      if (z != null) {
        if (mDebug) {
          System.out.println("Merge was succesful. Already merged? " +
                             merged.contains(x));
        }

        // Merger must cover its components
        addToComponents(z, f);
        connect(z, f);
        if (test) {
          if (!z.covers(f)) {
            System.out.println(
                "Fatal error: merged node did not cover its component");
            System.out.println("Merged: " + z);
            System.out.println("Component: " + f);
            System.exit(0);
          }
        }

        if (merged.contains(x)) {
          replaceMerger(x, z, true); // inherit components
        }
        else { // add initial merged object
          addToComponents(z, x);
          connect(z, x);
          if (test) {
            if (!z.covers(x)) {
              System.out.println(
                  "Fatal error: merged node did not cover its component");
              System.out.println("Merged: " + z);
              System.out.println("Component: " + x);
              System.exit(0);
            }
          }

        }

        // scan merge set for cover
        // merge succeeded
        Vector mt = (Vector) merged.clone();

        for (int j = 0; j < mt.size(); j++) {

          IFBase merger = (IFBase) mt.elementAt(j);
          incOps();
          if (z.covers(merger)) {
            if (mDebug) {
              System.out.println(
                  "Merge-set: Replacing existing merger due to cover (no comps)");
              System.out.println("Merger: " + merger);
              System.out.println("Current merger: " + z);
              System.out.println("-------------------");

            }
            // Merger was covered so we remove it.
            replaceMerger(merger, z, false); // do not inherit components
          }
        }

        // Scan root set for cover
        for (int j = 0; j < this.get().size(); j++) {
          Object obj = this.get().elementAt(j);
          IFBase filter = null;
          if (obj instanceof Node) {
            filter = (IFBase) ( (Node) obj).getData();
          }
          else {
            filter = (IFBase) obj;
          }
          if (filter == f) {
            continue; // skip this node
          }

          // We only check those nodes that are not covered by the merger.
          Vector CV = (Vector) mSuccessors.get(z);
          Vector CO = (Vector) components.get(z);

          if (CV != null && CV.contains(filter)) {
            ;
          }
          else if (CO != null && CO.contains(filter)) {
            ;
          }
          else {
            incOps();
            if (z.covers(filter)) {
              if (mDebug) {
                System.out.println("Merge-set: Node covered by merger");
                System.out.println("Filter: " + filter);
              }
              connect(z, filter);
            }
          }
        }

        // Finally, insert merged node to the set of mergers
        merged.add(z);
        return true;
      }
    }
    return false;
  }

  public void root_set_add(IFBase n) {
    boolean f = false;

    if (mDebug) {
      System.out.println("***************** add merge *************** ");
      System.out.println("Added filter: " + n);
      System.out.println("****************");
    }

    Vector mt = (Vector) merged.clone();

    for (int j = 0; j < mt.size(); j++) {
      IFBase merger = (IFBase) mt.elementAt(j);
      incOps();
      if (merger.covers(n)) {
        if (mDebug) {
          System.out.println("Root_set_add:  Existing merger covers this node");
        }
        connect(merger, n);
        f = true;
      }
      else if (!f) {
        incOps();
        if (n.covers(merger)) {
          if (mDebug) {
            System.out.println(
                "Root_set_add: Replacing existing merger due to cover");
          }
          removeMerger(merger);
        }
      }
    }

    if (f) {
      return;
    }

    if (mDebug) {
      System.out.println("Root_set_add: Attempting to merge with merged set");
    }
    f = merge_set(merged, n);

    if (f) {
      if (test) {
        testPredSet(n);
      }
      return;
    }

    if (mDebug) {
      System.out.println(
          "Root_set_add: Attempting to merge with non covered root set");
    }

    // t is the set of non-covered nodes (also noncomponents)
    Vector t = new Vector();

    for (int j = 0; j < this.get().size(); j++) {
      Object obj = this.get().elementAt(j);
      IFBase filter = null;
      if (obj instanceof Node) {
        filter = (IFBase) ( (Node) obj).getData();
      }
      else {
        filter = (IFBase) obj;
      }
      if (filter == n) {
        continue; // skip this node
      }
      Vector v = (Vector) mPreds.get(filter);
      if (v == null || v.size() == 0) {
        t.add(filter);
      }
    }
    merge_set(t, n);

    if (test) {
      testPredSet(n);
    }
  }

  public void testSets() {
    Enumeration e = mPreds.elements();
    while (e.hasMoreElements()) {
      Vector v = (Vector) e.nextElement();
      for (int i = 0; i < v.size(); i++) {
        IFBase x = (IFBase) v.elementAt(i);
        if (!merged.contains(x)) {
          System.out.println("Fatal: merged did not contain element in pred");
          System.exit(0);
        }
      }
    }

    for (int i = 0; i < merged.size(); i++) {
      IFBase m = (IFBase) merged.elementAt(i);

      Vector succ = (Vector) mSuccessors.get(m);
      if (succ != null) {
        for (int j = 0; j < succ.size(); j++) {
          IFBase x = (IFBase) succ.elementAt(j);
          if (!m.covers(x)) {
            System.out.println("Fatal: merger did not cover merged element");
            System.out.println("Merged:" + m);
            System.out.println("Filter:" + x);

            Vector comps = (Vector) components.get(m);

            System.out.println("Components size: " + comps.size());
            System.out.println("Components: " + comps);

            System.exit(0);
          }
        }
      }
      Vector comp = (Vector) components.get(m);
      if (comp != null) {
        if (comp.size() == 1) {
          System.out.println("Component size was 1");
          System.out.println("Merged: " + m);
          System.out.println("Comps: " + comp);
          System.exit(0);
        }
        for (int j = 0; j < comp.size(); j++) {
          IFBase x = (IFBase) comp.elementAt(j);
          if (!m.covers(x)) {
            System.out.println("Fatal: merger did not contain merged component");
            System.exit(0);
          }
        }
      }
      // check completeness
      for (int j = 0; j < this.get().size(); j++) {
        IFBase x = (IFBase)this.get().elementAt(j);
        if (m.covers(x) && !succ.contains(x)) {
          System.out.println(
              "Found node covered by merger but no in successor set");
          System.exit(0);
        }

      }

    }

    if (!MergingService.get().covers(this.getMerged(), this.get())) {
      System.out.println(
          "RootMerger: Fatal merged root does not cover non-merged root");
      System.exit(0);
    }
    // Note successor sets may have unnecessary elements when
    // nodes are pushed down in the structure, i.e., moved from
    // being roots.
    /*
         e = mSuccessors.elements();
         while (e.hasMoreElements()) {
      Vector v = (Vector) e.nextElement();
      for (int i = 0; i < v.size(); i++) {
        IFBase x = (IFBase) v.elementAt(i);
        if (!get().contains(x)) {
          System.out.println(
              "Fatal: root did not contain element in successor set");
          System.exit(0);
        }
      }
         }
     */
  }

  public void resetStructures(IFBase x) {
    if (mPreds.get(x) != null) {
      Vector v = (Vector) mPreds.remove(x);
      if (v != null) {
        v.clear();
      }
    }
    if (mSuccessors.get(x) != null) {
      Vector v = (Vector) mSuccessors.remove(x);
      if (v != null) {
        v.clear();
      }
    }
    if (components.get(x) != null) {
      Vector v = (Vector) components.remove(x);
      if (v != null) {
        v.clear();
      }
    }
  }

  public synchronized int remove(IFBase s, Object color) throws PosetException {
    if (mDebug) {
      System.out.println("***************** del merge *************** ");
    }

    Vector oldRoot = this.get();

    int ret = super.remove(s, color);

    // We get the new root nodes by set difference
    Vector newRoot = (Vector)this.get().clone();
    newRoot.removeAll(oldRoot);

    if (ret == this.POSET_ROOT) {
      // possible unmerge
      // 1. If s is a in a merger
      Vector pred = (Vector) mPreds.get(s);
      Vector removeSet = new Vector();
      if (pred != null) {
        Vector p = (Vector) pred.clone();
        for (int i = 0; i < p.size(); i++) {
          IFBase m = (IFBase) p.elementAt(i);
          Vector comps = (Vector) components.get(m);
          if (comps != null && comps.contains(s)) {
            // Add the components of the merger to the process set
//            newRoot.addAll(comps);
            // m is the merger that is removed.
            if (mDebug) {
              System.out.println("removeMerger called in del");
            }
            removeSet.add(m);
          }
          disconnect(m, s);
        }
        pred.clear();

        // Should be only one
        if (removeSet.size() > 1) {
          System.out.println(
              "Fatal: remove set in del contained more than one element");
          System.exit(0);
        }

        for (int i = 0; i < removeSet.size(); i++) {
          IFBase m = (IFBase) removeSet.elementAt(i);
          removeMerger(m);
          if (test) {
            testPredSet(m);
          }
        }
      }

      // 2. Process components and children that are now roots
      if (newRoot.size() > 0) {
        // The predecessor and successor lists are already updated for the nodes
        // They are updated in add. Now, we attempt to merge nodes that
        // are not totally covered by mergers. (i.e. from the same ifaces)
        for (int i = 0; i < newRoot.size(); i++) {
          IFBase x = (IFBase) newRoot.elementAt(i);
          resetStructures(x);
          root_set_add(x);
        }
      }
    }

    if (mDebug) {
      System.out.println("***************** end merge *************** ");
    }

    if (mTest) {
      testSets();
    }

    return ret;
  }

  public boolean forwardEquivalent(IFBase x, IFBase y) {
    if (x.getTid().containsAll(y.getTid()) &&
        y.getTid().containsAll(x.getTid())) {
      return true;
    }
    return false;
  }

  public int add(IFBase f, Object s) throws PosetException {
    int ret = super.add(f, s);
    if (ret == POSET_ROOT) {
      root_set_add(f);

      if (mTest) {
        testSets();
      }

      return ret;
    }
    return ret;
  }

}
