/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.filter.test;

import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.poset.*;
import fuegocore.notify.util.*;

/**
 * Simple suite of tests for the filter service.
 */
public class FilterTestSuite {
    Vector components;

    public FilterTestSuite() {
        components = new Vector();
    }

    public static void main(String[] args) {
        FilterTestSuite fts = new FilterTestSuite();
        fts.add(new Template());
        fts.runTests();
    }

    /** Add a component to be tested
     *  @param its the testable component
     */
    public void add(ITestComponent its) {
        components.add(its);
    }

    public void runTests() {
        boolean test = true;
        int failureCount = 0;

        for (int i = 0; i < components.size(); i++) {
            ITestComponent its = (ITestComponent) components.elementAt(i);
            if (its == null) {
                System.out.println("Component was null");
                test = false;
            } else {
                System.out.println("Testing component: " +
                                   its.getClass().getName());
                System.out.println("Test " + i + "/" + components.size());
                test = its.runTest();
                if (!test) {
                    failureCount++;
                    break;
                }
            }
            System.out.println("--------------------------------------");
        }
        if (failureCount != 0) {
            System.out.println(failureCount + " tests failed");
        } else {
            System.out.println("All tests ok");
        }
    }
}
