/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.serialization;

import fuegocore.notify.core.filter.Filter;
import fuegocore.util.xas.ContentCodecFactory;
import fuegocore.util.xas.ContentEncoder;
import fuegocore.util.xas.ContentDecoder;
import fuegocore.util.xas.Qname;
import fuegocore.util.xas.XasUtil;

/**
 * A typed content codec factory for the event service.  This {@link
 * ContentCodecFactory} builds encoders and decoders recognizing the
 * various types of objects used in event service messages.  The XML
 * names given for the types all have the namespace {@link
 * Filter#EVENT_NAMESPACE}.  Relevant type mappings are registered
 * during initialization of the class.
 */
public class EventCodecFactory extends ContentCodecFactory {

    private ContentCodecFactory factory;

    static {
        try {
            ContentCodecFactory.addTypeMapping
                    (Class.forName("fuegocore.notify.core.EventTriplet"),
                     new Qname(Filter.EVENT_NAMESPACE, "triplet"));
            ContentCodecFactory.addTypeMapping
                    (Class.forName("fuegocore.notify.core.NotificationValue"),
                     new Qname(Filter.EVENT_NAMESPACE, "value"));
            ContentCodecFactory.addTypeMapping
                    (Class.forName("fuegocore.notify.core.Notification"),
                     new Qname(Filter.EVENT_NAMESPACE, "notification"));
            ContentCodecFactory.addTypeMapping
                    (Class.forName("fuegocore.notify.core.ITemplate"),
                     new Qname(Filter.EVENT_NAMESPACE, "template"));
            ContentCodecFactory.addTypeMapping
                    (Class.forName("fuegocore.notify.core.filter.Filter"),
                     new Qname(Filter.EVENT_NAMESPACE, "filter"));
            ContentCodecFactory.addTypeMapping
                    (Class.forName(
                    "fuegocore.notify.core.session.SessionParams"),
                     new Qname(Filter.EVENT_NAMESPACE, "session"));
            ContentCodecFactory.addTypeMapping
                    (Class.forName(
                    "fuegocore.notify.core.channel.EventChannelParams"),
                     new Qname(Filter.EVENT_NAMESPACE, "channel"));
            /*
                  ContentCodecFactory.addTypeMapping
               (Class.forName("fuegocore.notify.core.channel.ECRRerouteInfo"),
                new Qname(Filter.EVENT_NAMESPACE, "reroute"));
             */
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public EventCodecFactory() {
        this(null);
    }

    public EventCodecFactory(ContentCodecFactory factory) {
        this.factory = factory;
    }

    public void register() {
        ContentCodecFactory.addNamespace(Filter.EVENT_NAMESPACE, "ev");
    }

    public ContentEncoder getChainedEncoder(ContentEncoder chain) {
        if (factory != null) {
            chain = factory.getChainedEncoder(chain);
        }
        return new EventEncoder(chain);
    }

    public ContentDecoder getChainedDecoder(ContentDecoder chain) {
        if (factory != null) {
            chain = factory.getChainedDecoder(chain);
        }
        return new EventDecoder(chain);
    }

}
