/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.serialization;

import java.io.*;
import java.util.*;

import fuegocore.util.xas.*;

/**
 * Interface for multiplexing and demultiplexing Templates
 */
public interface IEventMultiplexer {

    /**
     * Demultiplex an object from the given EventSequence
     * @param ses the event sequence
     * @return Object the demultiplexed object
     */
    public Object fromEventSequence(EventSequence ses);

    /**
     * Demultiplex a Vector of Templates from the given EventSequence
     * @param ses the event sequence
     * @throws IOException
     * @return Vector the demultiplexed vector of templates
     */
    public Vector demultiplexEventSequence(EventSequence ses) throws
            IOException;

    /**
     * Creates an EventSequence representation of a vector of templates
     * @param ser TypedXmlSerializer
     * @param templates Vector
     * @throws IOException
     */
    public void multiplexEventSequence(TypedXmlSerializer ser,
                                       Vector templates) throws
            IOException;

    /**
     * Creates a string representation of a vector of templates
     * @param templates the input templates for multiplexing
     * @return String the multiplexed string
     */
    public String multiplex(Vector templates);

    /**
     *  Creates a vector representation of multiple templates in a string
     * @param str the input string for parsing
     * @return Vector the parsed Template objects
     */
    public Vector demultiplex(String str);
}
