/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.server.handlers;

import fuegocore.notify.core.Template;
import fuegocore.message.service.MessageService;
import fuegocore.notify.core.server.ConnectionObject;
import java.net.URL;
import fuegocore.notify.core.channel.EventChannelServer;
import fuegocore.message.service.NameValuePair;
import fuegocore.message.MessageTransportService;
import fuegocore.notify.core.channel.EventChannel;
import java.util.Hashtable;
import fuegocore.notify.core.channel.ECRRerouteInfo;
import fuegocore.message.service.Message;
import fuegocore.notify.util.Log;
import java.util.Enumeration;
import fuegocore.message.service.RpcCall;
import java.util.Vector;
import fuegocore.message.protocol.MessageProtocol;
import fuegocore.notify.core.server.EventServer;
import fuegocore.notify.core.server.engine.IRoutingEngine;

/**
 * Access server handover task. This is from Prototype I (2003) and
 * should be integrated with the current codebase (2004).
 */
public class AccessServerHandoverTask {
    EventServer es;

    public AccessServerHandoverTask(EventServer _es) {
        es = _es;
    }

    /** Initiates the first part of a handover. This does not perform
     *  any modifications to any state.
     *
     * @param new_as  new access server
     * @param id      session identifier
     * @param device  device
     * @return the subscription templates
     */
    /*
    public String startHandoverPhase1(String new_as, String id,
                                      String device) {
        String iface = fuegocore.notify.profile.CommProfile.event_server_id;
        String sessiondata = "";

        Log.write("Access server: starting handover 1:" + new_as);

        // Currently we use id only
        ConnectionObject co = es.getConnection(id);
        if (co == null) {
            System.out.println("Connections: " + es.getConnectionsList());
            System.out.println("Tried to find: " + id);
            return "invalid connection";
        }

        // If the connection is not paused, we pause it now
        co.getMTP().pause();

        Log.write("-----------------------------------------");
        Log.write("After pausing connection: " + co.getMessages().size());
        Log.write("-----------------------------------------");

        // Locate user session
        // Remove templates for this interface (removes all filters also)
        Vector templates = FilterService.get().getTemplates(id);

        // Add templates to result string
        if (templates != null) {
            //System.out.println("Packaging templates: " + templates.size());
            for (int i = 0; i < templates.size(); i++) {
                Template te = (Template) templates.elementAt(i);
                if (te != null) {
                    sessiondata += te.toString();
                    //if (i < templates.size() - 1) {
                    sessiondata += "<_dlm/>";
                    //}
                } // if (te..
                // sessiondata += "<_dlm/>";
            } // for
        } // if (templates..
        // end connection if it is active
        return sessiondata;
    }
/*
    /** Complete the handover by contacting the event channel (if necessary)
     *  This call assumes that the subscriptions have been transferred to the
     *  new server.
     *
     * @param new_as new access server
     * @param id         session identifier
     * @param device     device
     * @param cover    are subscription covered by existing subs at the new server
     * @return the session (notifications)
     */
    /*
    public String startHandoverPhase2(String new_as, String id, String device,
                                      boolean cover) {
        String iface = fuegocore.notify.profile.CommProfile.event_server_id;

        Log.write("Access server: starting handover 2:" + new_as);

        // Currently we use id only
        ConnectionObject co = es.getConnection(id);
        if (co == null) {
            return "invalid connection";
        }

        // Determine the set of filters and how they affect current subscriptions
        // This is done as follows:
        // for the given id, each filter that is not subscribed by others is identifier
        // a unsubscribe flag is set if there are no subscribers
        // a modify request is sent to the set of event channels
        // the event channel either adds a new forward end point (if existing subscribers)
        // or modifies the existing subscriber for the filter
        // after this process we could send a "generalized" filter

        // We get event channel specific filters
        Hashtable _hsh = FilterService.get().getRSerouteInfo(id);
        // Then we partition them based on the servers, this reduces
        // extra messaging between servers

        // We update the event channel status only if it is not covered or
        // we the remove set is non-empty
        if (_hsh != null) {

            Enumeration aa = _hsh.keys(); // URL to event channels
            for (Enumeration k = _hsh.elements(); k.hasMoreElements(); ) {

                Hashtable hsh = (Hashtable) k.nextElement();

                if (hsh == null) {
                    return "event channel failure";
                }

                // Get first event channel name, we assume they must be on the same
                // server since the address is the same
                if (hsh.size() < 1) {
                    continue;
                }
                boolean updateNecessary = false;
                ECRRerouteInfo ecr = null;
                Enumeration enu = hsh.elements();

                for (; enu.hasMoreElements(); ) {
                    ecr = (ECRRerouteInfo) enu.nextElement();
                    if (ecr == null) {
                        continue;
                    }
                    if (ecr.getRemoved() != null && ecr.getRemoved().size() > 0) {
                        updateNecessary = true;
                    }
                }

                // If the templates are already covered and nothing needs to be removed
                // no update of this server is required.
                if (cover && !updateNecessary) {
                    System.out.println("SKIPPING EVENT CHANNEL UPDATE");
                    continue;
                }

                Log.write("Doing invocation to EC");

                // RPC invocation to event channel
                String channel = ecr.getChannel();

                // CHECK IF LOCAL (FAST)
                EventChannelServer ecs = EventChannelServer.get();
                EventChannel ec = ecs.lookup(channel);

                if (ec != null) {
                    boolean ret = false;

                    Log.write("Sending to ec: " + ECRRerouteInfo.multiplex(hsh) +
                              "for interface:" + iface + " new as:" + new_as);

                    // We offer two kinds of event channel updates:
                    // Merged sets and non-merged sets
                    if (!fuegocore.notify.profile.UserProfile.mergeEC) {
                        ret = ecs.handover(ecr.getChannel(), new_as.trim(),
                                           ECRRerouteInfo.multiplex(hsh),
                                           iface.trim());
                    } else {
                        // With merged mode we do this a bit differently
                        // We send the current event channel status (merged) to the event channel
                        // We could also send the delta (TBD)
                        // So the removeset intepretation is different for merging mode
                        // In addition, we send the subscriptions for the new interface also
                        // as merged. This commits the changes as well.

                        for (Enumeration kl = hsh.elements();
                                              kl.hasMoreElements(); ) {
                            ECRRerouteInfo ecr3 = (ECRRerouteInfo) kl.
                                                  nextElement();

                            Log.write("** HA: Remove size: " + ecr3.getRemoved() +
                                      " for " +
                                      ecr3.getChannel());

                            ecr3.setRemoved(es.findRemoveSetForMerge(id,
                                    ecr3.getChannel()));

                            Log.write("** HA: new remove size: " +
                                      ecr3.getRemoved() +
                                      " for " +
                                      ecr3.getChannel());
                        }

                        // actually we could skip the sending of the forward set when cover==true
                        ret = ecs.handover(ecr.getChannel(), new_as.trim(),
                                           ECRRerouteInfo.multiplexMerge(hsh),
                                           iface.trim());
                    }
                    if (!ret) {
                        // We roll back the changes that were made (this does not lose concurrently
                        // made modifications). The FilterService structure has not yet
                        // been updated.
                        if (fuegocore.notify.profile.UserProfile.mergeEC) {
                            Log.write("Rolling back");
                            for (Enumeration kl = hsh.elements();
                                                  kl.hasMoreElements(); ) {
                                ECRRerouteInfo ecr3 = (ECRRerouteInfo) kl.
                                        nextElement();
                                es.rollBackMerge(ecr3.getChannel());
                            }
                        }

                        return "event channel failure";
                    }
                    continue;
                }

                // RPC interaction

                URL url = (URL) es.lookup(channel);
                if (url == null) {
                    return "event channel failure";
                }

                // Make SOAP RPC to event channel
                try {
                    NameValuePair[] parameters = new NameValuePair[] {
                                                 new NameValuePair("arg1",
                            "String"),
                                                 new NameValuePair("arg2",
                            "String"),
                                                 new NameValuePair("arg3",
                            "String"),
                                                 new NameValuePair("arg4",
                            "String")
                    };

                    String sets = "";
                    if (!fuegocore.notify.profile.UserProfile.mergeEC) {
                        sets = ECRRerouteInfo.multiplexMerge(hsh);
                    } else {
                        // With merged mode we do this a bit differently
                        // We send the current event channel status (merged) to the event channel
                        // We could also send the delta (TBD)
                        // So the removeset intepretation is different for merging mode
                        // In addition, we send the subscriptions for the new interface also
                        // as merged. This commits the changes as well.

                        for (Enumeration kl = hsh.elements();
                                              kl.hasMoreElements(); ) {
                            ECRRerouteInfo ecr3 = (ECRRerouteInfo) kl.
                                                  nextElement();

                            ecr3.setRemoved(es.findRemoveSetForMerge(id,
                                    ecr3.getChannel()));

                            // update proxy object
                            EventChannel proxy = ecs.lookupProxy(ecr3.
                                    getChannel());
                            if (proxy != null) {
                                System.out.println(
                                        "Updating proxy in handover: " +
                                        ecr3.getChannel());
                                proxy.handover(ecr3.getActive(),
                                               ecr3.getRemoved(),
                                               new_as, iface, ecr3.getChannel());
                            }

                            //System.out.println("Remove size: " + ecr3.getRemoved() + " for " +
                            //                   ecr3.getChannel());

                        }

                        sets = ECRRerouteInfo.multiplex(hsh);
                    }

                    Log.write("Sending to ec:" + sets);

                    NameValuePair[] arguments = new NameValuePair[] {
                                                new NameValuePair("arg1",
                            ecr.getChannel()),
                                                new NameValuePair("arg2",
                            new_as.trim()),
                                                new NameValuePair("arg3", sets),
                                                new NameValuePair("arg4",
                            iface.trim())
                    };

                    MessageTransportService mts = MessageTransportService.init();

                    MessageService msz = mts.getService();

                    RpcCall call = null;

                    // We offer two kinds of event channel updates:
                    // Merged sets and non-merged sets

                    if (fuegocore.notify.profile.UserProfile.mergeEC) {
                        call = msz.createCall("handover", parameters, "Boolean");
                    } else {
                        call = msz.createCall("handover", parameters, "Boolean");

                    }

                    MessageProtocol mp = null;

                    if (fuegocore.notify.profile.UserProfile.useBaseProtocol) {
                        mp = mts.getService().getProtocol("base");
                    } else {
                        mp = mts.getProtocol();
                    }
                    Boolean reply = null;
                    try {
                        reply = (Boolean) call.synchInvoke(mp.constructUrl(url.
                                getHost(),
                                url.getPort(), "rpc_ec"), arguments,
                                es.SERVER_TIMEOUT);
                    } catch (fuegocore.message.service.TimeoutException e) {
                        reply = new Boolean(false);
                    }
                    if (!reply.booleanValue()) {
                        // We roll back the changes that were made (this does not lose concurrently
                        // made modifications). The FilterService structure has not yet
                        // been updated.
                        if (fuegocore.notify.profile.UserProfile.mergeEC) {
                            Log.write("Rolling back");
                            for (Enumeration kl = hsh.elements();
                                                  kl.hasMoreElements(); ) {
                                ECRRerouteInfo ecr3 = (ECRRerouteInfo) kl.
                                        nextElement();
                                es.rollBackMerge(ecr3.getChannel());
                            }
                        }

                        return "event channel failure";
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }

            } // for
        } // if (hsh..

        // Prevent delivery to session
        co.getMTP().handOverInProgress();

        Log.write("Packaging session data");

        // Return session information
        Vector data = null;
        synchronized (co.getMTP().getOutbound()) {
            data = (Vector) co.getMTP().getOutbound().getData();
        }
        String sessiondata = "";
        if (data == null) {
            Log.write("************** Data was null");
        }
        if (data != null) {
            Log.write("Converting " + data.size() + " + messages");

            for (int i = 0; i < data.size(); i++) {
                Message msg = (Message) data.elementAt(i);
                if (msg != null) {
                    sessiondata += (String) msg.getData();
                    sessiondata += "<_dlm/>";
                } // if (msg..
            } // for
        } // if (data..

        Log.write("-----------------------------------------");
        Log.write("Returning: " + sessiondata);
        Log.write("-----------------------------------------");

        return sessiondata;
    }
*/

    /** Final phase in the handover, remove session and connection.
     *
     * @param new_as new server
     * @param id     session identifier
     * @param device device
     * @return String
     */
    /*
    public String startHandoverPhase3(String new_as, String id, String device) {
        String iface = fuegocore.notify.profile.CommProfile.event_server_id;

        Log.write("Access server: starting handover 3:" + new_as);

        // Currently we use id only
        ConnectionObject co = es.getConnection(id);
        if (co == null) {
            return "invalid connection";
        }

        Vector templates = FilterService.get().removeAllTemplates(id);

        // test
        // es.dumpMergedSets();
        es.destroyConnection(id);
        return "ack";
    }
    */
}
