/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.server.manager;

import fuegocore.notify.core.server.*;

/**
 * Stores information about handovers in progress
 */
public class HAObject {
    private String source;
    private String target;
    private String sessionId;
    private long startTime;
    private long endTime;

    /** Handover is inactive */
    public static final int INACTIVE = 0;

    /** Handover is in progress */
    public static final int ACTIVE = 1;

    /** Handover state 1 */
    public static final int HANDOVER_1 = 2;

    /** Handover state 2 */
    public static final int HANDOVER_2 = 3;

    /** Handover state 3 */
    public static final int HANDOVER_3 = 4;

    private int state = INACTIVE;

    /**
     * Constructor
     * @param source String
     * @param target String
     * @param sessionId String
     */
    public HAObject(String source, String target, String sessionId) {
        this.source = source;
        this.target = target;
        this.sessionId = sessionId;
        startTime = System.currentTimeMillis();
    }

    /**
     * Get the source
     * @return String
     */
    public String getSource() {
        return source;
    }

    /**
     * Get the target
     * @return String
     */
    public String getTarget() {
        return target;
    }

    /**
     * Get the sessionId
     * @return String
     */
    public String getSessionId() {
        return sessionId;
    }

    /**
     * Get the start time
     * @return long
     */
    public long getStartTime() {
        return startTime;
    }

    /**
     * Get the end time
     * @return long
     */
    public long getEndTime() {
        return endTime;
    }

    /**
     * Set the end time
     * @param endTime long
     */
    public void setEndTime(long endTime) {
        this.endTime = endTime;
    }

    /**
     * Set the state
     * @param state int
     */
    public void setState(int state) {
        this.state = state;
    }

    /**
     * Get the state
     * @return int
     */
    public int getState() {
        return state;
    }

    /**
     * Is the handover object expired
     * @return boolean
     */
    public boolean isExpired() {
        if (startTime + EventServer.get().HANDOVER_TIMEOUT <=
            System.currentTimeMillis()) {
            return true;
        }
        return false;
    }
}
