/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.server.manager;

import java.util.*;

import fuegocore.notify.core.server.*;

/**
 * Manages handover objects
 */
public class HandoverManager {
    /** Stores handover objects */
    Hashtable handovers = new Hashtable();

    /**
     * Create and insert a handover object
     * @param old String
     * @param ac String
     * @param session String
     * @return HAObject
     */
    public HAObject insert(String old, String ac, String session) {
        HAObject hob = new HAObject(old, ac, session);
        handovers.put(session, hob);
        return hob;
    }

    /**
     * Get a handover object
     * @param session String
     * @return HAObject
     */
    public HAObject get(String session) {
        HAObject hob = (HAObject) handovers.get(session);
        return hob;
    }

    /**
     * Manage handover objects
     */
    public void manage() {
        Hashtable ha = (Hashtable) handovers.clone();
        for (Enumeration e = ha.elements(); e.hasMoreElements(); ) {
            HAObject hob = (HAObject) e.nextElement();
            if (hob.isExpired()) {
                handovers.remove(hob.getSessionId());
                EventServer.get().getMobilityEngine().handoverFailure(hob.
                        getSessionId());
            }
        }
    }
}
