/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.server.ui;

import java.util.*;

import fuegocore.notify.comms.*;
import fuegocore.notify.core.channel.*;

/**
 * This class implements the Fuego router UI (in html).
 */
public class FuegoUI implements RouterUI {
    public static String TABLE_BG1 = "#9eceee";
    public static String BACKGROUND = "#ffffff";
    public static String SEL_COLOR = "#ffffff";
    public static String NOSEL_COLOR = "#efefff";
    public static String GREY_BAR = "#777777";
    public static String TABLE_ROW1 = "#eeeeee";
    public static String TABLE_ROW2 = "#cccccc";

    public String getUI(String s1, String s2, String params) {
        String ret = "";

        EventChannelServer es = EventChannelServer.get();

        Hashtable a = (Hashtable) es.getEventChannels().clone();

        // Proxy channels for synchronized operation
        Hashtable proxies = es.getEventChannelProxies();

        if (proxies != null && a != null) {
            a.putAll(proxies);
        }
        try {
            // Hashtable does not keep order so we use identifiers to match
            Enumeration k = a.keys();
            for (Enumeration e = a.elements(); e.hasMoreElements(); ) {
                EventChannel ch = (EventChannel) e.nextElement();
                String name = (String) k.nextElement();
                if (ch != null) {
                    String id = name;
                    if (!s1.equals(id)) {
                        ret += "<tr bgcolor=\"" + TABLE_BG1 + "\"><td>";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id3=" + id + params +
                                "\" noborder>";
                        ret +=
                                "<img src=\"images/info.gif\" noborder align=center alt=\"open\"></a>";
                        ret += name + " (history size:" + ch.getHistorySize() +
                                ")";

                        if (proxies != null && proxies.contains(ch)) {
                            ret += " (proxy)";
                        } else {
                            if (ch.getTemporary()) {
                                ret += " (temporary)";
                            } else {
                                ret += " (permanent)";
                            }
                        }
                        ret += "</td></tr>";
                    } else { // selected entry
                        ret += "<tr bgcolor=\"" + SEL_COLOR +
                                "\"><td align=left>";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id3=" + params +
                                "\" noborder>";
                        ret += "<img src=\"images/info.gif\" noborder align=left valign=center alt=\"close\"></a>";
                        ret += name + " (history size:" + ch.getHistorySize() +
                                ")";

                        if (proxies != null && proxies.contains(ch)) {
                            ret += " (proxy)";
                        } else {
                            if (ch.getTemporary()) {
                                ret += " (temporary)";
                            } else {
                                ret += " (permanent)";
                            }
                        }
                        ret += "<br>";

                        // We print detailed session information and subscriptions
                        Vector ifaces = ch.getInterfaces();

                        ret += "&nbsp;&nbsp;Access server identifiers: " +
                                ifaces.size();
                        ret += "<br>";

                        ret += getAccessInterfaces(ch, id, s2, params);

                        ret += "</td></tr>";
                    }
                    // black border
                    ret +=
                            "<tr bgcolor=\"#000000\" height=1><td height=1></td></tr>";
                }
            }
        } catch (Exception e) {
            ret = "Exception occurred: " + e;
            e.printStackTrace();
        }
        return ret;
    }

    private static String getAccessInterfaces(EventChannel ec, String id0,
                                              String id2, String params) {
        String ret = "";

        // get templates for this user
        EventChannelServer es = EventChannelServer.get();
        Vector v = ec.getInterfaces();

        if (v == null || v.size() < 1) {
            ret += "<br>No access server interfaces";
        }

        ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";

        // grey border
        ret += "<tr bgcolor=\"" + GREY_BAR +
                "\" height=1><td height=1></td></tr>";
        for (int i = 0; i < v.size(); i++) {
            String s = (String) v.elementAt(i);
            if (s == null) {
                continue;
            }

            String id = "" + Integer.toString(s.hashCode());
            if (id.equals(id2)) {
                ret += "<tr bgcolor=\"" + SEL_COLOR +
                        "\"><td align=left>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id3=" + id0 + params +
                        "\" alt=\"close\" border=0>";
                ret +=
                        " <img src=\"images/sel_open.gif\" noborder align=center></a>";
                Vector filt = ec.getSubscriptions(s);
                int count = 0;
                if (filt != null) {
                    count = filt.size();
                }
                ret += "&nbsp;&nbsp; Interface:" + s;

                // Launch poset viewer
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id=ec_sub_poset&id2=" + id0 + "&id3=" + s +
                        "\" border=0> View poset</a><br>";

                ret += "&nbsp;&nbsp;Subscriptions (" + count + " filters)<br>";

                // Print information
                ret += "&nbsp;&nbsp;Filters<br>";
                ret += ManagementUI.printFilterTable(filt);

                filt = ec.getAdvertisements(s);
                count = 0;
                if (filt != null) {
                    count = filt.size();
                }
                ret += "&nbsp;&nbsp;Advertisements (" + count + " filters) ";

                // Launch poset viewer
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id=ec_adv_poset&id2=" + id0 + "&id3=" + s +
                        "\" border=0> View poset</a><br>";

                // Print subscription information
                ret += "&nbsp;&nbsp;Filters<br>";
                ret += ManagementUI.printFilterTable(filt);

                // Print Quench information

                filt = ec.getQuench(s);
                count = 0;
                if (filt != null) {
                    count = filt.size();
                }
                ret += "&nbsp;&nbsp;Quench (" + count + " filters)<br>";

                // Print subscription information
                ret += "&nbsp;&nbsp;Filters<br>";
                ret += ManagementUI.printFilterTable(filt);

                ret += "</td></tr>";
            } else {
                ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id3=" + id0 + "&id4=" +
                        id + params + "\" alt=\"open\" border=0>";
                ret += "<img src=\"images/sel.gif\" noborder align=center></a>";
                Vector filt = ec.getSubscriptions(s);
                int count = 0;

                if (filt != null) {
                    count = filt.size();
                }
                ret += "&nbsp;&nbsp;" + s + " (" + count + " subsc.)<br>";
                ret += "</td></tr>";
            }
            // black border
            ret += "<tr bgcolor=\"#000000\" height=1><td height=1></td></tr>";
        }

        // -------------------------------------------------

        Vector history = ec.getHistory();

        synchronized (history) {
            int count = 0;
            if (history != null) {
                count = history.size();
            }
            if (id2.equals("log")) {
                ret += "<tr bgcolor=\"" + SEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id3=" + id0 + params +
                        "\" alt=\"open\" border=0>";
                ret +=
                        "<img src=\"images/sel_open.gif\" noborder align=center></a>";
                ret += "&nbsp;&nbsp;Event Log (" + count + " notifications)  ";
                ret += "<b>10</b> <a href=\"http://localhost:" +
                        HTTPServlet.getPort() +
                        "/notify?id3=" + id0 +
                        "&id4=log2" + params + "\" alt=\"open\" border=0>";
                ret += "  50</a><br>";
                ret += ManagementUI.printNotifications(history, "10");
                ret += "</td></tr>";
            } else if (id2.equals("log2")) {
                ret += "<tr bgcolor=\"" + SEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id3=" + id0 + params +
                        "\" alt=\"open\" border=0>";
                ret +=
                        "<img src=\"images/sel_open.gif\" noborder align=center></a>";
                ret += "&nbsp;&nbsp;Event Log (" + count + " notifications) ";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id3=" + id0 + "&id4=log" +
                        params + "\" alt=\"open\" border=0>";
                ret += "10</a>   <b>50</b><br>";
                ret += ManagementUI.printNotifications(history, "50");
                ret += "</td></tr>";
            } else {
                ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id3=" + id0 + "&id4=log" +
                        params + "\" alt=\"open\" border=0>";
                ret += "<img src=\"images/sel.gif\" noborder align=center></a>";
                ret += "&nbsp;&nbsp;Event Log (" + count +
                        " notifications)<br>";
                ret += "</td></tr>";
            }
        }
        // white border
        ret += "<tr bgcolor=\"" + SEL_COLOR +
                "\" height=4><td height=4></td></tr>";
        ret += "</table>";
        ret += "<!-- susbcription tree ends --!>";
        return ret;
    }

}
