/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.core.server.ui;

import java.net.*;
import java.util.*;

import fuegocore.notify.comms.*;
import fuegocore.notify.core.*;
import fuegocore.notify.core.channel.*;
import fuegocore.notify.core.filter.*;
import fuegocore.notify.core.filter.poset.*;
import fuegocore.notify.core.server.*;
import fuegocore.notify.core.session.*;

/**
 * This class creates the management user interface
 */
public class ManagementUI {
    public static String TABLE_BG1 = "#9eceee";
    public static String BACKGROUND = "#ffffff";
    public static String SEL_COLOR = "#ffffff";
    public static String NOSEL_COLOR = "#efefff";
    public static String GREY_BAR = "#777777";
    public static String TABLE_ROW1 = "#eeeeee";
    public static String TABLE_ROW2 = "#cccccc";

    public ManagementUI() {
    }

    /**
     * Returns a html page according to the input parameters
     * @param id String
     * @param id2 String
     * @param id3 String
     * @param id4 String
     * @param debug String
     * @return String
     */
    public static String getHTMLPage(String id, String id2, String id3,
                                     String id4, String debug) {
        String ret = "";

        if (id == null) {
            id = "";
        }
        if (id2 == null) {
            id2 = "";
        }
        if (id3 == null) {
            id3 = "";
        }
        if (id4 == null) {
            id4 = "";
        }
        if (id4 == null) {
            id4 = "";
        }
        if (debug == null) {
            debug = "";
        }
        String params = "";

        // Poset viewer override
        if (id.equals("ec_sub_poset")) {
            ret += getIntro();
            params = "&id=" + id + "&id2=" + id2 +
                     "&id3=" + id3 + "&id4=" + id4;
            ret += getSubPoset(id2, id3);
            ret += getEnd(debug, params);
            return ret;
        } else if (id.equals("ec_adv_poset")) {
            ret += getIntro();
            params = "&id=" + id + "&id2=" + id2 +
                     "&id3=" + id3 + "&id4=" + id4;
            ret += getAdvPoset(id2, id3);
            ret += getEnd(debug, params);
            return ret;
        }

        ret += getIntro();
        params = "&id3=" + id3 + "&id4=" + id4;
        ret += getAccessServerColumn(id, id2, params);
        ret += closeAccessColumn();
        params = "&id=" + id + "&id2=" + id2;
        ret += getEventChannelColumn(id3, id4, params);
        ret += closeEventColumn();
        params = "&id=" + id + "&id2=" + id2 +
                 "&id3=" + id3 + "&id4=" + id4;
        ret += getEnd(debug, params);

        return ret;
    }

    // this is event channel specific
    private static String getSubPoset(String channel, String iface) {
        String ret = "";

        // get templates for this user
        EventChannelServer es = EventChannelServer.get();
        EventChannel ec = (EventChannel) es.lookup(channel);
        if (ec == null) {
            return "<html><h1>Channel was null</h1></html>";
        }
        IFilterPoset pos = ec.getSubPoset();
        if (pos == null) {
            return "<html><h1>Poset was null</h1></html>";
        }
        ret += pos.getHTMLOutput();
        return ret;

    }

    // this is event channel specific
    private static String getAdvPoset(String channel, String iface) {
        String ret = "";

        // get templates for this user
        EventChannelServer es = EventChannelServer.get();
        EventChannel ec = (EventChannel) es.lookup(channel);
        if (ec == null) {
            return "<html><h1>Channel was null</h1></html>";
        }
        IFilterPoset pos = ec.getAdvPoset();
        if (pos == null) {
            return "<html><h1>Poset was null</h1></html>";
        }
        ret += pos.getHTMLOutput();

        return ret;
    }

    private static String getAccessServerColumn(String s1, String s2,
                                                String params) {
        String ret = "";

        ret += "<table width=400 cellSpacing=0 cellPadding=0>";
        ret +=
                "<tr valign=top><td><img src=\"images/server.gif\" align=center noborder>";
        ret += "<font size=+1>Access Server Sessions</font>";
        ret += "</tr>";
        ret += "<!-- end of intro --!>";
        ret += "<tr valign=top><td>";
        ret += "<!-- session tree starts --!>";
        ret += "<table width=380 cellSpacing=0 cellPadding=0>";
        ret += "<tr bgcolor=\"" + TABLE_BG1 + "\" width=380><td>";
        ret += getSessions(s1, s2, params);
        ret += "</table>";
        ret += "<!-- session tree ends --!>";
        ret += "</tr><tr><td><br><br>";
        ret += getStatistics(getAccessMergeStatus() + handOverInfo());
        return ret;
    }

    private static String getSessions(String s1, String s2, String params) {
        String ret = "";
        EventServer eb = EventServer.get();
        Hashtable a = eb.getConnections();
        try {

            // Hashtable does not keep order so we use identifiers to match
            for (Enumeration e = a.elements(); e.hasMoreElements(); ) {
                ConnectionObject co = (ConnectionObject) e.nextElement();
                if (co != null) {
                    String pic = "";
                    if (co.getMTP().getPause()) {
                        pic = "pda.gif";
                        if (co.getDevice() != null &&
                            (co.getDevice().indexOf("cldc") != -1 ||
                             co.getDevice().indexOf("phone") !=
                             -1)) {
                            pic = "phone.gif";
                        }
                    } else {
                        pic = "pda_dis.gif";
                        if (co.getDevice() != null &&
                            (co.getDevice().indexOf("cldc") != -1 ||
                             co.getDevice().indexOf("phone") !=
                             -1)) {
                            pic = "phone_dis.gif";
                        }
                    }

                    String id = Integer.toString(co.getID().hashCode());
                    if (!s1.equals(id)) {

                        ret += "<tr bgcolor=\"" + TABLE_BG1 + "\"><td>";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + id + params +
                                "\" border=0>";
                        ret += "<img src=\"images/" + pic +
                                "\" noborder align=center alt=\"open\"></a>";
                        ret += co.getID() + " (" + co.getAddress() +
                                ") last server:" +
                                co.getLastServer();
                        if (co.handOverFailFlag) {
                            ret += " HO failure";
                        }
                        ret += "</td></tr>";
                    } else { // selected entry
                        ret += "<tr bgcolor=\"" + SEL_COLOR +
                                "\"><td align=left>";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + params +
                                "\" border=0>";
                        ret += "<img src=\"images/" + pic +
                                "\" noborder align=left valign=center alt=\"close\"></a>";
                        ret += co.getID() + " (" + co.getAddress() +
                                ") last server:" +
                                co.getLastServer();
                        if (co.handOverFailFlag) {
                            ret += " HO failure";
                        }
                        ret += "<br>";
                        // We print detailed session information and subscriptions
                        SessionParams sp = co.getSessionParams();
                        Vector msg = co.getMessages();
                        ret += "&nbsp;&nbsp;Session status: " + msg.size() +
                                "/" +
                                sp.getMaxQueueSize();
                        ret += "<br>";
                        if (sp.getPause()) {
                            ret +=
                                    "&nbsp;&nbsp;Session paused by client (pull-mode) ";
                        } else if (co.getMTP().getPause()) {
                            ret +=
                                    "&nbsp;&nbsp;Session active but client disconnected ";
                        } else {
                            ret += "&nbsp;&nbsp;Session active";

                        }
                        ret +=
                                getSubscriptions(co.getID(), id, s2, params, co.getMessages());

                        ret += "</td></tr>";
                    }
                    // black border
                    ret +=
                            "<tr bgcolor=\"#000000\" height=1><td height=1></td></tr>";
                }
            }
        } catch (Exception e) {
            ret = "Exception occurred: " + e;
        }
        return ret;
    }

    private static String getSubscriptions(String iface, String s1, String s2,
                                           String params, Vector session) {
        String ret = "";

        synchronized (session) {
            // get subscriptions for this user
            Vector v = EventServer.get().getRoutingEngine().getTemplatesForId(
                    iface);

            if (v == null || v.size() < 1) {
                ret += "<br>No subscriptions";
            } else {
                ret += "<br>Subscriptions";
                ret += "<!-- subscription tree starts --!>";
                ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";
                // grey border
                ret += "<tr bgcolor=\"" + GREY_BAR +
                        "\" height=1><td height=1></td></tr>";
                for (int i = 0; i < v.size(); i++) {
                    ITemplate t = (ITemplate) v.elementAt(i);
                    if (t == null) {
                        continue;
                    }
                    String id = "" + Integer.toString(t.getSubId().hashCode());
                    if (id.equals(s2)) {
                        ret += "<tr bgcolor=\"" + SEL_COLOR +
                                "\"><td align=left>&nbsp;";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + s1 + params +
                                "\" alt=\"close\" border=0>";
                        ret +=
                                " <img src=\"images/sel_open.gif\" noborder align=center></a>";
                        ret += "&nbsp;&nbsp;" + t.getType() + " (" + t.getSubId() +
                                ")<br>";
                        // Print subscription information
                        ret += "&nbsp;&nbsp;Filters<br>";
                        ret += printFilterTable(t);
                        ret += "</td></tr>";
                    } else {
                        ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>&nbsp;";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + s1 + "&id2=" +
                                id +
                                params + "\" alt=\"open\" border=0>";
                        ret +=
                                "<img src=\"images/sel.gif\" noborder align=center></a>";
                        ret += "&nbsp;&nbsp;" + t.getType() + " (" + t.getSubId() +
                                ")</td></tr>";
                    }
                    // black border
                    ret +=
                            "<tr bgcolor=\"#000000\" height=1><td height=1></td></tr>";
                }

                // white border
                ret += "<tr bgcolor=\"" + SEL_COLOR +
                        "\" height=4><td height=4></td></tr>";
                ret += "</table>";
            }

            // get advertisements for this user
            v = EventServer.get().getRoutingEngine().getAdvertisementsForId(
                    iface);

            if (v == null || v.size() < 1) {
                ret += "<br>No advertisements";
            } else {
                ret += "<br>Advertisements";
                ret += "<!-- advertisement tree starts --!>";
                ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";
                // grey border
                ret += "<tr bgcolor=\"" + GREY_BAR +
                        "\" height=1><td height=1></td></tr>";
                for (int i = 0; i < v.size(); i++) {
                    ITemplate t = (ITemplate) v.elementAt(i);
                    if (t == null) {
                        continue;
                    }
                    String id = "" + Integer.toString(t.getSubId().hashCode());
                    if (id.equals(s2)) {
                        ret += "<tr bgcolor=\"" + SEL_COLOR +
                                "\"><td align=left>&nbsp;";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + s1 + params +
                                "\" alt=\"close\" border=0>";
                        ret +=
                                " <img src=\"images/sel_open.gif\" noborder align=center></a>";
                        ret += "&nbsp;&nbsp;" + t.getType() + " (" + t.getSubId() +
                                ")<br>";
                        // Print subscription information
                        ret += "&nbsp;&nbsp;Filters<br>";
                        ret += printFilterTable(t);
                        ret += "</td></tr>";
                    } else {
                        ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>&nbsp;";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + s1 + "&id2=" +
                                id +
                                params + "\" alt=\"open\" border=0>";
                        ret +=
                                "<img src=\"images/sel.gif\" noborder align=center></a>";
                        ret += "&nbsp;&nbsp;" + t.getType() + " (" + t.getSubId() +
                                ")</td></tr>";
                    }
                    // black border
                    ret +=
                            "<tr bgcolor=\"#000000\" height=1><td height=1></td></tr>";
                }

                // white border
                ret += "<tr bgcolor=\"" + SEL_COLOR +
                        "\" height=4><td height=4></td></tr>";
                ret += "</table>";
            }

            // get advertisements for this user
            v = EventServer.get().getRoutingEngine().getQuenchForId(iface);

            if (v == null || v.size() < 1) {
                ret += "<br>No quench";
            } else {
                ret += "<br>Quench";
                ret += "<!-- quench tree starts --!>";
                ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";
                // grey border
                ret += "<tr bgcolor=\"" + GREY_BAR +
                        "\" height=1><td height=1></td></tr>";
                for (int i = 0; i < v.size(); i++) {
                    ITemplate t = (ITemplate) v.elementAt(i);
                    if (t == null) {
                        continue;
                    }
                    String id = "" + Integer.toString(t.getSubId().hashCode());
                    if (id.equals(s2)) {
                        ret += "<tr bgcolor=\"" + SEL_COLOR +
                                "\"><td align=left>&nbsp;";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + s1 + params +
                                "\" alt=\"close\" border=0>";
                        ret +=
                                " <img src=\"images/sel_open.gif\" noborder align=center></a>";
                        ret += "&nbsp;&nbsp;" + t.getType() + " (" + t.getSubId() +
                                ")<br>";
                        // Print subscription information
                        ret += "&nbsp;&nbsp;Filters<br>";
                        ret += printFilterTable(t);
                        ret += "</td></tr>";
                    } else {
                        ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>&nbsp;";
                        ret += "<a href=\"http://localhost:" +
                                HTTPServlet.getPort() +
                                "/notify?id=" + s1 + "&id2=" +
                                id +
                                params + "\" alt=\"open\" border=0>";
                        ret +=
                                "<img src=\"images/sel.gif\" noborder align=center></a>";
                        ret += "&nbsp;&nbsp;" + t.getType() + " (" + t.getSubId() +
                                ")</td></tr>";
                    }
                    // black border
                    ret +=
                            "<tr bgcolor=\"#000000\" height=1><td height=1></td></tr>";
                }
                // white border
                ret += "<tr bgcolor=\"" + SEL_COLOR +
                        "\" height=4><td height=4></td></tr>";
                ret += "</table>";
            }

            ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";

            int count = 0;
            count = session.size();
            if (s2.equals("log")) {
                ret += "<tr bgcolor=\"" + SEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id=" + s1 + params +
                        "\" alt=\"open\" border=0>";
                ret +=
                        "<img src=\"images/sel_open.gif\" noborder align=center></a>";
                ret += "&nbsp;&nbsp;Session Contents (" + count +
                        " notifications)  ";
                ret += "<b>10</b> <a href=\"http://localhost:" +
                        HTTPServlet.getPort() +
                        "/notify?id=" + s1 +
                        "&id2=log2" + params + "\" alt=\"open\" border=0>";
                ret += "  50</a><br>";
                ret += printNotifications(session, "10");
                ret += "</td></tr>";
            } else if (s2.equals("log2")) {
                ret += "<tr bgcolor=\"" + SEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id=" + s1 + params +
                        "\" alt=\"open\" border=0>";
                ret +=
                        "<img src=\"images/sel_open.gif\" noborder align=center></a>";
                ret += "&nbsp;&nbsp;Session Contents (" + count +
                        " notifications)  ";
                ret += " <a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id=" + s1 + "&id2=log" +
                        params + "\" alt=\"open\" border=0>";
                ret += "  10</a> <b>50</b><br>";
                ret += printNotifications(session, "50");
                ret += "</td></tr>";
            } else {
                ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>&nbsp;";
                ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                        "/notify?id=" + s1 + "&id2=log" +
                        params + "\" alt=\"open\" border=0>";
                ret += "<img src=\"images/sel.gif\" noborder align=center></a>";
                ret += "&nbsp;&nbsp;Session Contents (" + count +
                        " notifications)  ";
                ret += "</td></tr>";
            }

            // white border
            ret += "<tr bgcolor=\"" + SEL_COLOR +
                    "\" height=4><td height=4></td></tr>";
            ret += "</table>";
        }
        return ret;
    }

    private static String getAccessMergeStatus() {
        String ret = "";
        return ret;
        /*
             EventServer eb = EventServer.get();
             if (fuegocore.notify.profile.UserProfile.mergeEC) {
          ret += "Merge mode active with lazy unsubscriptions<br>";
          //FuegoRoutingEngine fr = (FuegoRoutingEngine)eb.getRoutingEngine();
          //Hashtable h = fr.mergedSets;
          Hashtable h = null;
          int i = 0;
          for (Enumeration e = h.elements(); e.hasMoreElements(); ) {
            Vector v = (Vector) e.nextElement();
            if (v != null) {
              i += v.size();
            }
          }
          Vector v = FilterService.get().getALLTemplates().getAllInOrder();
          int j = v.size();
          double p = 0;
          if (j != 0) {
            p = (double) i / (double) j;
          }
          p = p * 100;
          ret += i + " merged filters," + j + " local filters, " + p;
          ret += "% from local size";
          ret += "<br>";
          ret += "False positive count: " + eb.strayNotification;
          ret += "<br>";
          ret += "Lazy channel updates: " + eb.lazyUpdates;
             }
             else if (fuegocore.notify.profile.UserProfile.subCover) {
          ret += "Cover mode active";
             }
             else {
          ret += "No merging or covering active";
             }
             return ret;
         */
    }

    private static String handOverInfo() {
        String ret = "";
        EventServer eb = EventServer.get();
        Hashtable ho = eb.getHandoverBuffer();
        ret += "<br><br>Handovers in progress: " + ho.size();
        ret += "<br>Roaming client identifiers: <br>";
        for (Enumeration e = ho.keys(); e.hasMoreElements(); ) {
            String s = (String) e.nextElement();
            ret += s;
            if (e.hasMoreElements()) {
                ret += ",";
            }
        }
        return ret;
    }

    public static String printFilterTable(Object t) {
        String ret = "";
        boolean fl = false;

        if (t != null && t instanceof Vector) {
            Vector v = (Vector) t;
            for (int i = 0; i < v.size(); i++) {
                ret += printFilterTable(v.elementAt(i));
            }
            return ret;
        }

        if (t == null || !(t instanceof ITemplate)) {
            return "unknown filter";
        } else if (!(t instanceof Template)) {
            return ((ITemplate) t).toString();
        }

        Vector f = ((Template) t).getFilters();

        if (f == null) {
            return "No filters";
        }
        ret += "Tids: " + ((Template) t).getTid();
        ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";
        ret += "<tr><td width=10>&nbsp;</td><td><b>Name</b></td><td><b>Type</b></td><td><b>Filter clause</b></td></tr>";

        for (int j = 0; j < f.size(); j++) {
            Filter fi = (Filter) f.elementAt(j);
            if (fi == null) {
                continue;
            }
            String s = fi.getFilterString();
            s.replaceAll("&", "&amp;");
            if (fl) {
                ret += "<tr bgcolor=" + TABLE_ROW1 +
                        "><td width=10>&nbsp;</td><td>" +
                        fi.getName() + "</td><td>" +
                        EventTriplet.getTypeString(fi.getType()) +
                        "</td><td>" +
                        s + "</td></tr>";
                fl = false;
            } else {
                ret += "<tr bgcolor=" + TABLE_ROW2 +
                        "><td width=10>&nbsp;</td><td>" +
                        fi.getName() + "</td><td>" +
                        EventTriplet.getTypeString(fi.getType()) +
                        "</td><td>" +
                        s + "</td></tr>";
                fl = true;
            }
        }
        ret += "</table>";
        return ret;
    }

    /** Returns links to other servers
     * @return String the server list
     */
    public static String getServerList() {
        return "";
    }

    private static String closeAccessColumn() {
        return "</tr></table></td><td valign=top>";
    }

    private static String closeEventColumn() {
        return "</td></tr></table></td></tr>";
    }

    private static String getEventChannelColumn(String s1, String s2,
                                                String params) {
        String ret = "";
        ret += "<table width=400 cellSpacing=0 cellPadding=0>";
        ret +=
                "<tr valign=top><td><img src=\"images/server.gif\" align=center noborder>";
        ret += "<font size=+1>Resolution Server</font>";
        ret += "</tr>";
        ret += "<!-- end of intro --!>";
        ret += "<tr valign=top><td>";
        ret += "<table width=380 cellSpacing=0 cellPadding=0>";
        ret += "<tr bgcolor=\"" + TABLE_BG1 + "\" width=380><td>";

        RouterUI rou = EventServer.get().getRoutingEngine().getUI();
        ret += rou.getUI(s1, s2, params);
        ret += "</table>";
        ret += "</tr><tr><td><br><br>";
        ret += getStatistics(getCoreBrokers());
        return ret;
    }

    static String printNotifications(Vector f, String start) {
        String ret = "";
        boolean fl = false;

        if (f == null || f.size() < 1) {
            return "No notifications";
        }

        int index = 10;
        try {
            index = Integer.parseInt(start);
        } catch (Exception e) {
            index = 10;
        }

        for (int i = f.size() - 1; i >= f.size() - Math.min(index, f.size()); i--) {
            Notification n = null;
            if (f.elementAt(i) instanceof Notification) {
                n = (Notification) f.elementAt(i);
            }
            if (n == null) {
                continue;
            }
            ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";
            ret += "<tr bgcolor=\"" + SEL_COLOR + "\"><td>Notification: " +
                    n.getType() +
                    " (" + n.getOrder() + ")<br>";
            ret += "</tr></table>";
            ret += "<table width=380 border=0 cellSpacing=0 cellPadding=0>";
            ret += "<tr><td width=10>&nbsp;</td><td><b>Name</b></td><td><b>Type</b></td><td><b>Value</b></td></tr>";
            Vector v = n.get().getVector();
            for (int j = 0; j < v.size(); j++) {
                EventTriplet et = (EventTriplet) v.elementAt(j);
                if (et == null) {
                    continue;
                }
                if (fl) {
                    ret += "<tr bgcolor=" + TABLE_ROW1 +
                            "><td width=10>&nbsp;</td><td>" +
                            (String) et.getName() +
                            "</td><td>" + EventTriplet.getTypeString(et.getType()) +
                            "</td><td>" +
                            (String) et.getValueAsString() + "</td></tr>";
                    fl = false;
                } else {
                    ret += "<tr bgcolor=" + TABLE_ROW2 +
                            "><td width=10>&nbsp;</td><td>" +
                            (String) et.getName() +
                            "</td><td>" + EventTriplet.getTypeString(et.getType()) +
                            "</td><td>" +
                            (String) et.getValueAsString() + "</td></tr>";
                    fl = true;
                }
            }
            ret += "</table>";
        }
        return ret;
    }

    private static String getCoreBrokers() {
        String ret = "";
        Vector v = new Vector();
        EventServer eb = EventServer.get();
        ret += "Servers<br>";

        if (eb.getRoutingEngine().getRouterManager() == null) {
            ret += "No servers";
            return ret;
        }

        for (Enumeration k = eb.getRoutingEngine().getRouterManager().
                             getChannels().
                             elements();
                             k.hasMoreElements(); ) {
            URL host = (URL) k.nextElement();
            if (v.contains(host)) {
                ; // ensure uniqueness
            } else {
                v.add(host);
                ret += "<a href=\"" + host + "/notify\">" + host + "</a><br>";
            }
        }
        return ret;
    }

    public static String getStatistics(String stat) {
        String ret = "";
        ret += "<!--  general statistics --!>";
        ret += "<table width=380 cellSpacing=0 cellPadding=0>";
        ret += "<tr bgcolor=\"" + TABLE_BG1 +
                "\"><td><h2>Statistics</h2></td></tr>";
        ret += "<tr bgcolor=\"" + NOSEL_COLOR + "\"><td>" + stat;
        ret += "</td></tr></table>";
        ret += "<!--  general statistics ends --!>";
        ret += "<br><br>";
        return ret;
    }

    private static String getIntro() {
        String ret = "";

        ret += "<html>";
        ret += "<head>";
        ret += "<title>Fuego Event Service Management Interface</title>";
        ret += "<link rel=\"stylesheet\" href=\"http://localhost:" +
                HTTPServlet.getPort() + "/rn.css\" type=\"text/css\">";
        ret += "</head>";
        ret += "<body bgcolor=\"" + BACKGROUND + "\">";
        // ret += "<table width=800 cellSpacing=0 cellPadding=0><tr><td><center>";
        ret += "<img src=\"http://localhost:" + HTTPServlet.getPort() +
                "/images/fuegocore.gif\" noborder>";
        //<td align=right valign=center>";
        // ret += "</td></tr></table>";
        ret += "<table width=800 cellSpacing=0 cellPadding=0 border=0 bgcolor=\"#cfefff\"><tr valign=top><td>";
        return ret;
    }

    private static String getEnd(String command, String params) {
        String ret = "";
        ret += "</table>";
        if (command.equals("ec_off")) {
            fuegocore.notify.profile.UserProfile.debugEC = false;
        } else if (command.equals("ec_on")) {
            fuegocore.notify.profile.UserProfile.debugEC = true;
        } else if (command.equals("filter_off")) {
            fuegocore.notify.profile.UserProfile.debugFilter = false;
        } else if (command.equals("filter_on")) {
            fuegocore.notify.profile.UserProfile.debugFilter = true;
        } else if (command.equals("filterservice_off")) {
            fuegocore.notify.profile.UserProfile.debugFilterService = false;
        } else if (command.equals("filterservice_on")) {
            fuegocore.notify.profile.UserProfile.debugFilterService = true;
        } else if (command.equals("general_off")) {
            fuegocore.notify.profile.UserProfile.debugGeneral = false;
        } else if (command.equals("general_on")) {
            fuegocore.notify.profile.UserProfile.debugGeneral = true;

        }
        ret += "<br><br>";
        if (fuegocore.notify.profile.UserProfile.debugEC) {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=ec_off" + params +
                    "\">Turn event channel debug off</a>";
        } else {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=ec_on" + params +
                    "\">Turn event channel debug on</a>";
        }
        ret += " &nbsp;|&nbsp; ";
        if (fuegocore.notify.profile.UserProfile.debugFilter) {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=filter_off" + params +
                    "\">Turn filter debug off</a>";
        } else {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=filter_on" + params +
                    "\">Turn filter debug on</a>";
        }
        ret += " &nbsp;|&nbsp; ";

        if (fuegocore.notify.profile.UserProfile.debugFilterService) {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=filterservice_off" +
                    params + "\">Turn filter service debug off</a>";
        } else {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=filterservice_on" +
                    params + "\">Turn filter service debug on</a>";
        }
        ret += " &nbsp;|&nbsp; ";

        if (fuegocore.notify.profile.UserProfile.debugGeneral) {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=general_off" +
                    params + "\">Turn general debug off</a>";
        } else {
            ret += "<a href=\"http://localhost:" + HTTPServlet.getPort() +
                    "/notify?debug=general_on" + params +
                    "\">Turn general service debug on</a>";
        }
        ret += "</html>";
        return ret;
    }

}
