/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.server.*;
import fuegocore.notify.demo.visualize.*;
import fuegocore.util.*;

public class DemoFrame extends Canvas implements Runnable, ImageResolver {
    // Create a new animation object for controlling sprites
    Animator anim = new Animator(this);
    private Image fuegopic;

    /** Thread used for triggering repaints */
    private Thread thread;

    /** buffer image of the screen */
    private Image image;

    /** Time of most recent paint */
    private long paintTime;

    /** Time of most recent frame rate report */
    private long statsTime;

    /** Number of frames since last frame rate report */
    int frameCount;
    private String statusMessage = null;

    /** Last reported frame rate (for re-paint) */
    int frameRate;
    int imageWidth;
    int imageHeight;
    int imageX;
    int imageY;
    int offset = 0;
    int offset2 = 0;
    int TICK_SPEED = 3;
    int FRAME_RATE = 100;
    int ticker_width = 0;
    String text;
    String next_string;
    boolean alive = true;
    boolean wait_mode = true;
    boolean uflag = false;
    double rot_angle = 0;
    Font writerfont = new Font("Arial", Font.BOLD, 20);
    Font font = new Font("Arial", Font.BOLD, 17);
    Font font2 = new Font("Arial", Font.BOLD, 50);

    Font font3 = new Font("Arial", Font.BOLD, 12);
    Font smallFont = new Font("Arial", Font.BOLD, 11);
    FontMetrics mwriterfont = this.getFontMetrics(writerfont);
    FontMetrics mfont1 = this.getFontMetrics(font);
    FontMetrics mfont2 = this.getFontMetrics(font2);
    FontMetrics mfont3 = this.getFontMetrics(font3);
    FontMetrics msmallfont = this.getFontMetrics(smallFont);
    Image myImage = null;
    Image player1 = null;
    Image player2 = null;
    Image player3 = null;
    Image player4 = null;
    Toolkit tk = null;
    AffineTransform atImageSpace = new AffineTransform();
    Vector notifs;
    int writerSlide = 0;
    int writerSlideA = 0;
    public static DemoFrame demoframe = null;
    ConnectionObject co = null;
    boolean runDemo = false;
    RTTPlotBox rttbox = new RTTPlotBox(660, 50, 80);
    Vector matchedTemplates = new Vector();
    Vector matchedNotifications = new Vector();
    int MAXTEMPLATES = 10;
    boolean flop = true;

    /* Callback for matched templates */
    public void matchedTemplates(Vector v, Notification n) {
        synchronized (matchedTemplates) {
            matchedTemplates.addAll(v);
            if (matchedTemplates.size() > MAXTEMPLATES) {
                for (int j = MAXTEMPLATES; j < matchedTemplates.size(); j++) {
                    matchedTemplates.remove(j);
                }
            }
        }
        synchronized (matchedNotifications) {
            matchedNotifications.add(n);
            if (matchedNotifications.size() > MAXTEMPLATES) {
                matchedNotifications.remove(0);
            }
        }

        if (flop) {
            flop = false;
        } else {
            flop = true;
        }
    }

    public void setCO(ConnectionObject _co) {
        co = _co;
    }

    public static void restart() {
        System.out.println("Restarting");
        demoframe.anim.replay();
        startAnimTime = 0;
    }

    public Animator getAnimator() {
        return anim;
    }

    public Image resolveImage(int i) {
        if (i == 0) {
            return player1;
        } else if (i == 1) {
            return player2;
        } else if (i == 2) {
            return player3;
        } else if (i == 3) {
            return player4;
        }
        return null;
    }

    public DemoFrame(String tx, int x, int y) {
        demoframe = this;
        setSize(x, y);
        Graphics g;
        image = this.createImage(getWidth(), getHeight());
        offset = getWidth();
        text = tx;

        ticker_width = mfont1.stringWidth(text);

        // Load the background
        fuegopic = loadImage("images/fc_demo.jpg");

        // Load player characters
        player1 = loadImage("images/ukko.gif"); //right
        player2 = loadImage("images/ukko2.gif"); //left
        player3 = loadImage("images/ukkol.gif"); //left
        player4 = loadImage("images/ukko2r.gif"); //right

        if (fuegopic != null) {
            imageWidth = fuegopic.getWidth(this);
            imageHeight = fuegopic.getHeight(this);
        } else {
            imageWidth = 0;
            imageHeight = 0;
        }

        imageX = (getWidth() - imageWidth) / 2;
        imageY = (getHeight() - imageHeight) / 2;

        FRAME_RATE = 100;
    }

    public void kick() {
        thread = new Thread(this);
        thread.setPriority(Thread.MIN_PRIORITY);
        thread.start();
        runDemo = true;
    }

    public void setNotifs(Vector v) {
        notifs = v;
    }

    private Image loadImage(String filename) {
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Image image = toolkit.getImage(ExtUtil.getResource(filename));
        MediaTracker mediaTracker = new MediaTracker(this);
        mediaTracker.addImage(image, 0);
        try {
            mediaTracker.waitForID(0);
        } catch (InterruptedException ie) {
            System.err.println(ie);
            return null;
        }
        return image;
    }

    public void setWaitMode() {
        wait_mode = true;
        FRAME_RATE = 100;
    }

    public void updateText(String tx) {
        text = tx;
        ticker_width = mfont1.stringWidth(text);
        if (wait_mode) {
            wait_mode = false;
        }
    }

    public void statusMessage(String s) {
        statusMessage = s;
        if (wait_mode) {
            wait_mode = false;
        }
    }

    public void setNextString(String next) {
        next_string = next;
        writerSlide = 110;
        writerSlideA = 15;
        if (wait_mode) {
            FRAME_RATE = 100;
            wait_mode = false;
        }
    }

    public void destroy() {
        alive = false;
    }

    /**
     * Set the array of images to be displayed.
     * Update the width and height of the image and draw
     * the border to fit around it in the offscreen image.
     * @param im Image
     */
    void setImage(Image im) {
        this.fuegopic = im;
        if (fuegopic != null) {
            imageWidth = fuegopic.getWidth(this);
            imageHeight = fuegopic.getHeight(this);
        } else {
            imageWidth = 0;
            imageHeight = 0;
        }
        imageX = (getWidth() - imageWidth) / 2;
        imageY = (getHeight() - imageHeight) / 2;
    }

    /**
     * Reset
     */
    void reset() {
        fuegopic = null;
        thread = null;
    }

    /**
     * Runs the animation and makes the repaint requests.
     * The thread will exit when it is no longer the current
     * Animation thread.
     */
    public void run() {
        Thread mythread = Thread.currentThread();
        long scheduled = System.currentTimeMillis();
        statsTime = scheduled;
        paintTime = scheduled;
        frameCount = 0;
        frameRate = 0;

        while (thread == mythread && alive) {
            synchronized (this) {
                try {
                    // Update when the next frame should be drawn
                    // and compute the delta till then
                    scheduled += FRAME_RATE; // 25 frames per second
                    long delta = scheduled - paintTime;
                    if (delta > 0) {
                        this.wait(delta);
                    }
                    // Advance and repaint the screen

                    repaint();
                } catch (InterruptedException e) {
                }
            }
        }
    }

    int wobbleCounter = 0;
    int dY = 500;
    int dK = 0;
    int mode = 0;
    public static long startAnimTime = 0;

    public void update(Graphics g) {
        paint(g);
    }

    public void drawTemplates(Graphics2D og) {

        int x = 0;
        int y = 150;

        og.setFont(smallFont);

        og.setComposite(
                AlphaComposite.getInstance(
                        AlphaComposite.SRC_OVER, 0.5f));

        Vector v = null;
        Vector v2 = EventServer.get().getOpLog();
        if (v2 != null) {
            synchronized (v2) {
                v = (Vector) v2.clone();
            }
        }

        if (v.size() > 0) {
            boolean flip = true;

            int k = 0;
            for (int i = v.size() - 1; i > -1; i--, k++) {
                String name = (String) v.elementAt(i);
                if (flip) {
                    og.setColor(Color.black);
                    flip = false;
                } else {
                    og.setColor(new Color(100, 100, 120));
                    flip = true;
                }

                og.drawString(name, x, y);

                y += msmallfont.getHeight() + 4;
            }
        }

        x = this.getWidth() - 150;
        y = 150;

        synchronized (matchedNotifications) {
            v = (Vector) matchedNotifications.clone();
        }
        int k = 0;
        if (v.size() > 0) {
            boolean flip = flop;
            for (int i = v.size() - 1; i > -1; i--, k++) {
                if (flip) {
                    og.setColor(Color.black);
                    flip = false;
                } else {
                    og.setColor(new Color(100, 100, 120));
                    flip = true;
                }

                Notification n = (Notification) v.elementAt(i);
//            String name = "Event: " + n.getType();

                //           og.drawString(name, x, y);
//          y += msmallfont.getHeight() + 4;

                Vector v3 = n.get().getVector();
                if (v3 != null && v3.size() > 0) {
                    for (int j = 0; j < v3.size(); j++) {
                        EventTriplet et = (EventTriplet) v3.elementAt(j);
                        if (et == null) {
                            continue;
                        }
                        og.drawString((String) et.getName() + "=" +
                                      (String) et.getValueAsString(), x + 2, y);
                        y += msmallfont.getHeight();
                    }
                }
            }
        }

        og.setComposite(
                AlphaComposite.getInstance(
                        AlphaComposite.SRC_OVER, 1.0f));
    }

    /**
     * Paint is called whenever the canvas should be redrawn.
     * It clears the canvas and draws the frame and the current
     * current frame from the animation.
     * @param g the Graphics context to which to draw
     */
    public void paint(Graphics g) {
        // normal scroller
        paintTime = System.currentTimeMillis();
        if (image == null) {
            image = this.createImage(getWidth(), getHeight());

        }

        Graphics2D og = (Graphics2D) image.getGraphics();
        og.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);

        og.setColor(new Color(255, 255, 255));
        og.fillRect(0, 0, getWidth(), getHeight());

        if (fuegopic != null) {
            og.drawImage(fuegopic, imageX,
                         getHeight() - fuegopic.getHeight(this) + dY, this);
        }

        if (frameRate > 0) {
            og.setColor(new Color(0, 0, 0));
            og.drawString("FPS = " + frameRate, 80, getHeight() - 10);
        }

        if (wobbleCounter++ > 400 && runDemo) {
            wobbleCounter = 0;
            /*  if (mode == 0) {
                dY = 0;
                mode = 1;
              }*/
            if (mode == 1) {
                dY = -getHeight();
                mode = 0;
            } else if (mode == 2) {
                dY = 32;
                mode = 0;
            }
            dK = 0;
        }

        if (mode == 1 && dY < getHeight()) {
            dY += dK;
            dK++;
        }

        if (mode == 0 && dY < 0) {
            dY += dK;
            dK++;
            if (dY > 0) {
                dY = 0;
            }
        }
        if (dY > 0 && mode == 0) {
            dY -= dK;
            dK++;
            if (dY < 0) {
                dY = 0;
            }
        }

        frameCount++;

        og.setColor(new Color(0, 0, 0));
        if (font != null) {
            og.setFont(font);

        }
        offset -= TICK_SPEED + 1;
        og.drawString(text, offset, 60);

        // Once the message is completely off the left side of
        // the screen, we reset its location to be the right side
        // of the screen
        if (offset <= -ticker_width) {
            offset = getWidth();
        }

        // Once the message is completely off the left side of
        // the screen, we reset its location to be the right side
        // of the screen
        if (offset <= -ticker_width) {
            offset = getWidth();
        }

        if (statusMessage != null && statusMessage.length() > 0) {
            og.setColor(new Color(0, 0, 0));
            og.setFont(font2);
            og.drawString(statusMessage, 15,
                          getHeight() - mfont2.getHeight());
        }

        og.setClip(0, 0, getWidth(), getHeight());

        // Animate the canvas using the script
        if (dY == 0 && runDemo) {
            if (startAnimTime == 0) {
                startAnimTime = System.currentTimeMillis();

            }
            anim.animate((int) (System.currentTimeMillis() - startAnimTime), og,
                         imageX, getHeight() - fuegopic.getHeight(this) + dY);
        }

        // Normalize stroke
        og.setStroke(new BasicStroke(1));

        og.setFont(font3);

        // Draw RTT box
        rttbox.draw((Graphics) og, (getWidth() - rttbox.sx) / 2, 80);

        if (co != null) {
            Vector vv = (Vector) co.getMessages();
            // test box, insert random value+sd
            if (vv != null) {
                rttbox.addPoint(vv.size(), 0);
            } else {
                rttbox.addPoint(0, 0);
            }
        }

        drawTemplates(og);
        g.drawImage(image, 0, 0, this);

        // Update Frame rate
        int delta = (int) (paintTime - statsTime);
        if (delta > 1000 && delta < 10000) {
            frameRate = ((frameCount * 1000 + 500) / delta);
            frameCount = 0;
            statsTime = paintTime;
        }
    }

}
