/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo;

import fuegocore.notify.core.*;

/**
 * Test producer class for the Fuego demonstration
 */
public class HeadlineProducer extends EventBase {
    String[] headlines;
    String[] domain;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("headline");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        HeadlineProducer test = new HeadlineProducer();
    }

    public HeadlineProducer() {
        System.out.println("HeadlineProducer\n");
        headlines = new String[13];
        domain = new String[13];
        headlines[0] = "Business news topic 1";
        domain[0] = "work";
        headlines[1] = "Business news headline";
        domain[1] = "work";
        headlines[2] = "Company server alert 1";
        domain[2] = "work";
        headlines[3] = "New Java game available";
        domain[3] = "home";
        headlines[4] = "Home alert: coffee machine timeout";
        domain[4] = "home";
        headlines[5] = "Home alert: lights timeout";
        domain[5] = "home";
        headlines[6] = "You have a call waiting";
        domain[6] = "work";
        headlines[7] = "You have received a message";
        domain[7] = "home";
        headlines[8] = "Company stock notification";
        domain[8] = "work";
        headlines[9] = "refrigerator@home: no milk";
        domain[9] = "home";
        headlines[10] = "4 buddies online";
        domain[10] = "home";
        headlines[11] = "New business service available";
        domain[11] = "work";
        headlines[12] = "Priority message arrived";
        domain[12] = "work";

        // periodic publication of events
        int i = 0;
        for (; ; ) {
            try {
                Notification n = new Notification();
                n.setType("demochannel");
                n.add(new EventTriplet("number", i++));
                int j = (int) (Math.random() * headlines.length);
                n.add(new EventTriplet("data", headlines[j]));
                n.add(new EventTriplet("domain", domain[j]));

                System.out.println("Publishing event");
                publish(n);
                Thread.sleep(3000);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
            }
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
