/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo;

import java.awt.*;
import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;
import fuegocore.notify.core.server.*;
import fuegocore.notify.demo.visualize.*;
import fuegocore.notify.profile.*;

/**
 * Fuego Core 2004 demonstration main engine
 * Demonstrates a shared session.
 * Sasu Tarkoma
 */

public class MainDemo extends EventBase {
    private static final String TICKER =
            "Fuego Core Demonstration -- 2004 -- " +
            "Helsinki Institute for Information Technology";

    DemoFrame sc = null; // GUI
    static boolean skipWait = false;
    public static MainDemo mainref;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("democlient");

        try {
            if (args != null && args.length > 1 && args[0] != null && args[1] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        try {
            if (args != null && args.length > 2) {
                if (args[2] != null && args[2].indexOf("skip") != -1) {
                    System.out.println("Skip wait is true");
                    skipWait = true;
                }
            }
        } catch (Exception e) {}

        // kick server
        fuegocore.notify.StartServer.main(new String[] {"0"});
        // kick demo
        MainDemo main = new MainDemo();
        mainref = main;
    }

    public MainDemo() {
        System.out.println("Starting GUI");
        Frame f = new Frame();
        f.setSize(1024, 760);
        sc = new DemoFrame(TICKER, 1024, 700);

        // Get the connection to the demo client

        /* **************************************************************** */
        // Subscribe events that we do not currently use

        open();

        Template t0 = new Template();
        t0.setSource(this);

        Filter f0 = new Filter("type", EventTriplet.TYPE_STRING,
                               "demochannel");
        Filter f1 = new Filter("subtype", EventTriplet.TYPE_STRING,
                               "equ lunch");
        t0.addFilter(f0);
        t0.addFilter(f1);

        subscribe(t0);

        Template t1 = new Template();
        t1.setSource(this);

        Filter f2 = new Filter("type", EventTriplet.TYPE_STRING,
                               "demochannel");
        Filter f3 = new Filter("subtype", EventTriplet.TYPE_STRING,
                               "equ arrival");
        t1.addFilter(f2);
        t1.addFilter(f3);
        subscribe(t1);

        /* **************************************************************** */

        // Get animator
        Animator anim = sc.getAnimator();

        // Set script for the Fuego demo
        // coworker right-top corner
        anim.push(new AnimEvent(0, AnimEvent.SETPOINT, 1, 45000 + 24000, 610,
                                394));

        // coworker near elevator
        anim.push(new AnimEvent(0, AnimEvent.SETPOINT, 0, 46000 + 24000, 191,
                                382));

        // elevator
        anim.push(new AnimEvent(0, AnimEvent.MOVETO, 0, 4000, 77, 592, 77,
                                453));
        anim.push(new AnimEvent(4000, AnimEvent.MOVETO, 0, 4000, 77, 443, 122,
                                413));
        // enter building event

        anim.push(new AnimEvent(8000, AnimEvent.SETPOINT, 0, 3000, 122, 413));
        anim.push(new AnimEvent(8000, AnimEvent.EXPANDRING, 0, 3000, 122, 413,
                                50,
                                100));
        anim.push(new AnimEvent(8000, AnimEvent.SEND, 122, 413,
                                "Send Arrival"));

        // Actual publish event
        anim.push(new AnimEvent(8000, AnimEvent.PUB_ARRIVAL, 0, 0, ""));

        // coworker #1
        anim.push(new AnimEvent(9000, AnimEvent.COMPRESSRING, 0, 3000, 191,
                                382,
                                100, 30));
        anim.push(new AnimEvent(9000, AnimEvent.RECEIVE, 191, 382,
                                "Receive Arrival"));

        // coworker #2
        anim.push(new AnimEvent(9000, AnimEvent.COMPRESSRING, 0, 3000, 610,
                                394,
                                100, 30));
        anim.push(new AnimEvent(9000, AnimEvent.RECEIVE, 610, 394,
                                "Receive Arrival"));

        // continue movement
        anim.push(new AnimEvent(11000, AnimEvent.MOVETO, 0, 4000, 122, 413,
                                172,
                                457));

        anim.push(new AnimEvent(15000, AnimEvent.MOVETO, 0, 4000, 172, 457,
                                334,
                                368));
        anim.push(new AnimEvent(19000, AnimEvent.MOVETO, 0, 4000, 334, 368,
                                438,
                                315));
        anim.push(new AnimEvent(23000, AnimEvent.MOVETO, 2, 4000, 438, 315,
                                371,
                                291));

        // stationary
        anim.push(new AnimEvent(27000, AnimEvent.SETPOINT, 2, 18000 + 24000,
                                371,
                                291));

        // ring
        anim.push(new AnimEvent(27000, AnimEvent.MOVERINGTO, 0, 3000, 371,
                                291, 341,
                                317, 50));
        anim.push(new AnimEvent(30000, AnimEvent.DRAWRING, 0, 3000, 341, 317,
                                341,
                                317, 50));
        anim.push(new AnimEvent(30000, AnimEvent.SEND, 341, 317,
                                "End-point change"));

        // Actual publish event
        anim.push(new AnimEvent(30000, AnimEvent.PUB_CONTEXT, 0, 0, ""));

        anim.push(new AnimEvent(33000, AnimEvent.EXPANDRING, 0, 3000, 341,
                                317, 50,
                                100));

        // Shift time a bit
        anim.setOffset(24000);

        anim.push(new AnimEvent(40000, AnimEvent.EXPANDRING, 0, 3000, 191,
                                382, 50,
                                100));

        anim.push(new AnimEvent(40000, AnimEvent.SEND, 191, 382,
                                "Send Start Meeting"));

        // Actual publish event
        anim.push(new AnimEvent(40000, AnimEvent.PUB_MEETING, 0, 0, ""));

        // coworker #1
        anim.push(new AnimEvent(41000, AnimEvent.COMPRESSRING, 0, 3000, 371,
                                291,
                                100, 30));
        anim.push(new AnimEvent(41000, AnimEvent.RECEIVE, 371, 291,
                                "Receive Start Meeting"));

        // coworker #2
        anim.push(new AnimEvent(41000, AnimEvent.COMPRESSRING, 0, 3000, 610,
                                394,
                                100, 30));
        anim.push(new AnimEvent(41000, AnimEvent.RECEIVE, 610, 394,
                                "Receive Start Meeting"));

        // Client #1 mobility
        anim.push(new AnimEvent(45000, AnimEvent.SETPOINT, 2, 25000, 371, 291));

        // Guy near elevator movement to the meeting area
        anim.push(new AnimEvent(46000, AnimEvent.MOVETO, 0, 4000, 191, 382,
                                276,
                                407));
        anim.push(new AnimEvent(50000, AnimEvent.MOVETO, 0, 4000, 276, 407,
                                411,
                                360));
        anim.push(new AnimEvent(54000, AnimEvent.MOVETO, 0, 4000, 411, 360,
                                464,
                                470));
        anim.push(new AnimEvent(58000, AnimEvent.MOVETO, 2, 4000, 464, 470,
                                349,
                                490));

        // co-worker mobility right-top to meeting area
        anim.push(new AnimEvent(45000, AnimEvent.MOVETO, 1, 4000, 610, 394,
                                476,
                                322));
        anim.push(new AnimEvent(49000, AnimEvent.MOVETO, 1, 4000, 476, 322,
                                410,
                                390));
        anim.push(new AnimEvent(53000, AnimEvent.MOVETO, 3, 4000, 410, 390,
                                471,
                                464));
        anim.push(new AnimEvent(57000, AnimEvent.MOVETO, 1, 4000, 471, 464,
                                353,
                                409));

        // ring, change session back to mobile client
        anim.push(new AnimEvent(65000, AnimEvent.MOVERINGTO, 0, 3000, 341,
                                317, 371,
                                291, 50));
        anim.push(new AnimEvent(68000, AnimEvent.EXPANDRING, 0, 3000, 371,
                                291, 50,
                                100));

        // Actual publish event
        anim.push(new AnimEvent(65000, AnimEvent.PUB_CONTEXT2, 0, 0, ""));

        anim.push(new AnimEvent(70000, AnimEvent.MOVETO, 0, 4000, 371, 291,
                                446,
                                340));
        anim.push(new AnimEvent(74000, AnimEvent.MOVETO, 0, 4000, 446, 340,
                                451,
                                467));
        anim.push(new AnimEvent(78000, AnimEvent.MOVETO, 2, 4000, 451, 467,
                                351,
                                432));
        anim.push(new AnimEvent(82000, AnimEvent.MOVETO, 2, 4000, 351, 432,
                                289,
                                457));

        // meeting scenario
        // right-top
        anim.push(new AnimEvent(61000, AnimEvent.SETPOINT, 3, 89000, 353, 409));

        anim.push(new AnimEvent(100000, AnimEvent.EXPANDRING, 0, 3000, 353,
                                409, 50,
                                100));
        anim.push(new AnimEvent(100000, AnimEvent.SEND, 353, 409,
                                "Synchronize"));

        // Actual publish event
        anim.push(new AnimEvent(100000, AnimEvent.PUB_SYNC, 0, 0, ""));

        // elevator
        anim.push(new AnimEvent(62000, AnimEvent.SETPOINT, 2, 83000, 349, 490));
        anim.push(new AnimEvent(103000, AnimEvent.COMPRESSRING, 2, 3000, 349,
                                490,
                                100, 30));
        anim.push(new AnimEvent(103000, AnimEvent.RECEIVE, 349, 490,
                                "Receive Synchronize"));

        // workstation receive sync
        anim.push(new AnimEvent(103000, AnimEvent.COMPRESSRING, 0, 3000, 341,
                                317,
                                100, 30));
        anim.push(new AnimEvent(103000, AnimEvent.RECEIVE, 341, 317,
                                "Receive Synchronize"));

        // main dude
        anim.push(new AnimEvent(86000, AnimEvent.SETPOINT, 0, 64000, 289, 457));

        anim.push(new AnimEvent(103000, AnimEvent.COMPRESSRING, 0, 3000, 289,
                                457,
                                100, 30));
        anim.push(new AnimEvent(103000, AnimEvent.RECEIVE, 289, 457,
                                "Receive Synchronize"));

        // the meeting ends at 150 000 msec

        // go away right-top
        anim.push(new AnimEvent(150000, AnimEvent.MOVETO, 3, 4000, 353, 409,
                                513,
                                435));
        anim.push(new AnimEvent(154000, AnimEvent.SETPOINT, 1, 100000, 513,
                                435));

        // go away elevator guy
        anim.push(new AnimEvent(145000, AnimEvent.MOVETO, 2, 4000, 349, 490,
                                294,
                                510));
        anim.push(new AnimEvent(149000, AnimEvent.MOVETO, 2, 4000, 294, 510,
                                253,
                                475));
        anim.push(new AnimEvent(153000, AnimEvent.SETPOINT, 2, 100000, 253,
                                475));

        // go away main guy
        anim.push(new AnimEvent(150000, AnimEvent.MOVETO, 0, 4000, 289, 457,
                                385,
                                431));
        anim.push(new AnimEvent(154000, AnimEvent.MOVETO, 0, 4000, 385, 431,
                                473,
                                434));
        anim.push(new AnimEvent(158000, AnimEvent.MOVETO, 2, 4000, 473, 434,
                                354,
                                353));
        anim.push(new AnimEvent(162000, AnimEvent.MOVETO, 2, 4000, 354, 353,
                                168,
                                451));

        anim.push(new AnimEvent(166000, AnimEvent.EXPANDRING, 0, 3000, 168,
                                451, 50,
                                100));
        anim.push(new AnimEvent(166000, AnimEvent.SEND, 168, 451,
                                "I'm going to Pizzeria," +
                                "care to join?"));

        // Actual publish event
        anim.push(new AnimEvent(166000, AnimEvent.PUB_LUNCH, 0, 0, ""));

        anim.push(new AnimEvent(166000, AnimEvent.SETPOINT, 2, 8000, 168, 451));

        anim.push(new AnimEvent(170000, AnimEvent.COMPRESSRING, 0, 3000, 513,
                                435,
                                100, 30));
        anim.push(new AnimEvent(170000, AnimEvent.RECEIVE, 513, 435,
                                "Receive context msg"));

        anim.push(new AnimEvent(174000, AnimEvent.MOVETO, 2, 4000, 168, 451,
                                118,
                                424));

        // use elevator
        anim.push(new AnimEvent(178000, AnimEvent.MOVETO, 2, 4000, 118, 424,
                                77,
                                443));
        anim.push(new AnimEvent(182000, AnimEvent.MOVETO, 2, 4000, 77, 443,
                                77, 592));

        anim.push(new AnimEvent(186000, AnimEvent.RESTART, 0, 0, "Restart"));

        anim.finalizeAnim();
        anim.setOffset(0);

        // Finally, add to frame and make the frame visible
        f.add(sc);

        f.setVisible(true);

        EventServer eb = EventServer.get();
        ConnectionObject co = null;

        UserProfile.demoRef = this;
        // Wait until we have the session
        if (!skipWait) {
            do {
                try {
                    synchronized (this) {
                        wait(1000);
                    }
                } catch (Exception e) {}
                Hashtable a = eb.getConnections();
                co = (ConnectionObject) a.get("shared_session");
            } while (co == null);
            sc.setCO(co);
            // we wait a bit for the phone to get the connection going
            try {
                synchronized (this) {
                    wait(11000);
                }
            } catch (Exception e) {}
        }
        System.out.println("Kicking demo");
        sc.kick();
    }

    /* Callback for matched templates */
    public void matchedTemplates(Vector v, Notification n) {
        sc.matchedTemplates(v, n);
    }

    // Overrided for local operations
    public boolean publish(Notification n) {
        EventServer eb = EventServer.get();
        eb.getRoutingEngine().publish(n);
        return true;
    }

    // Overrided for local operations
    public boolean subscribex(Template t) {
        EventServer eb = EventServer.get();
        eb.getRoutingEngine().subscribe(t);
        return true;
    }

}
