/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo.simple;

import java.awt.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;
import fuegocore.notify.core.server.*;
import fuegocore.notify.demo.visualize.*;

/**
 * Simple example of event-based visualization. This class demonstrates
 * the basic animation features, integration with distributed pub/sub,
 * and reactive animation events.
 * Sasu Tarkoma
 */

public class SimpleAnim extends EventBase {
    static SimpleCanvas sc = null;
    static boolean skipWait = false;
    public static SimpleAnim mainref;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("simpleanim");

        try {
            if (args != null && args.length > 1 && args[0] != null && args[1] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        // kick server
        fuegocore.notify.StartServer.main(new String[] {"0"});

        // kick demo
        SimpleAnim main = new SimpleAnim();
        mainref = main;
    }

    public SimpleAnim() {
        System.out.println("Starting GUI");

        Frame f = new Frame();
        f.setSize(640, 480);
        sc = new SimpleCanvas(640, 480);

        // Get the connection to the demo client

        /* **************************************************************** */
        // Subscribe events that we do not currently use

        open();

        Template t0 = new Template();
        t0.setSource(this);

        Filter f0 = new Filter("type", EventTriplet.TYPE_STRING,
                               "demochannel");
        Filter f1 = new Filter("x", EventTriplet.TYPE_INT,
                               "rng 50 600");
        Filter f2 = new Filter("y", EventTriplet.TYPE_INT,
                               "rng 50 400");
        t0.addFilter(f0);
        t0.addFilter(f1);
        t0.addFilter(f2);

        subscribe(t0);

        /* **************************************************************** */

        // Get animator
        Animator anim = sc.getAnimator();

        anim.push(new SimpleAnimEvent(0, SimpleAnimEvent.MOVETO, 0, 4000, 77,
                                      592,
                                      77,
                                      453));
        anim.push(new SimpleAnimEvent(4000, SimpleAnimEvent.MOVETO, 0, 4000, 77,
                                      443, 122,
                                      413));
        // enter building event

        anim.push(new SimpleAnimEvent(8000, SimpleAnimEvent.SETPOINT, 0, 3000,
                                      122,
                                      413));
        anim.push(new SimpleAnimEvent(8000, SimpleAnimEvent.EXPANDRING, 0, 3000,
                                      122, 413,
                                      50,
                                      100));
        anim.push(new SimpleAnimEvent(8000, SimpleAnimEvent.SEND, 122, 413,
                                      "Send Arrival"));

        // Actual publish event
        anim.push(new SimpleAnimEvent(8000, SimpleAnimEvent.PUB_ARRIVAL, 0, 0,
                                      ""));

        // coworker #1
        anim.push(new SimpleAnimEvent(9000, SimpleAnimEvent.COMPRESSRING, 0,
                                      3000,
                                      191,
                                      382,
                                      100, 30));
        anim.push(new SimpleAnimEvent(9000, SimpleAnimEvent.RECEIVE, 191, 382,
                                      "Receive Arrival"));

        // coworker #2
        anim.push(new SimpleAnimEvent(9000, SimpleAnimEvent.COMPRESSRING, 0,
                                      3000,
                                      610,
                                      394,
                                      100, 30));
        anim.push(new SimpleAnimEvent(9000, SimpleAnimEvent.RECEIVE, 610, 394,
                                      "Receive Arrival"));

        // use elevator
        anim.push(new SimpleAnimEvent(11000, SimpleAnimEvent.MOVETO, 1, 4000,
                                      122,
                                      413,
                                      77,
                                      443));
        anim.push(new SimpleAnimEvent(15000, SimpleAnimEvent.MOVETO, 1, 4000,
                                      77,
                                      443,
                                      77, 592));

        anim.push(new SimpleAnimEvent(19000, SimpleAnimEvent.RESTART, 0, 0,
                                      "Restart"));

        anim.finalizeAnim();
        anim.setOffset(0);

        // Finally, add to frame and make the frame visible
        f.add(sc);

        f.setVisible(true);

        EventServer eb = EventServer.get();
        ConnectionObject co = null;

        System.out.println("Kicking demo");
        sc.kick();
    }

    // Incoming notifications
    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
        int x = 0, y = 0;

        // Retrieve the tuple value
        EventTriplet et = (EventTriplet) n.getField(EventTriplet.formatId("x",
                EventTriplet.TYPE_INT));
        if (et != null) {
            x = ((Integer) et.getValue()).intValue();
        }
        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId("y",
                EventTriplet.TYPE_INT));
        if (et != null) {
            y = ((Integer) et.getValue()).intValue();
        }
        // insert asynchronous animation event
        SimpleAnimEvent sim =
                new SimpleAnimEvent(sc.getAnimationTime(),
                                    SimpleAnimEvent.COMPRESSRING, 0, 5000, x,
                                    y,
                                    80, 10);

        sc.getAnimator().pushReactive(sim);

    }

    // Overrided for local operations
    public static boolean publishx(Notification n) {
        EventServer eb = EventServer.get();
        eb.getRoutingEngine().publish(n);
        return true;
    }

    // Overrided for local operations
    public boolean subscribex(Template t) {
        EventServer eb = EventServer.get();
        eb.getRoutingEngine().subscribe(t);
        return true;
    }

}
