/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo.simple;

import java.awt.*;

import fuegocore.notify.core.*;
import fuegocore.notify.demo.visualize.*;

/**
 * Represents one activity in the animation. May be non-graphical
 * activity, for example publishing an event
 */
public class SimpleAnimEvent extends AnimEvent {

    public static final int PUB_ARRIVAL = 9;

    public static final int NO_CODE = -99;

    /**
     * Create a new animation event
     * @param time int time of occurrence
     * @param code int event code
     * @param x int x-coordinate
     * @param y int y-coordinate
     * @param s String textual parameter
     */
    public SimpleAnimEvent(int time, int code, int x, int y, String s) {
        super(time, code, x, y, s);
    }

    /**
     * Create a new animation event
     * @param time int time of occurrence
     * @param code int event code
     * @param image int image number
     * @param duration int duration of event
     * @param x int x-coordinate
     * @param y int y-coordinate
     */
    public SimpleAnimEvent(int time, int code, int image, int duration, int x,
                           int y) {
        super(time, code, image, duration, x, y);
    }

    /**
     * Create a new animation event
     * @param time int
     * @param code int
     * @param image int
     * @param duration int
     * @param x int
     * @param y int
     * @param tx int
     * @param ty int
     */
    public SimpleAnimEvent(int time, int code, int image, int duration, int x,
                           int y, int
                           tx, int ty) {
        super(time, code, image, duration, x, y, tx, ty);
    }

    /**
     * Create a new animation event
     * @param time int
     * @param code int
     * @param image int
     * @param duration int
     * @param x int
     * @param y int
     * @param tx int
     * @param ty int
     * @param r int
     */
    public SimpleAnimEvent(int time, int code, int image, int duration, int x,
                           int y, int
                           tx, int ty, int r) {
        super(time, code, image, duration, x, y, tx, ty, r);
    }

    /**
     * Animation extensions. Add your routines here. Use unique event codes.
     * @param curtime int
     * @param g Graphics2D
     * @param offsetX int
     * @param offsetY int
     * @param ir ImageResolver
     * @return int
     */
    public int animateExtensions(int curtime, Graphics2D g, int offsetX,
                                 int offsetY,
                                 ImageResolver ir) {
        int ret = NO_CODE;

        switch (getCode()) {
        // Publish events (no visuals)
        case RESTART:
            System.out.println("Restarting demo");
            ret = EXITLOOP;
            SimpleCanvas.restart();
            break;
        case PUB_ARRIVAL:
            Notification n = new Notification();
            n.setType("demochannel");
            n.add(new EventTriplet("user", "demo"));
            n.add(new EventTriplet("subtype", "arrival"));

            if (!SimpleAnim.publishx(n)) {
                System.out.println("Publish of arrival event failed");
            }
            ret = DONE;
            break;
        }
        return ret;
    }
}
