/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo.simple;

import fuegocore.notify.core.*;

/**
 * Test producer class for the simple animation example
 */
public class SimpleProducer extends EventBase {
    String[] headlines;
    String[] domain;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName(
                "simpleproducer");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        SimpleProducer test = new SimpleProducer();
    }

    public SimpleProducer() {
        // periodic publication of events
        int i = 0;
        for (; ; ) {
            try {
                Notification n = new Notification();
                n.setType("demochannel");

                int x = (int) (Math.random() * 640);
                int y = (int) (Math.random() * 480);

                n.add(new EventTriplet("x", x));
                n.add(new EventTriplet("y", y));

                System.out.println("Publishing event");
                publish(n);
                Thread.sleep(200);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
            }
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
