/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.demo.visualize;

import java.awt.*;
import java.util.*;

/**
 * Implements the animation data storate
 */
public class Animator {
    ImageResolver ir;
    /** Vector for storing replay objects */
    public Vector replay = new Vector();

    /** Vector for storing the animation */
    public Vector anim = new Vector();

    int offset = 0;

    /**
     * Add an animation event to the current time. Offset is incremented.
     * @param ev AnimEvent
     */
    public void push(AnimEvent ev) {
        ev.time += offset;
        anim.add(ev);
    }

    /**
     * Add an animation event to the current time. Offset is not incremented.
     * @param ev AnimEvent
     */
    public void pushReactive(AnimEvent ev) {
        anim.add(ev);
        sort();
    }

    /**
     * Set offset
     * @param off int
     */
    public void setOffset(int off) {
        offset = off;
    }

    /**
     * Finalize animation for playback
     */
    public void finalizeAnim() {
        replay.addAll(anim);
    }

    /**
     * Start replay
     */
    public void replay() {
        anim.clear();
        anim.addAll(replay);
        System.out.println("added " + replay.size());
    }

    /**
     * Create a new animator
     * @param r ImageResolver for resolving numbers into images
     */
    public Animator(ImageResolver r) {
        ir = r;
    }

    /**
     * Sort the animation script
     */
    public void sort() {
        Collections.sort(anim);
    }

    /**
     * Remove the next visualization event
     * @return AnimEvent
     */
    public AnimEvent pop() {
        if (anim.size() == 0) {
            return null;
        }
        return (AnimEvent) anim.remove(0);
    }

    /**
     * Shift time by the given amount
     * @param time int
     * @param amount int
     */
    public void shift(int time, int amount) {
        for (int i = 0; i < anim.size(); i++) {
            AnimEvent ev = (AnimEvent) anim.elementAt(i);
            if (ev.time >= time) {
                ev.time += amount;
            }
        }
    }

    /**
     * Animate one frame given time, Graphics context and offsets
     * @param time int
     * @param g Graphics2D
     * @param offsetX int
     * @param offsetY int
     */
    public void animate(int time, Graphics2D g, int offsetX, int offsetY) {
        boolean f = false;
        int j = 0;
        Vector pushback = new Vector();

        do {
            AnimEvent ev = pop(); // remove item from queue
            if (ev == null) {
                f = true;
                break;
            }
            int i = ev.draw(time, g, offsetX, offsetY, ir);
            if (i == AnimEvent.EXPIRED) {
                ;
            } else if (i == AnimEvent.NOTYET) {
                pushback.add(ev);
            } else if (i == AnimEvent.STILLTOGO) {
                pushback.add(ev);
            } else if (i == AnimEvent.EXITLOOP) {
                break;
            } else {
                ev.reset(); // reset for the next round
            }
            j++;
        } while (!f);

        for (int i = 0; i < pushback.size(); i++) {
            AnimEvent ev = (AnimEvent) pushback.elementAt(i);
            push(ev);
        }
    }

}
