/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.profile;

import java.net.*;

import org.apache.axis.server.*;

/**
 * Communications Profile
 */
public class CommProfile {
    /** Ping port if ping service is used */
    public static int pingPort = 7777;

    /** Ping interval in milliseconds */
    public static int pingInterval = 5000;

    /** Determines if ping is active if the ping service is deployed */
    public static boolean pingActive = false;

    /** Gateway port */
    public static int gatewayPort = 22;

    /** D211 path (to executable file, only on Windows) */
    public static String D211Path = "C:\\dev\\java\\src\\history.exe";
    public static AxisServer axisengine; // reference to Axis engine

    /** current access server id */
    public static String access_server_id = "nodef";

    /**  current event server id */
    public static String event_server_id = "nodef"; //

    /**  for demo (roam button) */
    public static String roam_server_id = "nodef";

    private static String address = null;

    /**
     *  Returns the IP address and port of the event service on the current host
     * @return String
     */
    public static String getAddress() {
        if (address == null || address == "") {
            try {
                InetAddress th = InetAddress.getLocalHost();
                address = th.getHostAddress();
                address += ":" + gatewayPort;
            } catch (java.net.UnknownHostException eh) {
                System.out.println("Exception: " + eh);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
            }
        }
        return address;
    }

    /**
     * Returns the D211 executable path
     * @return String
     */
    public static String getD211Path() {
        return D211Path;
    }

    /**
     * Returns the ping port.
     * @return int
     */
    public static int getPingPort() {
        return pingPort;
    }

    /**
     * Returns the ping interval
     * @return int
     */
    public static int getPingInterval() {
        return pingInterval;
    }

    /**
     * Returns true if ping is active
     * @return boolean
     */
    public static boolean getPingActive() {
        return pingActive;
    }

}
