/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.profile;

import java.net.*;

import fuegocore.notify.demo.*;

/**
 * User Profile. This is currently used to store also system
 * configuration.
 */
public class UserProfile {
    /** Client user id, used in identifying sessions */
    public static String name = "test_name";

    /** Debug GUI flag */
    public static boolean debugGUI = false; // GUI active

    /** Status query active (Quench), currently not working  */
    public static boolean queryStatus = false;

    /** Debug active: transport  */
    public static boolean debugTransport = false;

    /** Debug active: filter  */
    public static boolean debugFilter = false;

    /** Debug active: filter service  */
    public static boolean debugFilterService = false;

    /** Callback for demo */
    public static MainDemo demoRef = null;

    /** Debug active: general  */
    public static boolean debugGeneral = true;

    /** Debug active: event channels  */
    public static boolean debugEC = true;

    /** Send merged filters to event channel. Note that subCover may not be used at the same time */
    public static boolean mergeEC = true;

    /* Lazy unsubscriptions. Not currently used. */
    public static boolean lazyUpdates = false;

    /** Use covering relations for subscription management. Note that merge may not be used at the same time */

    /** Experimental */
    public static boolean subCover = false;

    /** Not implemented  */
    public static boolean handleErrors = false;

    public static String DEF_MSG_PROTOCOL = "mop";

    /** Last server that was used (used in handover management) */
    public static String last_server = "-";

    /** SOAP (gateway) URL */
    public static URL soap_url;

    /** SOAP host as string */
    public static String soap_url_host;

    /**
     *  Set Client Destination SOAP URL
     * @return URL
     */
    public static URL getSOAPURL() {
        return soap_url;
    }

    /**
     * Set Client Destination SOAP URL
     * @return String
     */
    public static String getSOAPHost() {
        return soap_url_host;
    }

    /**
     *  Set Client Destination SOAP URL
     * @param host URL
     */
    public static void setSOAPURL(URL host) {
        soap_url = host;
    }

    /**
     * Set Client Destination SOAP URL
     * @param host String
     */
    public static void setSOAPURLHost(String host) {
        soap_url_host = host;
    }

    /**
     * Returns the name of the last server.
     * @return String
     */
    public static String getLastServer() {
        return last_server;
    }

    /**
     * Sets the name of the last server.
     * @param s String
     */
    public static void setLastServer(String s) {
        last_server = s;
    }

    /**
     * Returns the name of the user.
     * @return String
     */
    public static String getName() {
        return name;
    }

    /**
     *  Sets the name of the user.
     * @param s String
     */
    public static void setName(String s) {
        name = s;
    }

    /**
     *  Returns true if debug GUI is activated.
     * @return boolean
     */
    public static boolean getDebugGUI() {
        return debugGUI;
    }

    /**
     * Get query status mode (query event channel status from the server)
     * @return boolean
     */
    public static boolean getQueryStatus() {
        return queryStatus;
    }

    /**
     * Set query status mode (query event channel status from the server)
     * @param b boolean
     */
    public static void setQueryStatus(boolean b) {
        queryStatus = b;
    }

}
