/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.sync.client;

import java.util.Vector;

/**
 * Interfaces for synchronized spaces (collections).
 * A synchronized space allows to identify an object with a filter,
 * and query objects with specific filters. Thus context-based synchronization
 * is supported.
 */

public interface ISyncSpace {

    /**
     * Add an object into the synced space
     * @param t Object the filter (context profile) must contain URI!
     * @return boolean
     */
    public boolean add(Object t);

    /**
     * Remove a previously inserted object
     * @param t Object (the template to be removed)
     * @return boolean
     */
    public boolean remove(Object t);

    /**
     * Delete the collection
     * @return boolean
     */
    public boolean delete();

    /**
     * Add the given query
     * @param t Object
     * @return boolean
     */
    public boolean addQuery(Object t);

    /**
     * Remove the given query
     * @param t Object
     * @return boolean
     */
    public boolean removeQuery(Object t);

    /**
     * Register a listener
     * @param <any> ISyncCallback
     * @return boolean
     */
    public boolean addListener(ISyncCallback b);

    /**
     * Remove a listener
     * @param <any> ISyncCallback
     * @return boolean
     */
    public boolean removeListener(ISyncCallback b);

    /**
     * Returns the sync collection as ITemplates (must contain URIs)
     * @return Vector
     */
    public Vector getCollection();
}
