/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.sync.client;

import fuegocore.notify.core.*;
import fuegocore.notify.sync.service.SyncSpace;

/**
 * Producer for sync updates
 */
public class SyncClientProducer extends EventBase {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName(
                "SyncProducer");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        SyncClientProducer prod = new SyncClientProducer();
    }

    public SyncClientProducer() {
        System.out.println("SyncProducer\n");

        SyncSpace test = new SyncSpace(this);

        // periodic publication of events
        for (; ; ) {
            try {
                System.out.println("Producing image event 1");
                Notification n = createRandomDescription();
                test.add(n);
                Thread.sleep(10000);
                System.out.println("Producing image event 2");
                Notification n2 = createRandomDescription();
                test.add(n2);
                Thread.sleep(10000);
                System.out.println("Removing image 1");
                test.remove(n);
                Thread.sleep(10000);
                System.out.println("Removing image 2");
                test.remove(n2);
                Thread.sleep(10000);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
            }
        }
    }

    public Notification createRandomDescription() {
        Notification desc = new Notification();

        desc.setType("picture");
        desc.add(new EventTriplet("id", "123"));
        desc.add(new EventTriplet("category", "hobby"));
        desc.add(new EventTriplet("keywords", "keywords"));

        // ADD FILE URI HERE
        desc.add(new EventTriplet("URI", "http://koe"));

        desc.add(new EventTriplet("location.city", "Helsinki"));
        desc.add(new EventTriplet("location.street", "Mannerheimintie 10"));

        // random location for testing matching
        int x = (int) (Math.random() * 10);
        int y = (int) (Math.random() * 10);
        desc.add(new EventTriplet("location.lon", x));
        desc.add(new EventTriplet("location.lat", y));

        return desc;
    }
}
