/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.sync.client;

import java.util.Vector;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.Filter;
import fuegocore.notify.sync.service.SyncSpace;

/**
 * Runs a client of the SyncSpace system
 */
public class SyncClientSubscriber extends EventBase implements ISyncCallback {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName(
                "SyncSpace");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        "localhost", 10000);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        SyncClientSubscriber sub = new SyncClientSubscriber();
    }

    public SyncClientSubscriber() {
        System.out.println("SyncSpace\n");
        SyncSpace test = new SyncSpace(this);

        System.out.println("Subscribing");
        Template t = createRandomProfile();
        System.out.println(t.toString());
        test.addQuery(t);
        test.addListener(this);
        SyncThread st = new SyncThread(test);
        st.start();
    }

    /**
     * Called when collection has been updated. Useful for updating
     * user interfaces
     * @param object Object the collection
     */
    public void collectionChanged(Object object) {
        SyncSpace sp = (SyncSpace) object;
        Vector collection = sp.getCollection();
        System.out.println("New collection:");
        if (collection != null) {
            for (int i = 0; i < collection.size(); i++) {
                Notification n = (Notification) collection.elementAt(i);
                System.out.println(n.toString());

                // Retrieve the tuple value
                EventTriplet et = (EventTriplet) n.getField(EventTriplet.
                        formatId(
                                "URI", EventTriplet.TYPE_STRING));
                String uri = (String) et.getValue();

                System.out.println("Extracted URI:" + uri);
            }
        }
    }

    /**
     * Create a random profile
     * @return Template
     */
    public Template createRandomProfile() {
        Template t1 = new Template();
        // Add attribute filters to the template
        Filter f1 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ picture");
        Filter f2 = new Filter("category", EventTriplet.TYPE_STRING,
                               "equ hobby");
        // random location for testing matching
        int x = (int) (Math.random() * 10);
        int y = (int) (Math.random() * 10);

        // to ease the matching we allow any second value
        Filter f3 = new Filter("location.lon", EventTriplet.TYPE_INT,
                               "rng " + x + " " + (x + 5));
        Filter f4 = new Filter("location.lat", EventTriplet.TYPE_INT,
                               "rng 0 10");

        t1.addFilter(f1);
        t1.addFilter(f2);
        t1.addFilter(f3);
        t1.addFilter(f4);
        return t1;
    }

}


class SyncThread extends Thread {
    SyncSpace sp;
    boolean quit = false;

    public SyncThread(SyncSpace p) {
        sp = p;
    }

    public void run() {
        while (!quit) {
            try {
                Thread.sleep(30000);
                Vector syncSet = new Vector();
                Vector collection = sp.getCollection();
                if (collection != null) {
                    for (int i = 0; i < collection.size(); i++) {
                        Notification n = (Notification) collection.elementAt(i);
                        // Retrieve the tuple value
                        EventTriplet et = (EventTriplet) n.getField(
                                EventTriplet.formatId(
                                        "URI", EventTriplet.TYPE_STRING));
                        String uri = (String) et.getValue();
                        syncSet.add(uri);
                    }
                    // ADD YOUR SYNCHRONIZER HERE
                    // Synchronizer.sync(syncSet);
                    System.out.println("**** SyncSet:" + syncSet);
                }

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}
