/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */
package fuegocore.notify.sync.service;

import fuegocore.notify.core.*;

/** Listener for profile updates */
public class ProfileChangeListener implements Notifiable {
    int count = 0;
    SyncService _eb;

    public ProfileChangeListener(SyncService eb) {
        _eb = eb;
    }

    public int getCount() {
        return count;
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Received Restaurant Description");
        System.out.println("Notification:\n" + n);
        count++;

        // Retrieve the tuple value
        EventTriplet et = (EventTriplet) n.getField(EventTriplet.formatId(
                "operation",
                EventTriplet.TYPE_STRING));

        if (et == null) {
            System.out.println("Op was null");
            return;
        }

        String op = (String) et.getValue();

        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId("data",
                EventTriplet.TYPE_SERIALIZED));
        Notification desc = null;

        if (et != null) {
            desc = (Notification) et.getValue();
            System.out.println("*** received: " + desc);
        }

        if (desc == null) {
            System.out.println("ProfileChangeListener: data was null");
            return;
        }

        if (op.equals("announce-add")) {
            synchronized (_eb.getServices()) {
                _eb.getServices().add(desc);
            }
            // Match profiles against the new description
            _eb.matchProfilesAgainstService(desc);
        } else if (op.equals("announce-del")) {
            synchronized (_eb.getServices()) {
                // We have to do a string comparison, the reference is not the same
                for (int i = 0; i < _eb.getServices().size(); i++) {
                    Notification n2 = (Notification) _eb.getServices().
                                      elementAt(i);
                    if (n2.toString().equals(desc.toString())) {
                        if (_eb.getServices().remove(n2)) {
                            System.out.println("Service removed");
                            // we need to notify client collections about this change
                            _eb.notifyProfileChange(n2);
                        }
                        break;
                    }
                }
            }
        }
    }
}
