/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */
package fuegocore.notify.sync.service;

import fuegocore.notify.core.*;

/** Listener for profile information */
public class QueryChangeListener implements Notifiable {
    int count = 0;
    SyncService _eb;

    public QueryChangeListener(SyncService eb) {
        _eb = eb;
    }

    public int getCount() {
        return count;
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Received Profile Change");
        System.out.println("Notification:\n" + n);
        count++;

        // Retrieve the tuple value
        EventTriplet et = (EventTriplet) n.getField(EventTriplet.formatId(
                "operation",
                EventTriplet.TYPE_STRING));

        if (et == null) {
            System.out.println("Op was null");
            return;
        }

        String op = (String) et.getValue();

        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId(
                "return-address",
                EventTriplet.TYPE_STRING));

        if (et == null) {
            System.out.println("Return address was null");
            return;
        }

        String address = (String) et.getValue();

        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId("data",
                EventTriplet.TYPE_SERIALIZED));

        Template profile = null;

        if (et != null) {
            profile = (Template) et.getValue();
            if (profile == null) {
                System.out.println("Empty record");
            } else {
                // Set the return address
                profile.setInterface(address);
            }
        }

        if (profile == null) {
            System.out.println("QueryChangeListener: data was null");
            return;
        }

        if (op.equals("profile-change-add")) {
            synchronized (_eb.getProfiles()) {
                _eb.getProfiles().add(profile);
            }
            // Match profiles against the new description
            _eb.matchServicesAgainstProfile(profile);
        } else if (op.equals("profile-change-del")) {
            // We have to do a string comparison, the reference is not the same
            synchronized (_eb.getProfiles()) {
                for (int i = 0; i < _eb.getProfiles().size(); i++) {
                    Template t2 = (Template) _eb.getProfiles().elementAt(i);
                    if (t2.toString().equals(profile.toString())) {
                        _eb.getProfiles().remove(t2);
                        System.out.println("Profile removed");
                        break;
                    }
                }
            }
        }
    }
}
