/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.custom;

import fuegocore.notify.core.*;
import fuegocore.notify.core.serialization.*;
import fuegocore.notify.test.custom.serialization.*;

/**
 * Runs a test event producer
 */
public class CustomPublisherExample extends EventBase {

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("producer");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        CustomPublisherExample test = new CustomPublisherExample();
    }

    public CustomPublisherExample() {
        System.out.println("CustomPublisherExample\n");

        // Register custom serializer
        SimpleTemplateSerializer simser = new SimpleTemplateSerializer();
        EventSerializationService.getService().register(
                "fuegocore.notify.test.custom.core.SimpleTemplate", simser);
        EventSerializationService.getService().register("SimpleTemplate",
                simser);

        // periodic publication of events
        int i = 0;
        for (; ; ) {
            try {
                System.out.println("Producing event with topic: " + i);
                Notification n = new Notification();
                n.add(new EventTriplet("topic", "Topic" + i++));
                n.add(new EventTriplet("data", "text data"));

                System.out.println("Publishing event");
                publish(n);
                Thread.sleep(200);
                if (i > 10) {
                    i = 0; // wrap
                }
            } catch (Exception e) {
                System.out.println("Exception: " + e);
            }
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
