/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.custom;

import fuegocore.notify.core.*;
import fuegocore.notify.core.serialization.*;
import fuegocore.notify.test.custom.core.*;
import fuegocore.notify.test.custom.serialization.*;

/**
 * Runs a test event subscriber.
 */

public class CustomSubscriberExample extends EventBase {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("customsub");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        CustomSubscriberExample test = new CustomSubscriberExample();
        System.out.println("CustomSubscriberExample\n");
    }

    public CustomSubscriberExample() {
        // Register custom serializer
        SimpleTemplateSerializer simser = new SimpleTemplateSerializer();
        EventSerializationService.getService().register(
                "fuegocore.notify.test.custom.core.SimpleTemplate", simser);
        EventSerializationService.getService().register("SimpleTemplate",
                simser);

        // Open session
        String sid = open();

        if (sid == null) {
            System.out.println("Session open failed");
        } else {
            System.out.println("Session opened. Id: " + sid);

        }
        System.out.println("Creating custom template 1");

        SimpleTemplate t1 = new SimpleTemplate();

        // Define a listener class
        SubListener cl = new SubListener(this);
        t1.setSource(cl); // set the source
        t1.addFilter("Topic7");
        t1.addFilter("Topic9");

        System.out.println("Creating custom template 2");
        SimpleTemplate t2 = new SimpleTemplate();
        t2.addFilter("Topic1");
        t2.setSource(this);

        // We only subscribe once per session
        if (reopenedSession()) {
            // Since the session is already open we only restore the
            // local subscription table
            t1.setScopeToLocal();
            t2.setScopeToLocal();
            subscribe(t1);
            subscribe(t2);
        } else {
            // Do only local subscriptions. Here if we have server filtered identifiers,
            // the callback information is lost for the client.

            String sret = subscribe(t1);
            if (sret == null) {
                System.out.println("Subscription failure 1: " + sret);
                System.exit(0);
            }
            sret = subscribe(t2);
            if (sret == null) {
                System.out.println("Subscription failure 2: " + sret);
                System.exit(0);
            }

        }
        // unsubscribe(t2);

        System.out.println("Waiting for notifications");

        for (; ; ) {
            try {
                Thread.sleep(1000);
            } catch (Exception e) {}
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Sub 1: Received notification: " + n.getType() + "," +
                           n.get());
    }

}


class SubListener implements Notifiable {
    int count = 0;
    EventBase _eb;

    public SubListener(EventBase eb) {
        _eb = eb;
    }

    public int getCount() {
        return count;
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Sub 2: Received notification " + (count) +
                           ": " + n.getType() + "," + n.get());
        count++;
    }

}
