/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.custom;

import fuegocore.message.*;
import fuegocore.message.protocol.*;
import fuegocore.message.service.*;
import fuegocore.notify.core.serialization.*;
import fuegocore.notify.core.server.*;
import fuegocore.notify.core.server.handlers.*;
import fuegocore.notify.test.custom.core.*;
import fuegocore.notify.test.custom.serialization.*;

/**
 * This class is an example of a custom routing core and
 * custom serialization. The server is a simple router without
 * support for distributed operation.
 */
public class StartCustomServer {
    static int current_port = 10000;
    static int admin_port = 22;
    static String server = null;

    public static void main(String[] args) {
        // Set access server identifier
        fuegocore.notify.profile.CommProfile.access_server_id =
                "localhost:10000";

        try {
            // Register custom serializer
            SimpleTemplateSerializer simser = new SimpleTemplateSerializer();
            EventSerializationService.getService().register(
                    "fuegocore.notify.test.custom.core.SimpleTemplate", simser);
            EventSerializationService.getService().register("SimpleTemplate",
                    simser);

            // Creater router
            SimpleRouter simple = new SimpleRouter();
            // Start server
            EventServer es = EventServer.create(simple);

            MessageTransportService mts = MessageTransportService.init();
            MessageService ms = mts.getService();
            System.out.println("Starting access server");
            AccessServerHandler ace = new AccessServerHandler(es);
            ms.registerReceiver("messagerpc", ace);
            ms.registerRpcReceiver("rpc", ace);
            ms.registerReceiver("message", es);

            ms.listen(current_port,
                      Url.Protocol.parse(fuegocore.notify.profile.UserProfile.
                                         DEF_MSG_PROTOCOL));

        } catch (Exception ex) {
            ex.printStackTrace(System.err);
        }

        fuegocore.notify.comms.HTTPServlet.PORT = admin_port; // set http port
        fuegocore.notify.comms.HTTPServlet.startHttpServlet();
    }
}
