/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import fuegocore.notify.core.*;

/**
 * Runs a test event producer
 */
public class EventProducerTest extends EventBase {

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("producer");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        EventProducerTest test = new EventProducerTest();
    }

    public EventProducerTest() {
        System.out.println("EventProducerTest\n");

        // periodic publication of events
        int i = 0;
        for (; ; ) {
            try {
                System.out.println("Producing presence event " + i);
                Notification n = new Notification();
                n.setType("presence");
                n.add(new EventTriplet("number", i++));
                n.add(new EventTriplet("data", "text data"));
                // Activate the replace mode: the newest notification overrides older ones
                // in session queues.
                //n.setReplaceId("number123");
                //n.setReplaceActive(true);

                System.out.println("Publishing event");
                publish(n);
                Thread.sleep(2000);
            } catch (Exception e) {
                System.out.println("Exception: " + e);
            }
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
