/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import java.text.*;
import java.util.*;

import fuegocore.notify.core.*;

/**
 * This class demonstrates tuple extraction from notifications
 */
public class NotificationExtractionTest extends EventBase {
    public NotificationExtractionTest() {

        Notification n = new Notification();
        n.setType("presence");
        n.add(new EventTriplet("number", 77));
        n.add(new EventTriplet("test", "This is a test"));
        Date test = null;

        try {
            test = DateFormat.getDateTimeInstance().parse("3.7.2002 22:00:00");
        } catch (Exception e) {
            e.printStackTrace();
        }
        n.add(new EventTriplet("date", test));
        n.add(new EventTriplet("flag", true));

        // Retrieve the tuple value
        EventTriplet et = (EventTriplet) n.getField(EventTriplet.formatId(
                "number",
                EventTriplet.TYPE_INT));
        if (et == null) {
            System.out.println("Extract number failed: null value");
        }
        if (et != null) {
            int number = ((Integer) et.getValue()).intValue();
            if (number != 77) {
                System.out.println("Extract number failed: " + number +
                                   " != 77");
            } else {
                System.out.println("Extract number ok");
            }
        }
        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId("test",
                EventTriplet.TYPE_STRING));
        if (et == null) {
            System.out.println("Extract string failed: null value");
        }
        if (et != null) {
            if (!et.getValue().equals("This is a test")) {
                System.out.println("Extract string failed: " + et.getValue() +
                                   " This is a test");
            } else {
                System.out.println("Extract string ok");
            }
        }

        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId("date",
                EventTriplet.TYPE_DATE));
        if (et == null) {
            System.out.println("Extract date failed: null value");
        }
        if (et != null) {
            Date d = (Date) et.getValue();
            if (!d.equals(test)) {
                System.out.println("Extract date failed: " + d);
            } else {
                System.out.println("Extract date ok");
            }
        }

        // Retrieve the tuple value
        et = (EventTriplet) n.getField(EventTriplet.formatId("flag",
                EventTriplet.TYPE_BOOLEAN));
        if (et == null) {
            System.out.println("Extract boolean failed: null value");
        }
        if (et != null) {
            Boolean b = (Boolean) et.getValue();
            if (!b.booleanValue()) {
                System.out.println("Extract boolean failed: false != true");
            } else {
                System.out.println("Extract boolean ok");
            }
        }

    }

    public static void main(String[] args) {
        NotificationExtractionTest notificationExtractionTest1 = new
                NotificationExtractionTest();
    }

}
