/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;

/**
 * Runs a test event subscriber.
 */

public class PresenceSubTest extends EventBase {
    int counter = 0;
    static int timeout = 10000;
    static String sub = "";
    public static void main(String[] args) {
        // Create a random name
        sub = "sub" + (int) (Math.random() * 100000);
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName(sub);
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        PresenceSubTest test = new PresenceSubTest();
        System.out.println("PresenceSubTest\n");
    }

    public PresenceSubTest() {
        // Open session
        String sid = open();

        System.out.println("Creating template 1");

        Template t3 = new Template();
        // Add attribute filters to the template
        Filter f3 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ presence");
        Filter f4 = new Filter("number", EventTriplet.TYPE_INT,
                               "gte 10");

        // Define a listener class
        SubListenerTest cl = new SubListenerTest(this);
        t3.setSubId(sub + "_1");
        t3.setSource(cl); // set the source
        t3.addFilter(f3);
        t3.addFilter(f4);

        System.out.println("Creating template 2");
        Template t4 = new Template();
        // add a filter to the template
        Filter f5 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ mychannel");

        t4.addFilter(f5);
        t4.setSubId(sub + "_2");
        //t4.addFilter(f6);
        // Set the fields that we need
        //t4.setNotifyFields("number");

        // We only subscribe once per session
        if (reopenedSession()) {
            // Since the session is already open we only restore the
            // local subscription table
            t3.setScopeToLocal();
            t4.setScopeToLocal();
            subscribe(t3);
            subscribe(t4);
        } else {
            // Do only local subscriptions. Here if we have server filtered identifiers,
            // the callback information is lost for the client.
            if (subscribe(t3) == null) {
                System.out.println("Subscription failure");
                System.exit(0);
            }
            if (subscribe(t4) == null) {
                System.out.println("Subscription failure");
                System.exit(0);
            }
        }

        System.out.println("Waiting for notifications");
        System.out.println("Timeout:" + timeout);
        try {
            Thread.sleep(timeout);
        } catch (Exception e) {}
        System.out.println("Closing session");
        close();
        System.out.println("End");
    }

    public void notifyIncoming(Notification n) {
        /*
            if (counter++ > 30) {
              System.out.println("Closing session");
              String ret = closeSession();
              System.out.println("Close session returned: " + ret);
              System.exit(0);
            }
         */
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}


class SubListenerTest implements Notifiable {
    int count = 0;
    EventBase _eb;

    public SubListenerTest(EventBase eb) {
        _eb = eb;
    }

    public int getCount() {
        return count;
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Sub 2: Received notification " + (count) +
                           ": " + n.getType() + "," + n.get());
        count++;
    }

}
