/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;
import fuegocore.notify.core.session.*;

/**
 * Demonstrates notification pulling from the server.
 */

public class PullClient extends EventBase {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("pull");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        PullClient test = new PullClient();
        System.out.println("PullClient\n");
    }

    public PullClient() {
        String subID1 = "sub1", subID2 = "sub2";

        SessionParams sp = new SessionParams();
        sp.setMaxQueueSize(50);
        sp.setPause(true); // pause mode active

        // Open session
        String sid = open(sp);

        System.out.println("Creating template 1");

        Template t3 = new Template();
        // Add attribute filters to the template
        Filter f3 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ presence");
        Filter f4 = new Filter("number", EventTriplet.TYPE_INT,
                               "lte 10");
        t3.setSubId(subID1);

        // Define a listener class
        PullListener cl = new PullListener(this);
        t3.setSource(cl); // set the source
        t3.addFilter(f3);
        t3.addFilter(f4);

        System.out.println("Creating template 2");
        Template t4 = new Template();
        t4.setSubId(subID2);
        // add a filter to the template
        Filter f5 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ presence");
//    Filter f6 = new Filter("number", EventTriplet.TYPE_INT,
//                           "gte 1");
        t4.addFilter(f5);

        // We only subscribe once per session
        if (reopenedSession()) {
            // Since the session is already open we only restore the
            // local subscription table
            t3.setScopeToLocal();
            t4.setScopeToLocal();
        }

        // Do only local subscriptions. Here if we have server filtered identifiers,
        // the callback information is lost for the client.
        subscribe(t3);
        subscribe(t4);

        System.out.println("Waiting for notifications");

//    for (; ; ) {
        try {
            Thread.sleep(2000);
            System.out.println("Pulling");
            Vector v = pullEvents(subID1, 0, false);
            if (v != null) {
                System.out.println("Retrieved: " + v.size() +
                                   " events without destructive receipt");
            } else {
                System.out.println("Did not retrieve any notifications");

            }
            System.out.println("Pulling again using local dissemination");
            int i = pullAndDeliver(subID1 + "," + subID2, 0, true);
            System.out.println("Pulled and delivered " + i + " events");
        } catch (Exception e) {}

        close();
        System.exit(0);
//    }
    }

    public void notifyIncoming(Notification n) {
        /*
           if (counter++ > 30) {
             System.out.println("Closing session");
             String ret = closeSession();
             System.out.println("Close session returned: " + ret);
             System.exit(0);
           }
         */
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}


class PullListener implements Notifiable {
    int count = 0;
    EventBase _eb;

    public PullListener(EventBase eb) {
        _eb = eb;
    }

    public int getCount() {
        return count;
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Sub 2: Received notification " + (count) +
                           ": " + n.getType() + "," + n.get());
        count++;
    }

}
