/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;

/**
 * Runs a test event subscriber.
 */

public class SubscriberMulti extends EventBase {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("democlient");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        SubscriberMulti test = new SubscriberMulti();
        System.out.println("SubscriberMulti\n");
    }

    public SubscriberMulti() {
        // Open session
        String sid = open();

        for (int i = 0; i < 10; i++) {
            Template t3 = new Template();
            // add a filter to the template
            Filter f3 = new Filter("type", EventTriplet.TYPE_STRING,
                                   "equ channel" + i);
            t3.addFilter(f3);

            if (subscribe(t3) == null) {
                System.out.println("Subscribe failure");
                System.exit(0);
            }
        }

        System.out.println("Waiting for notifications");

        for (; ; ) {
            try {
                Thread.sleep(1000);
            } catch (Exception e) {}
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
