/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import fuegocore.notify.core.*;
import fuegocore.notify.core.session.*;
import fuegocore.notify.core.filter.*;

/**
 * Runs a test event subscriber.
 */

public class SubscriberTest extends EventBase {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("democlient");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        SubscriberTest test = new SubscriberTest();
        System.out.println("SubscriberTest\n");
    }

    public SubscriberTest() {
        // Open session
      SessionParams param = new SessionParams();
      param.blockedPull = true;
      param.pause = true;
      String sid = open(param);

        if (sid == null) {
            System.out.println("Session open failed");
        } else {
            System.out.println("Session opened. Id: " + sid);

        }
        System.out.println("Creating template 1");

        Template t3 = new Template();
        // Add attribute filters to the template
        Filter f3 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ presence");
        Filter f4 = new Filter("number", EventTriplet.TYPE_INT,
                               "lte 10");

        // Define a listener class
        SubListener cl = new SubListener(this);
        t3.setSource(cl); // set the source
        t3.addFilter(f3);
        t3.addFilter(f4);

        System.out.println("Creating template 2");
        Template t4 = new Template();
        // add a filter to the template
        Filter f5 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ presence");
        Filter f6 = new Filter("number", EventTriplet.TYPE_INT,
                               "gte 1");
        t4.addFilter(f5);
        t4.addFilter(f6);
        t4.setSource(this);

        // We only subscribe once per session
        if (reopenedSession()) {
            // Since the session is already open we only restore the
            // local subscription table
            t3.setScopeToLocal();
            t4.setScopeToLocal();
            subscribe(t3);
            subscribe(t4);
        } else {
            String sret = subscribe(t3);
            if (sret == null) {
                System.out.println("Subscription failure 1: " + sret);
                System.exit(0);
            }

            sret = subscribe(t4);
            if (sret == null) {
                System.out.println("Subscription failure 2: " + sret);
                System.exit(0);
            }
        }
        // Remove the first one (test)
//        unsubscribe(t4);

        System.out.println("Waiting for notifications");

        for (; ; ) {
            try {
                Thread.sleep(1000);
            } catch (Exception e) {}
        }
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Sub 1: Received notification: " + n.getType() + "," +
                           n.get());
    }

}


class SubListener implements Notifiable {
    int count = 0;
    EventBase _eb;

    public SubListener(EventBase eb) {
        _eb = eb;
    }

    public int getCount() {
        return count;
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Sub 2: Received notification " + (count) +
                           ": " + n.getType() + "," + n.get());
        count++;
    }

}
