/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import fuegocore.notify.core.*;
import fuegocore.notify.core.filter.*;

/**
 * Test subscription covering at the event channel.
 */

public class SubscriptionCoverTest extends EventBase {
    int counter = 0;

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName("democlient");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
        SubscriptionCoverTest test = new SubscriptionCoverTest();
        System.out.println("SubscriptionCoverTest\n");
    }

    public SubscriptionCoverTest() {
        // Open session
        String sid = open();

        // Subscribing a template that covers the first batch
        fuegocore.notify.profile.UserProfile.subCover = true;

        Template t0 = new Template();

        // Add attribute filters to the template

        Filter f1 = new Filter("type", EventTriplet.TYPE_STRING,
                               "equ presence");
        Filter f2 = new Filter("number1", EventTriplet.TYPE_INT,
                               "gte 0 & lte 1000");
        Filter f3 = new Filter("number2", EventTriplet.TYPE_INT,
                               "gte 50 & lte 1500");

        t0.setSource(this); // set the source
        t0.addFilter(f1);
        t0.addFilter(f2);
        t0.addFilter(f3);

        String coveringSub = subscribe(t0);
        if (coveringSub == null) {
            System.out.println("Subscription ID was null");
            System.exit(0);
        }
        System.out.println("Subcribing covered in 1 sec");
        try {
            Thread.sleep(1000);
        } catch (Exception e) {}

        String subids[] = new String[40];
        for (int i = 0; i < 40; i++) {
            System.out.println("Creating template: " + i);

            Template t1 = new Template();
            // Add attribute filters to the template
            f1 = new Filter("type", EventTriplet.TYPE_STRING,
                            "equ presence");
            f2 = new Filter("number1", EventTriplet.TYPE_INT,
                            "gte " + (100 - i) + " & lte 1000");
            f3 = new Filter("number2", EventTriplet.TYPE_INT,
                            "gte " + (100 - i + 50) + " & lte 999");
            t1.setSource(this); // set the source
            t1.addFilter(f1);
            t1.addFilter(f2);
            t1.addFilter(f3);

            subids[i] = subscribe(t1);
        }

        try {
            Thread.sleep(2000);
        } catch (Exception e) {}

        for (int i = 0; i < 40; i++) {
            System.out.println("Unsubscribing:" + unsubscribe(subids[i]));
        }

        // Unsubscribe all
        //System.out.println("Reset session: " + resetSession());

        try {
            Thread.sleep(2000);
        } catch (Exception e) {}
        System.out.println("Resubscribe cover");
        for (int i = 0; i < 40; i++) {
            System.out.println("Creating template: " + i);
            Template t1 = new Template();
            // Add attribute filters to the template
            f1 = new Filter("type", EventTriplet.TYPE_STRING,
                            "equ presence");
            f2 = new Filter("number1", EventTriplet.TYPE_INT,
                            "gte " + i + " & lte 1000");
            f3 = new Filter("number2", EventTriplet.TYPE_INT,
                            "gte " + (i + 50) + " & lte 999");
            t1.setSource(this); // set the source
            t1.addFilter(f1);
            t1.addFilter(f2);
            t1.addFilter(f3);
            String id = subscribe(t1);
            //System.out.println("Unsubscribing:" + unsubscribe(id));
        }
        // Unsubscribe all
        System.out.println("Reset session: " + reset());
        try {
            Thread.sleep(2000);
        } catch (Exception e) {}

        System.out.println("Resubscribe no cover");
        for (int i = 0; i < 40; i++) {
            System.out.println("Creating template: " + i);
            Template t1 = new Template();
            // Add attribute filters to the template
            f1 = new Filter("type", EventTriplet.TYPE_STRING,
                            "equ presence");
            f2 = new Filter("number1", EventTriplet.TYPE_INT,
                            "gte " + (100 - i) + " & lte 1000");
            f3 = new Filter("number2", EventTriplet.TYPE_INT,
                            "gte " + (100 - i + 50) + " & lte 999");
            t1.setSource(this); // set the source
            t1.addFilter(f1);
            t1.addFilter(f2);
            t1.addFilter(f3);
            String id = subscribe(t1);
        }
//        System.out.println("Unsubscribing:" + unsubscribe(coveringSub));

        System.out.println("Done");
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
