/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import fuegocore.notify.core.*;
import fuegocore.notify.core.channel.*;
import fuegocore.notify.core.filter.*;

/**
 * Creates a temporary event channel, subscribes event and
 * publishes event, then removes the channel.
 */
public class TemporaryChannelTest extends EventBase {

    public static void main(String[] args) {
        // set server address
        fuegocore.notify.core.client.EventConnection.get().setName(
                "channeltest");
        try {
            if (args != null && args.length > 0 && args[0] != null) {
                fuegocore.notify.core.client.EventConnection.get().setHostPort(
                        args[0],
                        Integer.parseInt(args[1]));
            } else {
                System.out.println("Incorrect arguments: host port");
                System.exit(0);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        TemporaryChannelTest test = new TemporaryChannelTest();
    }

    public TemporaryChannelTest() {
        System.out.println("TemporaryChannelTest\n");

        System.out.println("Open session");
        open();

        EventChannelParams epc = new EventChannelParams();
        if (!createChannel("mychannel", epc)) {
            System.out.println("Event channel could not be created");
        }

        // We subscribe the entire channel
        Template t = new Template();
        Filter f = new Filter("type", EventTriplet.TYPE_STRING,
                              "equ mychannel");
        t.addFilter(f);
        t.setLocalSend(true); // allow local send
        if (subscribe(t) == null) {
            System.out.println("Subscribe of mychannel failed");

            // periodic publication of events
        }
        System.out.println("Producing event");
        for (int k = 0; k < 2; k++) {
            Notification n = new Notification();
            n.setType("mychannel");
            n.add(new EventTriplet("number", 10));
            n.add(new EventTriplet("data", "text data"));

            System.out.println("Publishing event");
            publish(n);

            // wait a bit
            try {
                Thread.sleep(2000);
            } catch (Exception e) {}
        }

        System.out.println("Removing mychannel");
        removeChannel("mychannel");
        System.out.println("Trying publish, this should fail");

        Notification n2 = new Notification();
        n2.setType("mychannel");
        n2.add(new EventTriplet("number", 10));
        n2.add(new EventTriplet("data", "text data"));

        if (!publish(n2)) {
            System.out.println("Publish failed as expected");
        } else {
            System.out.println("Publish ok, should not happen!");
        }
        System.out.println("Closing session");
        close();
    }

    public void notifyIncoming(Notification n) {
        System.out.println("Client received notification: " + n.getType() + "," +
                           n.get());
    }

}
