/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.test.pubsub;

import java.util.*;

import fuegocore.notify.core.*;
import fuegocore.util.xas.*;

/**
 * This class demonstrates tuple extraction from notifications
 */
public class TripletOrderTest extends EventBase {
    public TripletOrderTest() {

        Notification n = new Notification();
        n.setType("presence");
        for (int i = 0; i < 10; i++) {
            EventTriplet et = new EventTriplet("number" + i, i);
            n.add(et);
        }

        System.out.println("Before parse");

        Vector contents = n.get().getVector();
        for (int i = 0; i < contents.size(); i++) {
            EventTriplet et = (EventTriplet) contents.elementAt(i);
            System.out.println(et);
        }
        System.out.println("---------------------------");
        System.out.println("After parse (clone)");
        Notification n2 = new Notification(n);
        contents = n2.get().getVector();
        for (int i = 0; i < contents.size(); i++) {
            EventTriplet et = (EventTriplet) contents.elementAt(i);
            System.out.println(et);
        }

        System.out.println("---------------------------");
        System.out.println("After parse (string)");
        n2 = new Notification(n.toString());
        contents = n2.get().getVector();
        for (int i = 0; i < contents.size(); i++) {
            EventTriplet et = (EventTriplet) contents.elementAt(i);
            System.out.println(et);
        }

        try {
            System.out.println("---------------------------");
            System.out.println("After parse (EventSequence)");
            EventSerializer evs = new EventSerializer();
            n.toEventSequence(evs);
            n2 = new Notification(evs.getCurrentSequence());
            contents = n2.get().getVector();
            for (int i = 0; i < contents.size(); i++) {
                EventTriplet et = (EventTriplet) contents.elementAt(i);
                System.out.println(et);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void main(String[] args) {
        TripletOrderTest TripletOrderTest1 = new
                                             TripletOrderTest();
    }

}
