/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.util;

import java.util.*;

/**
 * Auxiliary methods.
 */

public class Auxil {

    /** Extract element contents.
     * @param s the XML string
     * @param k the tag to look for
     * @return String the contents of the element
     **/
    public static String getTag(String s, String k) {
        if (s == null || k == null) {
            return null;
        }

        if (s.indexOf("<" + k + ">") == -1 ||
            s.indexOf("</" + k + ">") == -1) {
            return null;
        }

        int i = s.indexOf("<" + k + ">");
        int j = s.indexOf("</" + k + ">", i + 1);
        if (i == 1 || j == -1 || i >= j) {
            return null;
        }
        String val = s.substring(i + ("<" + k + ">").length(), j);

        //System.out.println("AUXIL Returning: " + val);
        return val;
    }

    /** Extract value after a keyword.
     * @param s the content string
     * @param k the keyword to look for
     * @return String the value of the keyword
     **/
    public static String getValue(String s, String k) {
        if (s == null || k == null) {
            return null;
        }

        int i = s.indexOf(k);
        int j = s.length();
        if (i == -1) {
            return null;
        }
        String val = s.substring(i + k.length(), j);
        return val;
    }

    /* Returns the host part of the given string */
    public static String getHost(String s) {
        if (s == null) {
            return null;
        }
        int i = s.indexOf(":");
        if (i > -1) {
            return s.substring(0, i);
        }
        return s;
    }

    /**
     * Returns the port part of the given string
     * @param s String
     * @return int
     */
    public static int getPort(String s) {
        if (s == null) {
            return -1;
        }
        try {
            int i = s.indexOf(":");
            if (i > -1) {
                Integer j = new Integer(s.substring(i + 1, s.length()));
                return j.intValue();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return -1;
    }

    /**
     *  Helper method for parsing dates mm.dd.yyyy
     * @param s String
     * @return Date
     */
    public static synchronized Date parseDate(String s) {
        Date ret = null;

        try {
            StringTokenizer st = new StringTokenizer(s, "./");
            Calendar cal = Calendar.getInstance();
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.HOUR, 0);

            if (st.hasMoreTokens()) {
                cal.set(Calendar.DATE,
                        (int) (new Integer(st.nextToken())).intValue());
            }
            if (st.hasMoreTokens()) {
                cal.set(Calendar.MONTH,
                        (int) (new Integer(st.nextToken())).intValue() - 1);
            }
            if (st.hasMoreTokens()) {
                cal.set(Calendar.YEAR,
                        (int) (new Integer(st.nextToken())).intValue());
            }
            ret = cal.getTime();
        } catch (Exception e) {
            e.printStackTrace();
            ret = null;
        }
        return ret;
    }

}
