/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.util;

import java.io.*;
import java.util.*;

/**
 * Basic logging class
 */
public class Log {
    static String datex = null;
    static SaveFile sf = new SaveFile("log.txt");

    public static void setFile(String s) {
        sf.close();
        sf = new SaveFile(s);
    }

    /**
     * Filter specific log
     * @param s String
     */
    public static void writeFilter(String s) {
        if (!fuegocore.notify.profile.UserProfile.debugFilter) {
            return; // log not active
        }
        Calendar cal = Calendar.getInstance();
        String date = "" + cal.get(Calendar.YEAR);
        date += "." + (cal.get(Calendar.MONTH) + 1);
        date += "." + cal.get(Calendar.DATE);
        sf.write("Filter Log [" + date + "]: " + s);
        System.out.println("Filter Log [" + date + "]: " + s);
    }

    /**
     * FilterService specific log
     * @param s String
     */
    public static void writeFilterService(String s) {
        if (!fuegocore.notify.profile.UserProfile.debugFilterService) {
            return; // log not active
        }
        Calendar cal = Calendar.getInstance();
        String date = "" + cal.get(Calendar.YEAR);
        date += "." + (cal.get(Calendar.MONTH) + 1);
        date += "." + cal.get(Calendar.DATE);
        sf.write("FS [" + date + "]: " + s);
        System.out.println("FS Log [" + date + "]: " + s);
    }

    /**
     * Transport specific log
     * @param s String
     */
    public static void writeTransport(String s) {
        if (!fuegocore.notify.profile.UserProfile.debugTransport) {
            return; // log not active
        }
        Calendar cal = Calendar.getInstance();
        String date = "" + cal.get(Calendar.YEAR);
        date += "." + (cal.get(Calendar.MONTH) + 1);
        date += "." + cal.get(Calendar.DATE);
        sf.write("Transport Log [" + date + "]: " + s);
        System.out.println("Transport Log [" + date + "]: " + s);
    }

    /**
     * General log
     * @param s String
     */
    public static void write(String s) {
        if (!fuegocore.notify.profile.UserProfile.debugGeneral) {
            return; // log not active
        }
        Calendar cal = Calendar.getInstance();
        String date = "" + cal.get(Calendar.YEAR);
        date += "." + (cal.get(Calendar.MONTH) + 1);
        date += "." + cal.get(Calendar.DATE);
        sf.write("Log [" + date + "]: " + s);
        System.out.println("Log [" + date + "]: " + s);
    }

    /**
     *  Event Channel Server log
     * @param s String
     */
    public static void writeEC(String s) {
        if (!fuegocore.notify.profile.UserProfile.debugEC) {
            return; // log not active
        }

        Calendar cal = Calendar.getInstance();
        String date = "" + cal.get(Calendar.YEAR);
        date += "." + (cal.get(Calendar.MONTH) + 1);
        date += "." + cal.get(Calendar.DATE);
        sf.write("Log EC [" + date + "]: " + s);
        System.out.println("Log EC [" + date + "]: " + s);
    }

    /** Flush log */
    public static void flush() {
        sf.close();
    }

    /**
     * Get textual representation of the current date
     * @return String
     */
    public static String getDate() {
        if (datex == null) {
            Calendar cal = Calendar.getInstance();
            datex = "" + cal.get(Calendar.DATE) +
                    "-" + (cal.get(Calendar.MONTH) + 1) +
                    "-" + cal.get(Calendar.YEAR);
        }
        return datex;
    }
}


class SaveFile {
    PrintWriter fout;

    public SaveFile(String s) {
        try {
            fout = new PrintWriter(new FileWriter(s));
        } catch (Exception e) {
            Log.write("Error opening file for saving:" + s);
        }
    }

    public void write(String s) {
        fout.println(s);
    }

    public void close() {
        fout.close();
    }
}
