/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.util;

import java.util.*;

import fuegocore.notify.core.*;

/**
 * Implements a simple routing table based on a hashtable and vectors.
 */

public class RoutingTable {
    private Hashtable tbl = new Hashtable();

    public void insert(String obj, Object obj2) {
        if (obj == null || obj2 == null) {
            return;
        }

        Vector vec = (Vector) tbl.get(obj);
        if (vec == null) {
            vec = new Vector();
            tbl.put(obj, vec);
        }

        if (obj2 instanceof Template) {
            vec.addElement(obj2);
        } else {
            vec.addElement(obj2);
        }
    }

    public void insertUnique(String obj, Object obj2) {
        if (obj == null || obj2 == null) {
            return;
        }

        Vector vec = (Vector) tbl.get(obj);
        if (vec == null) {
            vec = new Vector();
            tbl.put(obj, vec);
        }
        vec.addElement(obj2);
    }

    public boolean remove(String obj, Object obj2) {
        if (tbl == null) {
            return false;
        }

        Vector vec = (Vector) tbl.get(obj);
        if (vec != null) {
            vec.remove(obj2);
            return true;
        }
        return false;
    }

    public boolean removeUsingContent(String obj, Object obj2) {
        if (tbl == null) {
            return false;
        }
        Vector vec = (Vector) tbl.get(obj);
        if (vec != null) {
            for (int i = 0; i < vec.size(); i++) {
                ITemplate t = (ITemplate) vec.elementAt(i);
                if (t.toString().equals(((ITemplate) obj2).toString())) {
                    vec.remove(t);
                    return true;
                }
            }
        }
        return false;
    }

    public int size() {
        if (tbl == null) {
            return 0;
        }
        return tbl.size();
    }

    public Vector get(String obj) {
        if (tbl == null || obj == null) {
            return null;
        }

        Vector vec = (Vector) tbl.get(obj);
        return vec;
    }

    public Enumeration getAll() {
        if (tbl == null) {
            return null;
        }
        return tbl.elements();
    }

    public Enumeration getKeys() {
        if (tbl == null) {
            return null;
        }
        return tbl.keys();
    }

    public void print() {
        int j = 0;
        fuegocore.notify.util.Log.write("Printing routing table");
        for (Enumeration e = tbl.elements(); e.hasMoreElements(); ) {
            Vector vec = (Vector) e.nextElement();

            fuegocore.notify.util.Log.write((j++) + " --------------");

            if (vec != null) {
                for (int i = 0; i < vec.size(); i++) {
                    Template ev = (Template) vec.elementAt(i);
                    fuegocore.notify.util.Log.write(ev.toString());
                }
            }

        }
    }
}
