/*
 * Copyright 2006 Helsinki Institute for Information Technology
 *
 * This file is a part of Fuego middleware.  Fuego middleware is free
 * software; you can redistribute it and/or modify it under the terms
 * of the MIT license, included as the file MIT-LICENSE in the Fuego
 * middleware source distribution.  If you did not receive the MIT
 * license with the distribution, write to the Fuego Core project at
 * fuego-core-users@hoslab.cs.helsinki.fi.
 */

package fuegocore.notify.util;

import java.util.*;

/**
 * Basic statistical methods such as mean and standard deviation.
 */

public class Stat {
    public static void main(String[] args) {
        testDouble();
        testLong();
    }

    public static double getMean(Vector series) {
        double sum = 0, n = 0;

        for (int i = 0; i < series.size(); i++) {
            if (series.elementAt(i) instanceof Long) {
                Long l = (Long) series.elementAt(i);
                if (l != null) {
                    sum += l.longValue();
                    n++;
                }
            } else if (series.elementAt(i) instanceof Double) {
                Double l = (Double) series.elementAt(i);
                if (l != null) {
                    sum += l.doubleValue();
                    n++;
                }
            }
        }
        return (sum / n);
    }

    /** test method for double */
    public static void testDouble() {
        Vector vec = new Vector();
        for (int i = 0; i < 20; i++) {
            double r = Math.random() * 100;
            vec.add(new Double(r));
            System.out.println(r);
        }
        System.out.println("Mean:" + getMean(vec) + " SD: " + getSD(vec));
    }

    /** test method for long */
    public static void testLong() {
        Vector vec = new Vector();
        for (int i = 0; i < 20; i++) {
            long r = (long) (Math.random() * 100);
            vec.add(new Long(r));
            System.out.println(r);
        }
        System.out.println("Mean:" + getMean(vec) + " SD: " + getSD(vec));
    }

    public static double getSD(Vector series) {
        double sum = 0, n = 0, total = 0, avg = 0;

        avg = getMean(series);

        for (int i = 0; i < series.size(); i++) {
            double l = 0;
            if (series.elementAt(i) instanceof Long) {
                l = (double) ((Long) series.elementAt(i)).doubleValue();
            } else if (series.elementAt(i) instanceof Double) {
                l = ((Double) series.elementAt(i)).doubleValue();

            }
            sum += l;
            total += Math.pow((l - avg), 2.0);
            n++;

        }

        double mean = sum / n;
        double sd = Math.sqrt(total / (n - 1));
        fuegocore.notify.util.Log.write("avg : " + avg + " mean " + mean +
                                        " sd " +
                                        sd);
        return (sd);

    }
}
